@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 """_util module with private helper functions
 
 API Notes:
@@ -12,66 +12,25 @@ from itertools import islice
 from typing import (
     Any, Dict, List, Tuple,
     Optional, Union, Generic, TypeVar, Type,
-    Iterable, Iterator, Generator, MutableSequence, Mapping, MutableMapping,
+    Iterable, Iterator, MutableSequence, Mapping, MutableMapping,
     cast, overload,
 )
-from .typing import SingleOrMulti, IntFloat
+from .typing import MultiT, cast_MultiT
 
 # Typevars used for Generic Collection classes
 _elem_typevar_ = TypeVar("_elem_typevar_")
 _index_typevar_ = TypeVar("_index_typevar_", bound=Hashable)
-_childelem_typevar_ = TypeVar("_childelem_typevar_")
 _child_class_ = TypeVar("_child_class_")
 _iter_typevar_ = TypeVar("_iter_typevar_")
 
 
-@overload
-def i2f(i: None) -> None:
-    ... # pragma: no cover
-@overload
-def i2f(i: IntFloat) -> float:
-    ... # pragma: no cover
-def i2f(i):
-    if type(i) == bool:
-        raise ValueError("Use of bool as float not allowed")
-    return i if i is None else float(i)
-
 def i2f_recursive(values: Any) -> Any:
     """Recursively convert int and bool elements of an iterable.
     Iterables will be converted to tuples"""
     if is_iterable(values):
         return tuple(i2f_recursive(v) for v in values)
     else:
-        return i2f(values)
-
-def v2t(
-    value: SingleOrMulti[_elem_typevar_].T, *, n: Optional[int]=None,
-) -> Tuple[_elem_typevar_, ...]:
-    """Convert a single value or an iterable of value to a tuple.
-
-    Arguments:
-        value: a single value or an iterable of value
-        n: length specification for return value
-
-    Returns:
-        * If n is not specified and value is single value, a tuple with the
-          value will be returned.
-        * If n specified and value is a single value, a tuple with
-          n times the value will be returns.
-        * If n is specified and value is an iterable, the length of
-          iterable will be checked to correspond with given length
-    """
-    if is_iterable(value) and (not isinstance(value, str)):
-        v = tuple(cast(Iterable[_elem_typevar_], value))
-        if n is not None:
-            assert n == len(v)
-        return v
-    else:
-        t = (cast(_elem_typevar_, value),)
-        if n is None:
-            return t
-        else:
-            return n*t
+        return float(values)
 
 def is_iterable(it: Any) -> bool:
     """Check if a value is Iterable"""
@@ -84,7 +43,7 @@ def is_iterable(it: Any) -> bool:
     else:
         return True
 
-def nth(it: Iterable[_elem_typevar_], n) -> _elem_typevar_:
+def get_nth_of(it: Iterable[_elem_typevar_], *, n: int) -> _elem_typevar_:
     """Return nth element from an iterable.
 
     Arguments:
@@ -97,7 +56,7 @@ def nth(it: Iterable[_elem_typevar_], n) -> _elem_typevar_:
     """
     return next(islice(it, n, None))
 
-def first(it: Iterable[_elem_typevar_]) -> _elem_typevar_:
+def get_first_of(it: Iterable[_elem_typevar_]) -> _elem_typevar_:
     """Get first element of an iterable
 
     This function will consume the first element of the iterator
@@ -105,9 +64,9 @@ def first(it: Iterable[_elem_typevar_]) -> _elem_typevar_:
     Raises:
         StopIteration: if iterable is empty
     """
-    return nth(it, 0)
+    return get_nth_of(it, n=0)
 
-def last(it: Iterable[_iter_typevar_]) -> _iter_typevar_:
+def get_last_of(it: Iterable[_iter_typevar_]) -> _iter_typevar_:
     """Get last elemeent from an iterator.
 
     The iterator will be exhausted after calling this function.
@@ -136,7 +95,7 @@ def strip_literal(s: str) -> str:
 class IterableOverride(Iterable[_iter_typevar_], Generic[_iter_typevar_]):
     """Mixin class to allow to override element with Iterable element with more
     specific element. This is for static typing support to have right element
-    for an iterator whose elements are a subclass of
+    for an iterator whose elements are a subclass of parent's element type.
 
     Example:
 
@@ -168,17 +127,9 @@ class IterTypeMixin(Iterable[_elem_typevar_], Generic[_elem_typevar_]):
 
     TODO: Extended internal API documentation.
     """
-    @overload
     def __iter_type__(self,
-        type_: Tuple[Type[_iter_typevar_], ...],
-    ) -> Generator[_iter_typevar_, None, None]:
-        ... # pragma: no cover
-    @overload
-    def __iter_type__(self,
-        type_: Type[_iter_typevar_],
-    ) -> Generator[_iter_typevar_, None, None]:
-        ... # pragma: no cover
-    def __iter_type__(self, type_):
+        type_: Union[Type[_iter_typevar_], Tuple[Type[_iter_typevar_], ...]],
+    ) -> Iterable[_iter_typevar_]:
         """Iterate over elems of an Iterable of certain type
 
         Arguments:
@@ -186,10 +137,10 @@ class IterTypeMixin(Iterable[_elem_typevar_], Generic[_elem_typevar_]):
         """
         for elem in self:
             if isinstance(elem, type_):
-                yield cast(_iter_typevar_, elem)
+                yield elem
 
 
-class TypedList(
+class ExtendedList(
     List[_elem_typevar_], IterTypeMixin[_elem_typevar_],
     Generic[_elem_typevar_],
 ):
@@ -202,22 +153,14 @@ class TypedList(
 
         self._frozen__: bool = False
 
-    @property
-    @abc.abstractmethod
-    def _elem_type_(self) -> Union[
-        Type[_childelem_typevar_],
-        Tuple[Type[_childelem_typevar_], ...],
-    ]:
-        ... # pragma: no cover
-
     def __add__(self,  # type: ignore[override]
         x: Union[_elem_typevar_, List[_elem_typevar_]],
-    ) -> "TypedList[_elem_typevar_]":
-        if isinstance(x, self._elem_type_):
-            ret = super().__add__(cast(List[_elem_typevar_], [x]))
-        else:
+    ) -> "ExtendedList[_elem_typevar_]":
+        if isinstance(x, list):
             ret = super().__add__(cast(List[_elem_typevar_], x))
-        return cast("TypedList[_elem_typevar_]", ret)
+        else:
+            ret = super().__add__(cast(List[_elem_typevar_], [x]))
+        return cast("ExtendedList[_elem_typevar_]", ret)
 
     def __delitem__(self, i: Union[int, slice]) -> None:
         if self._frozen_:
@@ -225,19 +168,18 @@ class TypedList(
         return super().__delitem__(i)
 
     def __iadd__(self: _child_class_,
-        x: SingleOrMulti[_elem_typevar_].T,
+        x: MultiT[_elem_typevar_],
     ) -> _child_class_:
-        cself = cast(TypedList[_elem_typevar_], self)
-        if isinstance(x, cself._elem_type_):
-            cself.append(cast(_elem_typevar_, x))
-        else:
-            cself.extend(cast(Iterable[_elem_typevar_], x))
+        cself = cast(ExtendedList[_elem_typevar_], self)
+        if cself._frozen_:
+            raise TypeError("Can't extend frozen list")
+        cself.extend(cast_MultiT(x))
         return self
 
-    def __imul__(self: "TypedList[_elem_typevar_]", n: int) -> "TypedList[_elem_typevar_]":
+    def __imul__(self: "ExtendedList[_elem_typevar_]", n: int) -> "ExtendedList[_elem_typevar_]":
         if self._frozen_:
             raise TypeError("Can't extend frozen list")
-        return cast("TypedList[_elem_typevar_]", super().__imul__(n))
+        return cast("ExtendedList[_elem_typevar_]", super().__imul__(n))
 
     def __setitem__(self, i: Union[int, slice], value) -> None:
         if self._frozen_:
@@ -291,7 +233,7 @@ class TypedList(
     def _frozen_(self) -> bool:
         return self._frozen__
 
-    def _reorder_(self, neworder: Iterable[int]) -> None:
+    def _reorder_(self, *, neworder: Iterable[int]) -> None:
         if self._frozen_:
             raise TypeError("Can't reorder a frozen list")
         neworder = tuple(neworder)
@@ -311,13 +253,13 @@ class TypedList(
 
     def __eq__(self, o: object) -> bool:
         return (
-            isinstance(o, TypedList)
+            isinstance(o, ExtendedList)
             and (len(self) == len(o))
             and all(self[i] == o[i] for i in range(len(self)))
         )
 
 
-class TypedListMapping(
+class ExtendedListMapping(
     MutableSequence[_elem_typevar_], MutableMapping[_index_typevar_, _elem_typevar_],
     IterTypeMixin[_elem_typevar_], Generic[_elem_typevar_, _index_typevar_],
 ):
@@ -327,31 +269,10 @@ class TypedListMapping(
     TODO: Extended internal API documentation.
 
     API Notes:
-        TypedListMapping assumes not isinstance(Iterable[_elem_typevar], _elem_typevar)
-        _elem_type_ has to be valid when _typedListMapping.__init__() is called.
+        ExtendedListMapping assumes not isinstance(Iterable[_elem_typevar], _elem_typevar)
     """
-    T = TypeVar("T")
-    class _List(TypedList):
-        def __init__(self,
-            iterable: Iterable["TypedListMapping.T"], parent: "TypedListMapping",
-        ):
-            super().__init__(iterable)
-            self._parent_ = parent
-
-        @property
-        def _elem_type_(self):
-            return self._parent_._elem_type_
-
-    def __init__(self, iterable: SingleOrMulti[_elem_typevar_].T=tuple()):
-        self._list_: TypedList[_elem_typevar_]
-        if isinstance(iterable, self._elem_type_):
-            self._list_ = self.__class__._List(
-                (cast(_elem_typevar_, iterable),), self,
-            )
-        else:
-            self._list_ = self.__class__._List(
-                cast(Iterable[_elem_typevar_], iterable), self,
-            )
+    def __init__(self, iterable: MultiT[_elem_typevar_]=tuple()):
+        self._list_ = ExtendedList[_elem_typevar_](cast_MultiT(iterable))
 
         attr_name = self._index_attribute_
         assert isinstance(attr_name, str)
@@ -363,19 +284,6 @@ class TypedListMapping(
             attr: _index_typevar_ = getattr(elem, attr_name)
             self._map_[attr] = elem
 
-    @property
-    @abc.abstractmethod
-    def _elem_type_(self) -> Union[
-        Type[_childelem_typevar_],
-        Tuple[Type[_childelem_typevar_], ...],
-    ]:
-        ... # pragma: no cover
-
-    @property
-    @abc.abstractmethod
-    def _index_type_(self) -> Type[_index_typevar_]:
-        ... # pragma: no cover
-
     @property
     @abc.abstractmethod
     def _index_attribute_(self) -> str:
@@ -390,7 +298,7 @@ class TypedListMapping(
     def __getitem__(self: _child_class_, # type: ignore[override]
         key: Union[int, slice, _index_typevar_],
     ) -> Union[_elem_typevar_, _child_class_]:
-        cself = cast(TypedListMapping[_elem_typevar_, _index_typevar_], self)
+        cself = cast(ExtendedListMapping[_elem_typevar_, _index_typevar_], self)
         if isinstance(key, int):
             return cself._list_[key]
         elif isinstance(key, slice):
@@ -398,12 +306,9 @@ class TypedListMapping(
                 cself._list_.__getitem__(key),
             )
             return cast(_child_class_, o)
-        elif isinstance(key, cself._index_type_):
+        else:
+            # type(key) is _index_typevar_
             return cself._map_[key]
-        raise TypeError(
-            f"idx has to be of type 'int', 'slice' or {cself._index_type_}, "
-            f"not {type(key)}"
-        )
 
     @overload
     def __setitem__(self,
@@ -415,7 +320,7 @@ class TypedListMapping(
         ... # pragma: no cover
     def __setitem__(self,
         key: Union[int, slice, _index_typevar_],
-        value: SingleOrMulti[_elem_typevar_].T,
+        value: MultiT[_elem_typevar_],
     ) -> None:
         if self._frozen_:
             raise TypeError("Can't change a frozen list")
@@ -426,21 +331,14 @@ class TypedListMapping(
             except: # pragma: no cover
                 # If the elem is in _list_ it should also be in _map_
                 raise RuntimeError("Internal error")
-            # Todo: can we avoid cast to please Pylance
-            if not isinstance(value, cast(Any, self._elem_type_)):
-                raise TypeError(
-                    f"value has to be of type '{self._elem_type_}', "
-                    f"not {type(value)}"
-                )
-            self._list_[key] = value
             key2 = getattr(value, self._index_attribute_)
+            self._list_[key] = cast(_elem_typevar_, value)
             self._map_[key2] = cast(_elem_typevar_, value)
         elif isinstance(key, slice): # pragma: no cover
             raise NotImplementedError(
-                "Assigning to slice of TypedListMapping"
+                "Assigning to slice of ExtendedListMapping"
             )
-        elif isinstance(key, self._index_type_):
-            assert isinstance(value, self._elem_type_)
+        else:
             value = cast(_elem_typevar_, value)
             for i, elem in enumerate(self._list_):
                 if getattr(elem, self._index_attribute_) == key:
@@ -461,15 +359,11 @@ class TypedListMapping(
             self._list_.__delitem__(key)
         elif isinstance(key, slice): # pragma: no cover
             raise NotImplementedError(
-                "Deketing slice of TypedListMapping"
+                "Deleting slice of ExtendedListMapping"
             )
-        elif isinstance(key, self._index_type_):
+        else:
             v = self._map_.pop(key)
             self._list_.remove(v)
-        else:
-            raise TypeError(
-                f"key has to be of type 'int', 'slice' or '{self._index_type_}'"
-            )
 
     def clear(self) -> None:
         if self._frozen_:
@@ -485,22 +379,21 @@ class TypedListMapping(
         if key is None:
             elem = self._list_.pop()
             self._map_.pop(getattr(elem, self._index_attribute_))
-        elif isinstance(key, self._index_type_):
-            elem = self._map_.pop(key)
-            self._list_.remove(elem)
-        else:
-            assert isinstance(key, int)
+        elif isinstance(key, int):
             elem = self._list_.pop(key)
             self._map_.pop(getattr(elem, self._index_attribute_))
+        else:
+            elem = self._map_.pop(key)
+            self._list_.remove(elem)
         return elem
 
     def popitem(self):
-        raise NotImplementedError("TypedListMapping.popitem()")
+        raise NotImplementedError("ExtendedListMapping.popitem()")
 
     def update(self, # type: ignore[override]
         __m: Mapping[_index_typevar_, _elem_typevar_]
     ) -> None:
-        raise NotImplementedError("TypedListMapping.update()")
+        raise NotImplementedError("ExtendedListMapping.update()")
 
     def __iter__(self) -> Iterator[_elem_typevar_]:
         return iter(self._list_)
@@ -534,16 +427,12 @@ class TypedListMapping(
         return self._map_.values()
 
     def __iadd__(self: _child_class_,
-        x: SingleOrMulti[_elem_typevar_].T,
+        x: MultiT[_elem_typevar_],
     ) -> _child_class_:
-        if cast("TypedListMapping", self)._frozen_:
+        if cast("ExtendedListMapping", self)._frozen_:
             raise TypeError("Can't change a frozen list")
-        cself = cast(TypedListMapping[_elem_typevar_, _index_typevar_], self)
-        new: Tuple[_elem_typevar_, ...]
-        if isinstance(x, cself._elem_type_):
-            new = (cast(_elem_typevar_, x),)
-        else:
-            new = tuple(cast(Iterable[_elem_typevar_], x))
+        cself = cast(ExtendedListMapping[_elem_typevar_, _index_typevar_], self)
+        new = cast_MultiT(x)
         cself._list_ += new
         for e in new:
             cself._map_[getattr(e, cself._index_attribute_)] = e
@@ -559,7 +448,7 @@ class TypedListMapping(
     def _reorder_(self, neworder: Iterable[int]) -> None:
         if self._frozen_:
             raise TypeError("Can't reorder a frozen list")
-        self._list_._reorder_(neworder)
+        self._list_._reorder_(neworder=neworder)
 
 
 class _ListMappingOverride(
@@ -593,7 +482,7 @@ class _ListMappingOverride(
         ... # pragma: no cover
     def __setitem__(self,
         key: Union[int, slice, _index_typevar_],
-        value: SingleOrMulti[_elem_typevar_].T,
+        value: MultiT[_elem_typevar_],
     ) -> None:
         return cast(Any, super()).__setitem__(key, value)
 
@@ -622,23 +511,19 @@ class _ListMappingOverride(
         return cast(Any, super()).insert(index, value)
 
     def __iadd__(self: _child_class_,
-        x: SingleOrMulti[_elem_typevar_].T,
+        x: MultiT[_elem_typevar_],
     ) -> _child_class_:
         return cast(Any, super()).__iadd__(x)
 
 
-class TypedListStrMapping(TypedListMapping[_elem_typevar_, str], Generic[_elem_typevar_]):
-    """TypeListMapping where the _index_type_ is `str`. By default this also take 'name'
+class ExtendedListStrMapping(ExtendedListMapping[_elem_typevar_, str], Generic[_elem_typevar_]):
+    """TypeListMapping where the index `type_` is `str`. By default this also take 'name'
     as default attribute name for the index. This can be overloaded in a subclass if
     needed.
 
     TODO: Extended internal API documentation.
     """
     @property
-    def _index_type_(self):
-        return str
-    # Set default attribute name to 'name'
-    @property
     def _index_attribute_(self):
         return "name"
 
