@@ -1,10 +1,10 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 
 """The dispatch module contains classes inspirec by the visitor pattern in order
 to allow to execute code specific to a certain object type but also allow to
 have common code for all subclasses of a common parent class.
 
-For each type of base class there is a sepearate *Dispatched class that are based
+For each type of base class there is a sepearate \*Dispatched class that are based
 on the same principle.
 If one wants to implement some code for a class, typically one subclasses the
 corresponding dispatcher class and then overloads one of the methods to implement
