@@ -1,5 +1,5 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
-from ..technology import edge as edg
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
+from ..technology import edge as _edg
 
 
 class EdgeDispatcher:
@@ -7,7 +7,7 @@ class EdgeDispatcher:
 
     This dispatcher follows the common way of working in the `dispatch` module.
     """
-    def __call__(self, edge: edg._Edge, *args, **kwargs):
+    def __call__(self, edge: _edg.EdgeT, *args, **kwargs):
         classname = edge.__class__.__name__.split(".")[-1]
         return getattr(self, classname, self._pd_unhandled)(edge, *args, **kwargs)
 
@@ -17,19 +17,19 @@ class EdgeDispatcher:
             f"{edge.__class__.__name__}"
         )
 
-    def _Edge(self, edge: edg._Edge, *args, **kwargs):
+    def _Edge(self, edge: _edg.EdgeT, *args, **kwargs):
         raise NotImplementedError(
             f"No dispatcher implemented for object of type {edge.__class__.__name__}"
         )
 
-    def _DualEdgeOperation(self, op: edg._DualEdgeOperation, *args, **kwargs):
+    def _DualEdgeOperation(self, op: _edg._DualEdgeOperation, *args, **kwargs):
         return self._Edge(op, *args, **kwargs)
 
-    def MaskEdge(self, edge: edg.MaskEdge, *args, **kwargs):
+    def MaskEdge(self, edge: _edg.MaskEdge, *args, **kwargs):
         return self._Edge(edge, *args, **kwargs)
 
-    def Join(self, join: edg.Join, *args, **kwargs):
+    def Join(self, join: _edg.Join, *args, **kwargs):
         return self._Edge(join, *args, **kwargs)
 
-    def Intersect(self, intersect: edg.Intersect, *args, **kwargs):
+    def Intersect(self, intersect: _edg.Intersect, *args, **kwargs):
         return self._Edge(intersect, *args, **kwargs)
