@@ -1,16 +1,16 @@
 """Generate coriolis setup file"""
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 from textwrap import dedent, indent
 from itertools import product
 from typing import Set, Tuple, Optional, cast, overload
 
-from ... import _util, dispatch as dsp
-from ...typing  import GDSLayerSpecDict, SingleOrMulti
+from ... import dispatch as _dsp
+from ...typing  import GDSLayerSpecDict
 from ...technology import (
-    mask as msk, primitive as prm, geometry as geo, technology_ as tch,
+    mask as _msk, primitive as _prm, geometry as _geo, technology_ as _tch,
 )
-from ...design import layout as lay, library as lbr
-
+from ...design import cell as _cell, routinggauge as _rg, library as _lbry
+from ...design.layout import layout_ as _laylay
 
 __all__ = ["FileExporter"]
 
@@ -38,7 +38,7 @@ def _str_create_basic(name, mat, *,
 
 
 def _str_create_via(via):
-    assert isinstance(via, prm.Via)
+    assert isinstance(via, _prm.Via)
 
     def _str_bottomtop(bottom, top):
         s_via = f"{bottom.name}_{via.name}_{top.name}"
@@ -51,14 +51,14 @@ def _str_create_via(via):
         """[1:])
 
     return "".join(_str_bottomtop(bottom, top) for bottom, top in product(
-        filter(lambda p: isinstance(p, prm.MetalWire), via.bottom),
-        filter(lambda p: isinstance(p, prm.MetalWire), via.top),
+        filter(lambda p: isinstance(p, _prm.MetalWire), via.bottom),
+        filter(lambda p: isinstance(p, _prm.MetalWire), via.top),
     ))
 
 
-def _args_gds_layer(prim: prm._MaskPrimitive, *, gds_layers: GDSLayerSpecDict):
+def _args_gds_layer(prim: _prm.MaskPrimitiveT, *, gds_layers: GDSLayerSpecDict):
     mask = prim.mask
-    if isinstance(mask, msk.DesignMask):
+    if isinstance(mask, _msk.DesignMask):
         gds_layer = gds_layers.get(mask.name, None)
         if gds_layer is not None:
             if not isinstance(gds_layer, tuple):
@@ -70,28 +70,28 @@ def _args_gds_layer(prim: prm._MaskPrimitive, *, gds_layers: GDSLayerSpecDict):
         return {}
 
 
-class _LayerGenerator(dsp.PrimitiveDispatcher):
-    def __init__(self, *, tech: tch.Technology, gds_layers: GDSLayerSpecDict):
+class _LayerGenerator(_dsp.PrimitiveDispatcher):
+    def __init__(self, *, tech: _tch.Technology, gds_layers: GDSLayerSpecDict):
         self.gds_layers = gds_layers
         # TODO: get the poly layers
         self.poly_layers = {
-            gate.poly for gate in tech.primitives.__iter_type__(prm.MOSFETGate)
+            gate.poly for gate in tech.primitives.__iter_type__(_prm.MOSFETGate)
         }
-        self.via_conns = via_conns = cast(Set[prm._Primitive], set())
-        for via in tech.primitives.__iter_type__(prm.Via):
+        self.via_conns = via_conns = cast(Set[_prm.PrimitiveT], set())
+        for via in tech.primitives.__iter_type__(_prm.Via):
             via_conns.update(via.bottom)
             via_conns.update(via.top)
         self.blockages = {prim.blockage for prim in filter(
             lambda p: p.blockage is not None,
-            tech.primitives.__iter_type__(prm._BlockageAttribute),
+            tech.primitives.__iter_type__(_prm.BlockageAttrPrimitiveT),
         )}
 
-    def _Primitive(self, prim: prm._Primitive):
+    def _Primitive(self, prim: _prm.PrimitiveT):
         raise NotImplementedError(
             f"layer code generation for '{prim.__class__.__name__}'"
         )
 
-    def Marker(self, prim: prm.Marker):
+    def Marker(self, prim: _prm.Marker):
         type_ = "blockage" if prim in self.blockages else "other"
         return _str_create_basic(prim.name, type_, **_args_gds_layer(
             prim, gds_layers=self.gds_layers,
@@ -102,43 +102,47 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             prim, gds_layers=self.gds_layers,
         ))
 
-    def ExtraProcess(self, prim: prm.ExtraProcess):
+    def ExtraProcess(self, prim: _prm.ExtraProcess):
         return _str_create_basic(prim.name, "other", **_args_gds_layer(
             prim, gds_layers=self.gds_layers,
         ))
 
-    def Implant(self, prim: prm.Implant):
+    def Implant(self, prim: _prm.Implant):
         return _str_create_basic(
             prim.name,
-            f"{prim.type_}Implant" if prim.type_ in ("n", "p") else "other",
+            (
+                f"{prim.type_.value}Implant"
+                if prim.type_ in (_prm.nImplT, _prm.pImplT)
+                else "other"
+            ),
             minsize=prim.min_width, minspace=prim.min_space,
             minarea=prim.min_area,
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
-    def Insulator(self, prim: prm.Insulator):
+    def Insulator(self, prim: _prm.Insulator):
         return _str_create_basic(prim.name, "other", **_args_gds_layer(
             prim, gds_layers=self.gds_layers,
         ))
 
-    def Well(self, prim: prm.Well):
+    def Well(self, prim: _prm.Well):
         return _str_create_basic(
-            prim.name, prim.type_+"Well",
+            prim.name, prim.type_.value+"Well",
             minsize=prim.min_width, minspace=prim.min_space,
             minarea=prim.min_area,
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
-    def DeepWell(self, prim: prm.DeepWell):
+    def DeepWell(self, prim: _prm.DeepWell):
         # Treat it as a regular well
         return _str_create_basic(
-            prim.name, prim.type_+"Well",
+            prim.name, prim.type_.value+"Well",
             minsize=prim.min_width, minspace=prim.min_space,
             minarea=prim.min_area,
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
-    def WaferWire(self, prim: prm.WaferWire):
+    def WaferWire(self, prim: _prm.WaferWire):
         return _str_create_basic(
             prim.name, "active",
             minsize=prim.min_width, minspace=prim.min_space,
@@ -146,7 +150,7 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
-    def GateWire(self, prim: prm.GateWire):
+    def GateWire(self, prim: _prm.GateWire):
         return _str_create_basic(
             prim.name, "poly",
             minsize=prim.min_width, minspace=prim.min_space,
@@ -154,7 +158,7 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
-    def MetalWire(self, prim: prm.MetalWire):
+    def MetalWire(self, prim: _prm.MetalWire):
         return _str_create_basic(
             prim.name, "metal",
             minsize=prim.min_width, minspace=prim.min_space,
@@ -162,7 +166,7 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
-    def Via(self, prim: prm.Via, *, via_layer: bool=False):
+    def Via(self, prim: _prm.Via, *, via_layer: bool=False):
         if via_layer:
             return _str_create_via(prim)
         else:
@@ -172,7 +176,7 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
                 **_args_gds_layer(prim, gds_layers=self.gds_layers),
             )
 
-    def PadOpening(self, prim: prm.PadOpening):
+    def PadOpening(self, prim: _prm.PadOpening):
         return _str_create_basic(
             prim.name, "cut",
             minsize=prim.min_width, minspace=prim.min_space,
@@ -180,7 +184,7 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
-    def Resistor(self, prim: prm.Resistor):
+    def Resistor(self, prim: _prm.Resistor):
         if len(prim.indicator) == 1:
             s_indicator = f"'{prim.indicator[0].name}'"
         else:
@@ -190,10 +194,10 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             f"{s_indicator})\n"
         )
 
-    def MIMCapacitor(self, prim: prm.MIMCapacitor):
+    def MIMCapacitor(self, prim: _prm.MIMCapacitor):
         return f"# MIMCAP '{prim.name}': '{prim.top}' over '{prim.bottom}'"
 
-    def Diode(self, prim: prm.Diode):
+    def Diode(self, prim: _prm.Diode):
         if len(prim.indicator) == 1:
             s_indicator = f"'{prim.indicator[0].name}'"
         else:
@@ -203,14 +207,14 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             f"{s_indicator})\n"
         )
 
-    def MOSFETGate(self, prim: prm.MOSFETGate):
+    def MOSFETGate(self, prim: _prm.MOSFETGate):
         s_oxide = f", '{prim.oxide.name}'" if prim.oxide is not None else ""
         return (
             f"# GateLayer.create(tech, '{prim.name}', '{prim.active.name}', "
             f"'{prim.poly.name}'{s_oxide})\n"
         )
 
-    def MOSFET(self, prim: prm.MOSFET):
+    def MOSFET(self, prim: _prm.MOSFET):
         impl_names = tuple(impl.name for impl in prim.implant)
         s_impl = f"'{impl_names[0]}'" if len(impl_names) == 1 else str(impl_names)
         s_well = f", '{prim.well.name}'" if prim.well is not None else ""
@@ -219,29 +223,30 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             f"{s_impl}{s_well})\n"
         )
 
-    def Bipolar(self, prim: prm.Bipolar):
+    def Bipolar(self, prim: _prm.Bipolar):
         return f"# Not implemented: Bipolar '{prim.name}'"
 
 
-class _AnalogGenerator(dsp.PrimitiveDispatcher):
+class _AnalogGenerator(_dsp.PrimitiveDispatcher):
     def __init__(self, tech):
         self.tech = tech
 
-    def _Primitive(self, prim: prm._Primitive):
+    def _Primitive(self, prim: _prm.PrimitiveT):
         raise NotImplementedError(
             f"analog code generation for '{prim.__class__.__name__}'"
         )
 
-    def _MaskPrimitive(self, prim: prm._MaskPrimitive):
+    def _DesignMaskPrimitive(self, prim: _prm.DesignMaskPrimitiveT):
         s = ""
         if prim.grid is not None:
             s += f"('grid', '{prim.name}', {prim.grid}, Length, ''),\n"
         return s
 
-    def _WidthSpacePrimitive(self, prim: prm._WidthSpacePrimitive):
+    def _WidthSpacePrimitive(self, prim: _prm.WidthSpacePrimitiveT):
         s = f"('minWidth', '{prim.name}', {prim.min_width}, Length, ''),\n"
         s += f"('minSpacing', '{prim.name}', {prim.min_space}, Length, ''),\n"
-        s += self._MaskPrimitive(prim)
+        if isinstance(prim, _prm.DesignMaskPrimitiveT):
+            s += self._DesignMaskPrimitive(prim)
         if prim.min_area is not None:
             s += f"('minArea', '{prim.name}', {prim.min_area}, Area, ''),\n"
         if prim.min_density is not None:
@@ -253,13 +258,16 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
     # primitives handled by base classes of hierarchy:
     # Marker, Auxiliary, ExtraProcess, Implant, Insulator, GateWire, (Top)MetalWire
 
-    def Well(self, prim: prm.Well):
+    def Base(self, prim: _prm.Base):
+        return ""
+
+    def Well(self, prim: _prm.Well):
         s = self._WidthSpacePrimitive(prim)
         if prim.min_space_samenet is not None:
             s += f"('minSpacingSameNet', '{prim.name}', {prim.min_space_samenet}, Length, ''),\n"
         return s
 
-    def WaferWire(self, prim: prm.WaferWire):
+    def WaferWire(self, prim: _prm.WaferWire):
         s = self._WidthSpacePrimitive(prim)
         for i in range(len(prim.well)):
             well = prim.well[i]
@@ -269,7 +277,7 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
                 " Length|Asymmetric, ''),\n"
             )
         if prim.min_substrate_enclosure is not None:
-            for well in self.tech.primitives.__iter_type__(prm.Well):
+            for well in self.tech.primitives.__iter_type__(_prm.Well):
                 s += (
                     f"('minSpacing', '{well.name}', '{prim.name}', "
                     f" {prim.min_substrate_enclosure.spec}, Length|Asymmetric, ''),\n"
@@ -280,8 +288,8 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
         )
         return s
 
-    def Via(self, prim: prm.Via):
-        s = self._MaskPrimitive(prim)
+    def Via(self, prim: _prm.Via):
+        s = self._DesignMaskPrimitive(prim)
         s += f"('minWidth', '{prim.name}', {prim.width}, Length, ''),\n"
         s += f"('maxWidth', '{prim.name}', {prim.width}, Length, ''),\n"
         s += f"('minSpacing', '{prim.name}', {prim.min_space}, Length, ''),\n"
@@ -301,7 +309,7 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
             )
         return s
 
-    def PadOpening(self, prim: prm.PadOpening):
+    def PadOpening(self, prim: _prm.PadOpening):
         s = self._WidthSpacePrimitive(prim)
         s += (
             f"('minEnclosure', '{prim.bottom.name}', '{prim.name}', "
@@ -309,8 +317,9 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
         )
         return s
 
-    def Resistor(self, prim: prm.Resistor):
-        s = self._WidthSpacePrimitive(prim)
+    def Resistor(self, prim: _prm.Resistor):
+        s = f"('minWidth', '{prim.name}', {prim.min_width}, Length, ''),\n"
+        s += f"('minSpacing', '{prim.name}', {prim.min_space}, Length, ''),\n"
         for i in range(len(prim.indicator)):
             ind = prim.indicator[i]
             enc = prim.min_indicator_extension[i]
@@ -321,8 +330,14 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
         s = indent(s, prefix="# ")
         return s
 
-    def Diode(self, prim: prm.Diode):
-        s = self._WidthSpacePrimitive(prim)
+    def MIMCapacitor(self, prim: _prm.Diode):
+        s = f"('minWidth', '{prim.name}', {prim.min_width}, Length, ''),\n"
+        s += "# TODO: MIMCapacitor rules\n"
+
+        return s
+
+    def Diode(self, prim: _prm.Diode):
+        s = f"('minWidth', '{prim.name}', {prim.min_width}, Length, ''),\n"
         for i in range(len(prim.indicator)):
             ind = prim.indicator[i]
             enc = prim.min_indicator_enclosure[i]
@@ -333,7 +348,7 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
         s = indent(s, prefix="# ")
         return s
 
-    def MOSFETGate(self, prim: prm.MOSFETGate):
+    def MOSFETGate(self, prim: _prm.MOSFETGate):
         s = ""
         if prim.min_l is not None:
             s += f"# ('minTransistorL', '{prim.name}', {prim.min_l}, Length, ''),\n"
@@ -361,7 +376,7 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
             )
         return s
 
-    def MOSFET(self, prim: prm.MOSFET):
+    def MOSFET(self, prim: _prm.MOSFET):
         s = ""
         if prim.min_l is not None:
             s += f"# ('minTransistorL', '{prim.name}', {prim.min_l}, Length, ''),\n"
@@ -396,10 +411,13 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
             )
         return s
 
-    def Bipolar(self, prim: prm.Bipolar):
+    def Bipolar(self, prim: _prm.Bipolar):
         return f"# Not implemented: Bipolar '{prim.name}'\n"
 
-    def Spacing(self, prim: prm.Spacing):
+    def MinWidth(self, prim: _prm.MinWidth):
+        return f"# ('minWidth', '{prim.prim.name}', '{prim.min_width}', Length, '')"
+
+    def Spacing(self, prim: _prm.Spacing):
         if prim.primitives2 is None:
             return "".join(
                 f"('minSpacing', '{prim1.name}', {prim.min_space}, Length, ''),\n"
@@ -412,42 +430,38 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
                 for prim1, prim2 in product(prim.primitives1, prim.primitives2)
             )
 
-    def Enclosure(self, prim: prm.Enclosure):
+    def Enclosure(self, prim: _prm.Enclosure):
         return (
             f"('minEnclosure', '{prim.by.name}', '{prim.prim.name}', {prim.min_enclosure.spec}, "
             "Length|Asymmetric, ''),\n"
         )
 
-    def NoOverlap(self, prim: prm.NoOverlap):
+    def NoOverlap(self, prim: _prm.NoOverlap):
         return f"# ('noOverlap', '{prim.prim1.name}', '{prim.prim2.name}'),\n"
 
 
 class _LibraryGenerator:
-    def __init__(self, *, tech: tch.Technology):
+    def __init__(self, *, tech: _tch.Technology):
         self.tech = tech
-        self.metals: Tuple[prm.MetalWire, ...] = tuple(filter(
-            lambda m: not isinstance(m, prm.MIMTop),
-            tech.primitives.__iter_type__(prm.MetalWire),
+        self.metals: Tuple[_prm.MetalWire, ...] = tuple(filter(
+            lambda m: not isinstance(m, _prm.MIMTop),
+            tech.primitives.__iter_type__(_prm.MetalWire),
         ))
-        self.vias: Tuple[prm.Via, ...] = tuple(tech.primitives.__iter_type__(prm.Via))
+        self.vias: Tuple[_prm.Via, ...] = tuple(tech.primitives.__iter_type__(_prm.Via))
         assert len(self.metals) == len(self.vias)
         self.pinmasks = pinmasks = {}
-        for prim in filter(
-            lambda p: p.pin is not None,
-            tech.primitives.__iter_type__(prm._PinAttribute),
-        ):
-            pinmasks.update({
-                p.mask: cast(prm._MaskPrimitive, prim).mask
-                for p in cast(Tuple[prm.Marker], prim.pin)
-            })
+        for prim in tech.primitives.__iter_type__(_prm.PinAttrPrimitiveT):
+            if prim.pin is not None:
+                assert isinstance(prim, _prm.DesignMaskPrimitiveT)
+                pinmasks[prim.pin.mask] = prim.mask
 
         self.metalsdir: Optional[Tuple[Optional[str], ...]] = None
 
-    def __call__(self, lib: lbr.Library, *, routinggauge: Optional[lbr.RoutingGauge]):
+    def __call__(self, lib: _lbry.Library, *, routinggauge: Optional[_rg.RoutingGauge]):
         def otherdir(dir_):
             return "vertical" if dir_ == "horizontal" else "horizontal"
 
-        if isinstance(lib, lbr.StdCellLibrary):
+        if isinstance(lib, _lbry.RoutingGaugeLibrary):
             assert routinggauge is None
             routinggauge = lib.routinggauge[0]
 
@@ -520,7 +534,7 @@ class _LibraryGenerator:
 
         """[1:])
 
-        if isinstance(lib, lbr.StdCellLibrary):
+        if isinstance(lib, _lbry.RoutingGaugeLibrary):
             s += indent(
                 "\n".join(
                     (self._s_gauge(lib), self._s_pnr(lib), self._s_plugins())
@@ -626,11 +640,11 @@ class _LibraryGenerator:
                 metal2 = self.metals[i+1]
                 via_name = f"{metal.name}_{via.name}_{metal2.name}"
                 if routedir == "horizontal":
-                    henc = max(enc.min(), 0.5*(mwidth_above - via.width))
-                    venc = max(enc.max(), 0.5*(mwidth - via.width))
+                    henc = max(enc.max(), 0.5*(mwidth_above - via.width))
+                    venc = max(enc.min(), 0.5*(mwidth - via.width))
                 else:
-                    henc = max(enc.max(), 0.5*(mwidth - via.width))
-                    venc = max(enc.min(), 0.5*(mwidth_above - via.width))
+                    henc = max(enc.min(), 0.5*(mwidth - via.width))
+                    venc = max(enc.max(), 0.5*(mwidth_above - via.width))
                 s += dedent(f"""
                     via = tech.getLayer('{via_name}')
                     setEnclosures(via, metal, (u({henc:.6}), u({venc:.6})))
@@ -663,7 +677,7 @@ class _LibraryGenerator:
         return s
 
     def _s_pnr(self, lib):
-        topmetal = tuple(self.tech.primitives.__iter_type__(prm.MetalWire))[-1]
+        topmetal = tuple(self.tech.primitives.__iter_type__(_prm.MetalWire))[-1]
         return dedent(f"""
             # Place & Route setup
             with CfgCache(priority=Cfg.Parameter.Priority.ConfigurationFile) as cfg:
@@ -784,7 +798,7 @@ class _LibraryGenerator:
 
         return s
 
-    def _s_cell(self, lib: lbr.Library, cell: lbr._Cell):
+    def _s_cell(self, lib: _lbry.Library, cell: _cell.Cell):
         try:
             s = dedent(f"""
                 cell = new_cells['{cell.name}']
@@ -796,8 +810,8 @@ class _LibraryGenerator:
                 bnd = layout.boundary
                 assert bnd is not None, f"Cell boundary needed for {cell.name}"
 
-                pls = tuple(layout.sublayouts.__iter_type__(lay.MaskShapesSubLayout))
-                def get_netname(sl: lay.MaskShapesSubLayout):
+                pls = tuple(layout._sublayouts.__iter_type__(_laylay._MaskShapesSubLayout))
+                def get_netname(sl: _laylay._MaskShapesSubLayout):
                     return "*" if sl.net is None else sl.net.name
 
                 netnames = {get_netname(sl) for sl in pls}
@@ -814,11 +828,6 @@ class _LibraryGenerator:
                     + "\n    }\n"
                 )
 
-                if lib.global_nets is not None:
-                    for net in netnames:
-                        if net in lib.global_nets:
-                            s += f"    nets['{net}'].setGlobal(True)\n"
-
                 for sl in pls:
                     s += indent(
                         f"net = nets['{get_netname(sl)}']\n" +
@@ -826,19 +835,19 @@ class _LibraryGenerator:
                         prefix="    ",
                     )
 
-                for sl in layout.sublayouts.__iter_type__(lay._InstanceSubLayout):
+                for sl in layout._sublayouts.__iter_type__(_laylay._InstanceSubLayout):
                     # Currently usage of af.getCell() may not work as intended when
                     # two libraries have a cell with the same name.
                     # TODO: support libraries with cells with same name properly
                     r = {
-                        geo.Rotation.R0: "ID",
-                        geo.Rotation.R90: "R1",
-                        geo.Rotation.R180: "R2",
-                        geo.Rotation.R270: "R3",
-                        geo.Rotation.MX: "MX",
-                        geo.Rotation.MX90: "XR",
-                        geo.Rotation.MY: "MY",
-                        geo.Rotation.MY90: "YR",
+                        _geo.Rotation.R0: "ID",
+                        _geo.Rotation.R90: "R1",
+                        _geo.Rotation.R180: "R2",
+                        _geo.Rotation.R270: "R3",
+                        _geo.Rotation.MX: "MX",
+                        _geo.Rotation.MX90: "XR",
+                        _geo.Rotation.MY: "MY",
+                        _geo.Rotation.MY90: "YR",
                     }[sl.rotation]
                     s += indent(
                         dedent(f"""
@@ -860,7 +869,7 @@ class _LibraryGenerator:
         except NotImplementedError:
             return f"# Export failed for cell '{cell.name}'"
 
-    def _s_shape(self, shape: geo.MaskShape) -> str:
+    def _s_shape(self, shape: _geo.MaskShape) -> str:
         metalmasks = tuple(metal.mask for metal in self.metals)
 
         mask = shape.mask
@@ -935,7 +944,7 @@ class _LibraryGenerator:
 
         return s
 
-    def _s_point(self, point: geo.Point):
+    def _s_point(self, point: _geo.Point):
         # TODO: put on grid
         x = round(point.x, 6)
         y = round(point.y, 6)
@@ -944,7 +953,7 @@ class _LibraryGenerator:
 
 
 class _TechnologyGenerator:
-    def __call__(self, *, tech: tch.Technology, gds_layers: GDSLayerSpecDict):
+    def __call__(self, *, tech: _tch.Technology, gds_layers: GDSLayerSpecDict):
         self.tech = tech
         self.gds_layers = gds_layers
 
@@ -994,7 +1003,7 @@ class _TechnologyGenerator:
         # Take smallest transistor length as lambda
         lambda_ = min(
             trans.computed.min_l
-            for trans in self.tech.primitives.__iter_type__(prm.MOSFET)
+            for trans in self.tech.primitives.__iter_type__(_prm.MOSFET)
         )
 
         assert (self.tech.grid % 1e-6) < 1e-9, "Unsupported grid"
@@ -1020,24 +1029,24 @@ class _TechnologyGenerator:
 
         s_prims = ""
         written_prims = set()
-        vias = tuple(self.tech.primitives.__iter_type__(prm.Via))
+        vias = tuple(self.tech.primitives.__iter_type__(_prm.Via))
 
         for prim in self.tech.primitives:
             # Some primitives are handled later or don't need to be handled
             if isinstance(prim, (
                 # Handled by Via
-                prm.WaferWire, prm.GateWire, prm.MetalWire,
+                _prm.WaferWire, _prm.GateWire, _prm.MetalWire,
                 # Handled later
-                prm.Resistor, prm.MIMCapacitor, prm.Diode,
-                prm.MOSFETGate, prm.MOSFET, prm.Bipolar,
+                _prm.Resistor, _prm.MIMCapacitor, _prm.Diode,
+                _prm.MOSFETGate, _prm.MOSFET, _prm.Bipolar,
                 # Not exported
-                prm._RulePrimitive,
+                _prm.Base, _prm.RulePrimitiveT,
             )):
                 continue
 
             # We have to make sure via layers are defined in between top and bottom
             # metal layers
-            if isinstance(prim, prm.Via):
+            if isinstance(prim, _prm.Via):
                 for prim2 in prim.bottom:
                     s_prims += gen(prim2)
                     written_prims.add(prim2)
@@ -1050,7 +1059,7 @@ class _TechnologyGenerator:
             written_prims.add(prim)
 
             # For top via also do the top layers
-            if isinstance(prim, prm.Via) and prim == vias[-1]:
+            if isinstance(prim, _prm.Via) and prim == vias[-1]:
                 for prim2 in prim.top:
                     s_prims += gen(prim2)
                     written_prims.add(prim2)
@@ -1072,11 +1081,11 @@ class _TechnologyGenerator:
         s_prims += "\n# Blockages\n"
         for prim in filter(
             lambda p: p.blockage is not None,
-            self.tech.primitives.__iter_type__(prm._BlockageAttribute),
+            self.tech.primitives.__iter_type__(_prm.BlockageAttrPrimitiveT),
         ):
             s_prims += dedent(f"""
                 tech.getLayer('{prim.name}').setBlockageLayer(
-                    tech.getLayer('{cast(prm.Marker, prim.blockage).name}')
+                    tech.getLayer('{cast(_prm.Marker, prim.blockage).name}')
                 )
             """[1:])
 
@@ -1093,25 +1102,25 @@ class _TechnologyGenerator:
             s_prims += _str_create_basic(name, mat)
 
         s_prims += "\n# Resistors\n"
-        for prim in self.tech.primitives.__iter_type__(prm.Resistor):
+        for prim in self.tech.primitives.__iter_type__(_prm.Resistor):
             assert prim not in written_prims
             s_prims += gen(prim)
             written_prims.add(prim)
 
         s_prims += "\n# Capacitors\n"
-        for prim in self.tech.primitives.__iter_type__(prm.MIMCapacitor):
+        for prim in self.tech.primitives.__iter_type__(_prm.MIMCapacitor):
             assert prim not in written_prims
             s_prims += gen(prim)
             written_prims.add(prim)
 
         s_prims += "\n# Transistors\n"
-        for prim in self.tech.primitives.__iter_type__((prm.MOSFETGate, prm.MOSFET)):
+        for prim in self.tech.primitives.__iter_type__((_prm.MOSFETGate, _prm.MOSFET)):
             assert prim not in written_prims
             s_prims += gen(prim)
             written_prims.add(prim)
 
         s_prims += "\n# Bipolars\n"
-        for prim in self.tech.primitives.__iter_type__(prm.Bipolar):
+        for prim in self.tech.primitives.__iter_type__(_prm.Bipolar):
             assert prim not in written_prims
             s_prims += gen(prim)
             written_prims.add(prim)
@@ -1170,38 +1179,37 @@ class _TechnologyGenerator:
         clrs = ("Blue", "Aqua", "LightPink", "Green", "Yellow", "Violet", "Red")
 
         s += "\n    # Active Layers.\n"
-        for prim in self.tech.primitives.__iter_type__(prm.Well):
-            rgb = "Tan" if prim.type_ == "n" else "LightYellow"
+        for prim in self.tech.primitives.__iter_type__(_prm.Well):
+            rgb = "Tan" if prim.type_ == _prm.nImplT else "LightYellow"
             s += (
                 f"    style.addDrawingStyle(group='Active Layers', name='{prim.name}'"
                 f", color=toRGB('{rgb}'), pattern=toHexa('urgo.8'), border=1"
                 ", threshold=threshold)\n"
             )
         for prim in filter(
-            lambda p: not isinstance(p, prm.Well),
-            self.tech.primitives.__iter_type__(prm.Implant),
+            lambda p: not isinstance(p, _prm.Well),
+            self.tech.primitives.__iter_type__(_prm.Implant),
         ):
-            rgb = "LawnGreen" if prim.type_ == "n" else "Yellow"
+            rgb = "LawnGreen" if prim.type_ == _prm.nImplT else "Yellow"
             s += (
                 f"    style.addDrawingStyle(group='Active Layers', name='{prim.name}'"
                 f", color=toRGB('{rgb}'), pattern=toHexa('antihash0.8'), border=1"
                 ", threshold=threshold)\n"
             )
-        for prim in self.tech.primitives.__iter_type__(prm.WaferWire):
+        for prim in self.tech.primitives.__iter_type__(_prm.WaferWire):
             s += (
                 f"    style.addDrawingStyle(group='Active Layers', name='{prim.name}'"
                 ", color=toRGB('White'), pattern=toHexa('antihash0.8'), border=1"
                 ", threshold=threshold)\n"
             )
             if prim.pin is not None:
-                for pin in prim.pin:
-                    s += (
-                        "    style.addDrawingStyle(group='Active Layers'"
-                        f", name='{pin.name}', color=toRGB('White')"
-                        ", pattern=toHexa('antihash0.8'), border=2"
-                        ", threshold=threshold)\n"
-                    )
-        for i, prim in enumerate(self.tech.primitives.__iter_type__(prm.GateWire)):
+                s += (
+                    "    style.addDrawingStyle(group='Active Layers'"
+                    f", name='{prim.pin.name}', color=toRGB('White')"
+                    ", pattern=toHexa('antihash0.8'), border=2"
+                    ", threshold=threshold)\n"
+                )
+        for i, prim in enumerate(self.tech.primitives.__iter_type__(_prm.GateWire)):
             rgb = "Red" if i == 0 else "Orange"
             s += (
                 f"    style.addDrawingStyle(group='Active Layers', name='{prim.name}'"
@@ -1209,16 +1217,15 @@ class _TechnologyGenerator:
                 ", threshold=threshold)\n"
             )
             if prim.pin is not None:
-                for pin in prim.pin:
-                    s += (
-                        "    style.addDrawingStyle(group='Active Layers'"
-                        f", name='{pin.name}', color=toRGB('{rgb}')"
-                        ", pattern=toHexa('antihash0.8'), border=2"
-                        ", threshold=threshold)\n"
-                    )
+                s += (
+                    "    style.addDrawingStyle(group='Active Layers'"
+                    f", name='{prim.pin.name}', color=toRGB('{rgb}')"
+                    ", pattern=toHexa('antihash0.8'), border=2"
+                    ", threshold=threshold)\n"
+                )
 
         s += "\n    # Routing Layers.\n"
-        for i, prim in enumerate(self.tech.primitives.__iter_type__(prm.MetalWire)):
+        for i, prim in enumerate(self.tech.primitives.__iter_type__(_prm.MetalWire)):
             rgb = clrs[i%len(clrs)]
             hexa = "slash.8" if i == 0 else "poids4.8"
             s += (
@@ -1227,16 +1234,15 @@ class _TechnologyGenerator:
                 ", threshold=threshold)\n"
             )
             if prim.pin is not None:
-                for pin in prim.pin:
-                    s += (
-                        f"    style.addDrawingStyle(group='Routing Layers'"
-                        f", name='{pin.name}', color=toRGB('{rgb}'), pattern=toHexa('{hexa}')"
-                        ", border=2, threshold=threshold)\n"
-                    )
+                s += (
+                    f"    style.addDrawingStyle(group='Routing Layers'"
+                    f", name='{prim.pin.name}', color=toRGB('{rgb}'), pattern=toHexa('{hexa}')"
+                    ", border=2, threshold=threshold)\n"
+                )
 
         s += "\n    # Cuts (VIA holes).\n"
         for i, prim in enumerate(
-            self.tech.primitives.__iter_type__((prm.Via, prm.PadOpening)),
+            self.tech.primitives.__iter_type__((_prm.Via, _prm.PadOpening)),
         ):
             rgb = clrs[i%len(clrs)] if i > 0 else "0,150,150"
             s += (
@@ -1247,10 +1253,10 @@ class _TechnologyGenerator:
         s += "\n    # Blockages.\n"
         blockages = {prim.blockage for prim in filter(
             lambda p: p.blockage is not None,
-            self.tech.primitives.__iter_type__(prm._BlockageAttribute),
+            self.tech.primitives.__iter_type__(_prm.BlockageAttrPrimitiveT),
         )}
         for i, prim in enumerate(filter(
-            lambda p: p in blockages, self.tech.primitives.__iter_type__(prm.Marker)
+            lambda p: p in blockages, self.tech.primitives.__iter_type__(_prm.Marker)
         )):
             rgb = clrs[i%len(clrs)]
             hexa = "slash.8" if i == 0 else "poids4.8"
@@ -1293,31 +1299,31 @@ class _TechnologyGenerator:
 
 
 class FileExporter:
-    def __init__(self, *, tech: tch.Technology, gds_layers: GDSLayerSpecDict):
+    def __init__(self, *, tech: _tch.Technology, gds_layers: GDSLayerSpecDict):
         self.tech = tech
         self.gds_layers = gds_layers
 
     @overload
     def __call__(self,
         obj: None, *, routinggauge: None=None,
-    ):
+    ) -> str:
         ...
     @overload
     def __call__(self,
-        obj: lbr.StdCellLibrary, *, routinggauge: None=None,
-    ):
+        obj: _lbry.RoutingGaugeLibrary, *, routinggauge: None=None,
+    ) -> str:
         ...
     @overload
     def __call__(self,
-        obj: lbr.Library, *, routinggauge: Optional[lbr.RoutingGauge]=None,
-    ):
+        obj: _lbry.Library, *, routinggauge: Optional[_rg.RoutingGauge]=None,
+    ) -> str:
         ...
     def __call__(self,
-        obj: Optional[lbr.Library]=None, *, routinggauge: Optional[lbr.RoutingGauge]=None,
-    ):
+        obj: Optional[_lbry.Library]=None, *, routinggauge: Optional[_rg.RoutingGauge]=None,
+    ) -> str:
         if obj is None:
             s = self._s_tech()
-        elif isinstance(obj, lbr.Library):
+        elif isinstance(obj, _lbry.Library):
             s = self._s_lib(obj, routinggauge=routinggauge)
         else:
             raise TypeError("object has to be None or of type 'Library'")
@@ -1328,6 +1334,6 @@ class FileExporter:
         gen = _TechnologyGenerator()
         return gen(tech=self.tech, gds_layers=self.gds_layers)
 
-    def _s_lib(self, lib: lbr.Library, *, routinggauge: Optional[lbr.RoutingGauge]):
+    def _s_lib(self, lib: _lbry.Library, *, routinggauge: Optional[_rg.RoutingGauge]):
         gen = _LibraryGenerator(tech=self.tech)
         return gen(lib, routinggauge=routinggauge)
