@@ -1,16 +1,18 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 from textwrap import dedent
 from itertools import product
 from xml.etree import ElementTree as ET
-from typing import Any, Tuple, Dict, Set, Generator, Optional, Union, cast, overload
+from typing import Any, Tuple, Dict, Set, Iterable, Optional, Union, cast, overload
 
-from ... import _util, dispatch as dsp
+from ... import _util, dispatch as _dsp
 from ...typing import GDSLayerSpecDict
 from ...technology import (
-    property_ as prp, rule as rle, wafer_ as wfr, mask as msk, edge as edg,
-    geometry as geo, primitive as prm, net as _net, technology_ as tch,
+    property_ as _prp, rule as _rle, wafer_ as _wfr, mask as _msk, edge as _edg,
+    geometry as _geo, primitive as _prm, net as _net, technology_ as _tch,
 )
-from ...design import layout as lay, library as lbry
+from ...design import layout as _lay, cell as _cell, library as _lbry
+from ...design.layout import layout_ as _laylay
+from ..spice import SpicePrimParamsT, SpicePrimsParamSpec
 
 pya: Any # Silence pylance
 import pya
@@ -19,13 +21,13 @@ import pya
 __all__= ["FileExporter", "export2db"]
 
 
-class _MaskConverter(dsp.MaskDispatcher):
-    def __init__(self, *, tech: tch.Technology) -> None:
+class _MaskConverter(_dsp.MaskDispatcher):
+    def __init__(self, *, tech: _tch.Technology) -> None:
         super().__init__()
 
         self.tech = tech
 
-    def __call__(self, mask: msk._Mask) -> str:
+    def __call__(self, mask: _msk.MaskT) -> str:
         return super().__call__(mask)
 
     @staticmethod
@@ -34,29 +36,29 @@ class _MaskConverter(dsp.MaskDispatcher):
             name = "_" + name
         return name.replace(".", "_").replace(":", "__")
 
-    def DesignMask(self, mask: msk.DesignMask) -> str:
+    def DesignMask(self, mask: _msk.DesignMask) -> str:
         return self.__legalized_maskname(mask.name)
 
-    def _MaskAlias(self, mask: msk._MaskAlias) -> str:
+    def _MaskAlias(self, mask: _msk._MaskAlias) -> str:
         return self.__legalized_maskname(mask.name)
 
     # Handled on higher level
     # def _PartsWith(self, pw: msk._PartsWith):
     #     ...
 
-    def Join(self, join: msk.Join) -> str:
+    def Join(self, join: _msk.Join) -> str:
         return f"({'+'.join(self(m) for m in join.masks)})"
 
-    def Intersect(self, intersect: msk.Intersect) -> str:
+    def Intersect(self, intersect: _msk.Intersect) -> str:
         return f"({'&'.join(self(m) for m in intersect.masks)})"
 
-    def _MaskRemove(self, mr: msk._MaskRemove):
+    def _MaskRemove(self, mr: _msk._MaskRemove):
         return f"({self(mr.from_)}-{self(mr.what)})"
 
-    def _Wafer(self, wafer: wfr._Wafer, *args, **kwargs):
+    def _Wafer(self, wafer: _wfr._Wafer, *args, **kwargs):
         # Size box to be sure it is big enough for substrate enclosure check
         bias = 0.0
-        for act in self.tech.primitives.__iter_type__(prm.WaferWire):
+        for act in self.tech.primitives.__iter_type__(_prm.WaferWire):
             for enc in (act.min_substrate_enclosure, act.min_substrate_enclosure_same_type):
                 if enc is not None:
                     bias = max(bias, enc.max())
@@ -65,18 +67,18 @@ class _MaskConverter(dsp.MaskDispatcher):
 _mask_conv: Any = None
 
 
-class _EdgeConverter(dsp.EdgeDispatcher):
-    def __call__(self, edge: edg._Edge) -> str:
+class _EdgeConverter(_dsp.EdgeDispatcher):
+    def __call__(self, edge: _edg.EdgeT) -> str:
         return super().__call__(edge)
 
-    def MaskEdge(self, edge: edg.MaskEdge) -> str:
+    def MaskEdge(self, edge: _edg.MaskEdge) -> str:
         return f"{_mask_conv(edge.mask)}.edges"
 
-    def _DualEdgeOperation(self, op: edg._DualEdgeOperation) -> str:
+    def _DualEdgeOperation(self, op: _edg._DualEdgeOperation) -> str:
         s_edge1 = self(op.edge1)
-        if isinstance(op.edge2, msk._Mask):
+        if isinstance(op.edge2, _msk.MaskT):
             s_edge2 = _mask_conv(op.edge2)
-        elif isinstance(op.edge2, edg._Edge):
+        elif isinstance(op.edge2, _edg.EdgeT):
             s_edge2 = self(op.edge2)
         else: # pragma: no cover
             raise TypeError(f"[Internal error]Unexpected type for edge2 of {str(op)}")
@@ -85,34 +87,34 @@ class _EdgeConverter(dsp.EdgeDispatcher):
         else: # pragma: no cover
             raise NotImplementedError(f"[Internal error]Operation: {op.operation}")
 
-    def Join(self, join: edg.Join) -> str:
+    def Join(self, join: _edg.Join) -> str:
         s_join = "+".join(
-            _mask_conv(e) if isinstance(e, msk._Mask) else self(e)
+            _mask_conv(e) if isinstance(e, _msk.MaskT) else self(e)
             for e in join.edges
         )
         return f"({s_join})"
 
-    def Intersect(self, intersect: edg.Intersect) -> str:
+    def Intersect(self, intersect: _edg.Intersect) -> str:
         s_intersect = "&".join(
-            _mask_conv(e) if isinstance(e, msk._Mask) else self(e)
+            _mask_conv(e) if isinstance(e, _msk.MaskT) else self(e)
             for e in intersect.edges
         )
         return f"({s_intersect})"
 _edge_conv = _EdgeConverter()
 
 
-def _str_designmask(mask: msk.DesignMask, *, gds_layers: GDSLayerSpecDict):
+def _str_designmask(mask: _msk.DesignMask, *, gds_layers: GDSLayerSpecDict):
     gds_layer = gds_layers[mask.name]
     if not isinstance(gds_layer, tuple):
         gds_layer = (gds_layer, 0)
     return f"{_mask_conv(mask)} = input{gds_layer}\n"
 
 
-def _str_alias(mask: msk._MaskAlias):
+def _str_alias(mask: _msk._MaskAlias):
     return f"{_mask_conv(mask)} = {_mask_conv(mask.mask)}\n"
 
 
-def _str_grid(mask: msk._Mask, grid: float):
+def _str_grid(mask: _msk.MaskT, grid: float):
     s_mask = _mask_conv(mask)
     return dedent(f"""
         {s_mask}.ongrid({grid}).output(
@@ -121,8 +123,8 @@ def _str_grid(mask: msk._Mask, grid: float):
     """[1:])
 
 
-class _RuleConverter(dsp.RuleDispatcher):
-    def __call__(self, rule: rle._Rule, *, allow_unimplented: bool=False) -> str:
+class _RuleConverter(_dsp.RuleDispatcher):
+    def __call__(self, rule: _rle.RuleT, *, allow_unimplented: bool=False) -> str:
         if allow_unimplented:
             try:
                 s = super().__call__(rule)
@@ -133,10 +135,10 @@ class _RuleConverter(dsp.RuleDispatcher):
 
         return f"# {rule}\n{s}"
 
-    def GreaterEqual(self, ge: prp.Operators.GreaterEqual):
+    def GreaterEqual(self, ge: _prp.Operators.GreaterEqual):
         left = ge.left
         right = ge.right
-        if isinstance(left, msk._MaskProperty):
+        if isinstance(left, _msk._MaskProperty):
             s_mask = _mask_conv(left.mask)
             prop = left.prop_name
             if prop in {"width", "space"}:
@@ -160,31 +162,31 @@ class _RuleConverter(dsp.RuleDispatcher):
                         "{s_mask} density", "{s_mask} minimum density: {round(100*right)}%"
                     )
                 """[1:])
-        elif isinstance(left, msk._DualMaskProperty):
+        elif isinstance(left, _msk._DualMaskProperty):
             prop = left.prop_name
             if (
                 (prop == "space")
                 and (
-                    isinstance(left.mask1, msk._PartsWith)
-                    or isinstance(left.mask2, msk._PartsWith)
+                    isinstance(left.mask1, _msk._PartsWith)
+                    or isinstance(left.mask2, _msk._PartsWith)
                 )
             ):
                 # Special code for handling width based spacing rules
                 # Main objective if to support space tabels specified for primitives
                 # Other application area are mainly untested
-                if isinstance(left.mask1, msk._PartsWith):
+                if isinstance(left.mask1, _msk._PartsWith):
                     assert len(left.mask1.condition) == 1
                     cond = left.mask1.condition[0]
                     assert left.mask1.mask == left.mask2
                     s_mask = _mask_conv(left.mask1.mask)
                 else:
-                    assert isinstance(left.mask2, msk._PartsWith)
+                    assert isinstance(left.mask2, _msk._PartsWith)
                     assert len(left.mask2.condition) == 1
                     cond = left.mask2.condition[0]
                     assert left.mask2.mask == left.mask1
                     s_mask = _mask_conv(left.mask2.mask)
-                assert isinstance(cond, prp.Operators.GreaterEqual)
-                assert isinstance(cond.left, msk._MaskProperty)
+                assert isinstance(cond, _prp.Operators.GreaterEqual)
+                assert isinstance(cond.left, _msk._MaskProperty)
                 assert cond.left.prop_name == "width"
                 return dedent(f"""
                     space4width_check({s_mask}, {cond.right}, {right}).output(
@@ -201,13 +203,6 @@ class _RuleConverter(dsp.RuleDispatcher):
                         "Minimum spacing between {s_mask1} and {s_mask2}: {right}µm"
                     )
                 """[1:])
-            elif prop == "overlapwidth":
-                return dedent(f"""
-                    ({s_mask1}&{s_mask2}).width({right}).output(
-                        "{s_mask1}:{s_mask2} overlap width",
-                        "Minimum overlap widht of {s_mask1} and {s_mask2}: {right}µm"
-                    )
-                """[1:])
             elif prop == "extend_over":
                 return dedent(f"""
                     extend_check({s_mask2}, {s_mask1}, {right}).output(
@@ -215,13 +210,13 @@ class _RuleConverter(dsp.RuleDispatcher):
                         "Minimum extension of {s_mask1} of {s_mask2}: {right}µm"
                     )
                 """[1:])
-        elif isinstance(left, msk._DualMaskEnclosureProperty):
+        elif isinstance(left, _msk._DualMaskEnclosureProperty):
             s_mask1 = _mask_conv(left.mask1)
             s_mask2 = _mask_conv(left.mask2)
             prop = left.prop_name
             if prop == "enclosed_by":
                 # TODO: Proper typing for Property
-                assert isinstance(right, prp.Enclosure)
+                assert isinstance(right, _prp.Enclosure)
                 if not right.is_assymetric:
                     return dedent(f"""
                         {s_mask2}.enclosing({s_mask1}, {right.first}).output(
@@ -240,7 +235,7 @@ class _RuleConverter(dsp.RuleDispatcher):
                             "{s_desc}"
                         )
                     """[1:])
-        elif isinstance(left, edg._EdgeProperty):
+        elif isinstance(left, _edg._EdgeProperty):
             s_edge = _edge_conv(left.edge)
             prop = left.prop_name
             if prop == "length":
@@ -250,10 +245,10 @@ class _RuleConverter(dsp.RuleDispatcher):
                         "Minimum length of {s_edge}: {right}µm"
                     )
                 """[1:])
-        elif isinstance(left, edg._DualEdgeProperty):
+        elif isinstance(left, _edg._DualEdgeProperty):
             s_edge1 = _edge_conv(left.edge1)
             s_edge2 = (
-                _mask_conv(left.edge2)+".edges" if isinstance(left.edge2, msk._Mask)
+                _mask_conv(left.edge2)+".edges" if isinstance(left.edge2, _msk.MaskT)
                 else _edge_conv(left.edge2)
             )
             prop = left.prop_name
@@ -267,12 +262,11 @@ class _RuleConverter(dsp.RuleDispatcher):
 
         raise NotImplementedError(f"GreateEqual rule '{ge}'") # pragma: no cover
 
-    def Equal(self, eq: prp.Operators.Equal) -> str:
+    def Equal(self, eq: _prp.Operators.Equal) -> str:
         left = eq.left
         right = eq.right
-        # TODO: proper typing of Property
-        assert isinstance(right, float)
-        if isinstance(left, msk._MaskProperty):
+        assert isinstance(right, left.value_type)
+        if isinstance(left, _msk._MaskProperty):
             s_mask = _mask_conv(left.mask)
             prop = left.prop_name
             if prop == "width":
@@ -285,7 +279,7 @@ class _RuleConverter(dsp.RuleDispatcher):
                 if round(right, 6) != 0.0:
                     raise ValueError("For area equal check value can only be 0.0")
                 return f'{s_mask}.output("{s_mask} empty")\n'
-        elif isinstance(left, edg._EdgeProperty):
+        elif isinstance(left, _edg._EdgeProperty):
             s_edge = _edge_conv(left.edge)
             prop = left.prop_name
             if prop == "length":
@@ -295,10 +289,10 @@ class _RuleConverter(dsp.RuleDispatcher):
 
         raise NotImplementedError(f"Equal rule '{eq}'") # pragma: no cover
 
-    def _MaskAlias(self, alias: msk._MaskAlias) -> str:
+    def _MaskAlias(self, alias: _msk._MaskAlias) -> str:
         return f"{_mask_conv(alias)} = {_mask_conv(alias.mask)}\n"
 
-    def Connect(self, conn: msk.Connect) -> str:
+    def Connect(self, conn: _msk.Connect) -> str:
         return "".join(
             f"connect({_mask_conv(mask1)}, {_mask_conv(mask2)})\n"
             for mask1, mask2 in product(conn.mask1, conn.mask2)
@@ -306,26 +300,28 @@ class _RuleConverter(dsp.RuleDispatcher):
 _rule_conv = _RuleConverter()
 
 
-def _str_lvsresistor(res: prm.Resistor):
+def _str_lvsresistor(res: _prm.Resistor, *, params: SpicePrimParamsT):
     s = f"# {res.name}\n"
 
     s_res = _mask_conv(res.mask)
     s_conn = _mask_conv(res.wire.conn_mask)
 
-    s += dedent(f"""
-        extract_devices(resistor("{res.name}", {res.sheetres}), {{
-            "R" => {s_res}, "C" => {s_conn},
-        }})
-        same_device_classes("{res.name}", "RES")
-    """[1:])
+    sheetres = ["sheetres"]
+    if sheetres is not None:
+        s += dedent(f"""
+            extract_devices(resistor("{res.name}", {sheetres}), {{
+                "R" => {s_res}, "C" => {s_conn},
+            }})
+            same_device_classes("{res.name}", "RES")
+        """[1:])
 
     return s
 
 
-def _str_lvsdiode(tech: tch.Technology, diode: prm.Diode):
+def _str_lvsdiode(tech: _tch.Technology, diode: _prm.Diode, spice_params: SpicePrimParamsT):
     s = f"# {diode.name}\n"
 
-    is_n = diode.implant.type_ == "n"
+    is_n = diode.implant.type_ == _prm.nImplT
 
     s_diode = _mask_conv(diode.mask)
     s_conn = _mask_conv(diode.wire.conn_mask)
@@ -344,7 +340,7 @@ def _str_lvsdiode(tech: tch.Technology, diode: prm.Diode):
         s_conn_port = "tA"
 
     s += dedent(f"""
-        extract_devices(diode("{diode.model}"), {{
+        extract_devices(diode("{spice_params['model']}"), {{
             "P" => {s_p}, "N" => {s_n}, "{s_conn_port}" => {s_conn}
         }})
     """[1:])
@@ -352,7 +348,7 @@ def _str_lvsdiode(tech: tch.Technology, diode: prm.Diode):
     return s
 
 
-def _str_lvsmosfet(tech: tch.Technology, mosfet: prm.MOSFET):
+def _str_lvsmosfet(tech: _tch.Technology, mosfet: _prm.MOSFET, spice_params: SpicePrimParamsT):
     s = f"# {mosfet.name}\n"
 
     s_sd = _mask_conv(mosfet.gate.active.conn_mask)
@@ -364,7 +360,7 @@ def _str_lvsmosfet(tech: tch.Technology, mosfet: prm.MOSFET):
     s_poly = _mask_conv(mosfet.gate.poly.conn_mask)
 
     s += dedent(f"""
-        extract_devices(mos4("{mosfet.model}"), {{
+        extract_devices(mos4("{spice_params['model']}"), {{
             "SD" => {s_sd}, "G" => {s_gate}, "tG" => {s_poly}, "W" => {s_bulk},
         }})
     """[1:])
@@ -374,12 +370,13 @@ def _str_lvsmosfet(tech: tch.Technology, mosfet: prm.MOSFET):
 
 class FileExporter:
     def __init__(self, *,
-        tech: tch.Technology, export_name: Optional[str]=None,
-        gds_layers: GDSLayerSpecDict,
+        tech: _tch.Technology, export_name: Optional[str]=None,
+        gds_layers: GDSLayerSpecDict, prims_spiceparams: SpicePrimsParamSpec,
     ):
         self.tech = tech
         self.export_name = tech.name if export_name is None else export_name
         self.gds_layers = gds_layers
+        self.prims_spiceparams = prims_spiceparams
 
         global _mask_conv
         _mask_conv = _MaskConverter(tech=tech)
@@ -474,35 +471,36 @@ class FileExporter:
         """[1:])
 
         s += "\n# Define layers\n"
-        dms = tuple(self.tech.rules.__iter_type__(msk.DesignMask))
+        assert isinstance(self.tech.rules, _rle.Rules), "Internal error"
+        dms = tuple(self.tech.rules.__iter_type__(_msk.DesignMask))
         s += "".join(_str_designmask(dm, gds_layers=self.gds_layers) for dm in dms)
 
         s += "\n# Grid check\n"
-        gridrules = cast(Tuple[prp.Operators.Equal, ...], tuple(filter(
+        gridrules = cast(Tuple[_prp.Operators.Equal, ...], tuple(filter(
                 lambda rule: (
-                    isinstance(rule, prp.Operators.Equal)
-                    and isinstance(rule.left, msk._MaskProperty)
+                    isinstance(rule, _prp.Operators.Equal)
+                    and isinstance(rule.left, _msk._MaskProperty)
                     and (rule.left.prop_name == "grid")
                 ),
                 self.tech.rules,
             )
         ))
         gridspecs = {
-            cast(msk._MaskProperty, gridrule.left).mask: gridrule.right
+            cast(_msk._MaskProperty, gridrule.left).mask: gridrule.right
             for gridrule in gridrules
         }
-        globalgrid = gridspecs[wfr.wafer]
+        globalgrid = gridspecs[_wfr.wafer]
         s += "".join(
             _str_grid(dm, cast(float, gridspecs.get(dm, globalgrid)))
             for dm in dms
         )
 
         s += "\n# Derived layers\n"
-        aliases = tuple(self.tech.rules.__iter_type__(msk._MaskAlias))
+        aliases = tuple(self.tech.rules.__iter_type__(_msk._MaskAlias))
         s += "".join(_rule_conv(alias) for alias in aliases)
 
         s += "\n# Connectivity\n"
-        conns = tuple(self.tech.rules.__iter_type__(msk.Connect))
+        conns = tuple(self.tech.rules.__iter_type__(_msk.Connect))
         s += "".join(_rule_conv(conn) for conn in conns)
 
         s += "\n# DRC rules\n" + "".join(
@@ -577,26 +575,36 @@ class FileExporter:
     def _s_extractrules(self):
         # TODO: bug report for failing LVS on hierarchical LVS and diodes
         s = "flat\n\n# Define layers\n"
-        dms = tuple(self.tech.rules.__iter_type__(msk.DesignMask))
+        assert isinstance(self.tech.rules, _rle.Rules), "Internal error"
+        dms = tuple(self.tech.rules.__iter_type__(_msk.DesignMask))
         s += "".join(_str_designmask(dm, gds_layers=self.gds_layers) for dm in dms)
-        aliases = tuple(self.tech.rules.__iter_type__(msk._MaskAlias))
+        aliases = tuple(self.tech.rules.__iter_type__(_msk._MaskAlias))
         s += "".join(_str_alias(alias) for alias in aliases)
 
         s += "\n# Connectivity\n"
-        conns = tuple(self.tech.rules.__iter_type__(msk.Connect))
+        conns = tuple(self.tech.rules.__iter_type__(_msk.Connect))
         s += "".join(_rule_conv(conn) for conn in conns)
 
         s += "\n# Resistors\n"
-        resistors = tuple(self.tech.primitives.__iter_type__(prm.Resistor))
-        s += "".join(_str_lvsresistor(res) for res in resistors)
+        resistors = tuple(self.tech.primitives.__iter_type__(_prm.Resistor))
+        s += "".join(
+            _str_lvsresistor(res, params=self.prims_spiceparams[res])
+            for res in resistors
+        )
 
         s += "\n# Diodes\n"
-        diodes = tuple(self.tech.primitives.__iter_type__(prm.Diode))
-        s += "".join(_str_lvsdiode(self.tech, diode) for diode in diodes)
+        diodes = tuple(self.tech.primitives.__iter_type__(_prm.Diode))
+        s += "".join(
+            _str_lvsdiode(self.tech, diode, self.prims_spiceparams[diode])
+            for diode in diodes
+        )
 
         s += "\n# Transistors\n"
-        mosfets = tuple(self.tech.primitives.__iter_type__(prm.MOSFET))
-        s += "".join(_str_lvsmosfet(self.tech, mosfet) for mosfet in mosfets)
+        mosfets = tuple(self.tech.primitives.__iter_type__(_prm.MOSFET))
+        s += "".join(
+            _str_lvsmosfet(self.tech, mosfet, self.prims_spiceparams[mosfet])
+            for mosfet in mosfets
+        )
 
         s += "\nnetlist\n"
 
@@ -615,7 +623,7 @@ class FileExporter:
         ropts = ET.SubElement(lyt, "reader-options")
         roptscom = ET.SubElement(ropts, "common")
         ET.SubElement(roptscom, "create-other-layers").text = "true"
-        def s_gds_layer(m: msk.DesignMask):
+        def s_gds_layer(m: _msk.DesignMask):
             try:
                 l = self.gds_layers[m.name]
             except KeyError: # pragma: no cover
@@ -638,44 +646,44 @@ class FileExporter:
         return lyt
 
 
-class _ShapeExporter(dsp.ShapeDispatcher):
+class _ShapeExporter(_dsp.ShapeDispatcher):
     """Converts a _geo,_Shape object to KLayout database object"""
     def __init__(self, *, export_fullshape: bool):
-        self._mps_exported: Optional[Set[geo.MultiPartShape]]
+        self._mps_exported: Optional[Set[_geo.MultiPartShape]]
         self._mps_exported = set() if export_fullshape else None
 
-    def _pointsshapes(self, shape: geo._Shape) -> Generator[Any, None, None]:
+    def _pointsshapes(self, shape: _geo._Shape) -> Iterable[Any]:
         # Helper to convert the individual pointsshapes
         for pointshape in shape.pointsshapes:
             conv = self(pointshape)
             assert not _util.is_iterable(conv), "Internal error: unsupported"
             yield conv
 
-    def _Shape(self, shape: geo._Shape): # pragma: no cover
+    def _Shape(self, shape: _geo._Shape): # pragma: no cover
         raise ValueError(f"Unsupported object of type {shape.__class__.__name__}")
 
-    def Point(self, point: geo.Point) -> "pya.DPoint": # type: ignore
+    def Point(self, point: _geo.Point) -> "pya.DPoint": # type: ignore
         return pya.DPoint(point.x, point.y)
 
-    def Line(self, line: geo.Line) -> "pya.DPath": # type: ignore
+    def Line(self, line: _geo.Line) -> "pya.DPath": # type: ignore
         # We represent a Line by a path with zero width
         points = (self.Point(line.point1), self.Point(line.point2))
         return pya.DPath(points, 0.0)
 
-    def Polygon(self, polygon: geo.Polygon, **_) -> "pya.DSimplePolygon": # type: ignore
+    def Polygon(self, polygon: _geo.Polygon, **_) -> "pya.DSimplePolygon": # type: ignore
         # In PDKMaster polygon needs last point to be same as first point;
         # in klayout this is not the case.
-        points = tuple(self.Point(point) for point in polygon.points[:-1])
+        points = tuple(self.Point(point) for point in tuple(polygon.points)[:-1])
         return pya.DSimplePolygon(points)
 
-    def Rect(self, rect: geo.Rect) -> "pya.DBox": # type: ignore
+    def Rect(self, rect: _geo.Rect) -> "pya.DBox": # type: ignore
         return pya.DBox(rect.left, rect.bottom, rect.right, rect.top)
 
-    def RectRing(self, rs: geo.RectRing) -> Generator[Any, None, None]:
+    def RectRing(self, rs: _geo.RectRing) -> Iterable[Any]:
         # TODO: Can repetition information be retained in KLayout ?
         return self._pointsshapes(rs)
 
-    def MultiPartShape(self, mps: geo.MultiPartShape):
+    def MultiPartShape(self, mps: _geo.MultiPartShape):
         if self._mps_exported is None:
             return self(mps.fullshape)
         else:
@@ -685,13 +693,13 @@ class _ShapeExporter(dsp.ShapeDispatcher):
                 self._mps_exported.add(mps)
                 return self(mps.fullshape)
 
-    def MultiPartShape__Part(self, part: geo.MultiPartShape._Part):
+    def MultiPartShape__Part(self, part: _geo.MultiPartShape._Part):
         if self._mps_exported is None:
             return self(part.partshape)
         else:
             return self(part.multipartshape)
 
-    def MultiShape(self, ms: geo.MultiShape) -> Generator[Any, None, None]:
+    def MultiShape(self, ms: _geo.MultiShape) -> Iterable[Any]:
         for shape in ms.shapes:
             conv = self(shape)
             if _util.is_iterable(conv):
@@ -699,7 +707,7 @@ class _ShapeExporter(dsp.ShapeDispatcher):
             else:
                 yield conv
 
-    def RepeatedShape(self, rs: geo.RepeatedShape) -> Generator[Any, None, None]:
+    def RepeatedShape(self, rs: _geo.RepeatedShape) -> Iterable[Any]:
         # TODO: Can repetition information be retained in KLayout ?
         return self._pointsshapes(rs)
 
@@ -707,12 +715,12 @@ class _ShapeExporter(dsp.ShapeDispatcher):
     # ArrayShape -> RepeatedShape
 
 
-class _MaskLayerDict(Dict[msk.DesignMask, int]):
+class _MaskLayerDict(Dict[_msk.DesignMask, int]):
     def __init__(self, *, layout: "pya.Layout", gds_layers: GDSLayerSpecDict): # type: ignore
         self._layout = layout
         self._gds_layers = gds_layers
 
-    def __getitem__(self, mask: msk.DesignMask) -> int:
+    def __getitem__(self, mask: _msk.DesignMask) -> int:
         if mask not in self:
             layer = self._gds_layers[mask.name]
             if isinstance(layer, tuple):
@@ -723,15 +731,15 @@ class _MaskLayerDict(Dict[msk.DesignMask, int]):
         return super().__getitem__(mask)
 
 
-_rotation_to_rot_mirr: Dict[geo.Rotation, Tuple[int, bool]] = {
-    geo.Rotation.No: (0, False),
-    geo.Rotation.R90: (1, False),
-    geo.Rotation.R180: (2, False),
-    geo.Rotation.R270: (3, False),
-    geo.Rotation.MX: (0, True),
-    geo.Rotation.MX90:(1, True),
-    geo.Rotation.MY: (2, True),
-    geo.Rotation.MY90: (3, True),
+_rotation_to_rot_mirr: Dict[_geo.Rotation, Tuple[int, bool]] = {
+    _geo.Rotation.No: (0, False),
+    _geo.Rotation.R90: (1, False),
+    _geo.Rotation.R180: (2, False),
+    _geo.Rotation.R270: (3, False),
+    _geo.Rotation.MX: (0, True),
+    _geo.Rotation.MX90:(1, True),
+    _geo.Rotation.MY: (2, True),
+    _geo.Rotation.MY90: (3, True),
 }
 
 
@@ -742,15 +750,15 @@ class _LayoutExporter:
     def _clear(self):
         self.layout = None
         self.layerdict = None
-        self.cell_lookup: Dict[str, Tuple[Optional[lbry._Cell], "pya.Layout"]] = {} # type: ignore
-        self.cells_todo: Set[lbry._Cell] = set()
-        self.cells_done: Set[lbry._Cell] = set()
+        self.cell_lookup: Dict[str, Tuple[Optional[_cell.Cell], "pya.Layout"]] = {} # type: ignore
+        self.cells_todo: Set[_cell.Cell] = set()
+        self.cells_done: Set[_cell.Cell] = set()
         self.cell = None
         self.shapeexporter = None
-        self.pinmasks: Optional[Tuple[msk._Mask, ...]] = None
+        self.pinmasks: Optional[Tuple[_msk.MaskT, ...]] = None
 
     def __call__(self, *,
-        obj: Union[geo.MaskShape, geo.MaskShapes, lay._Layout, lbry.Library],
+        obj: Union[_geo.MaskShape, _geo.MaskShapes, _lay.LayoutT, _lbry.Library],
         gds_layers: GDSLayerSpecDict, cell_name: Optional[str],
         merge: bool, add_pin_label: bool, dbu: float=0.001,
     ) -> "pya.Layout": # type: ignore
@@ -759,7 +767,7 @@ class _LayoutExporter:
         self.layerdict = _MaskLayerDict(layout=layout, gds_layers=gds_layers)
         self.shapeexporter = _ShapeExporter(export_fullshape=True)
 
-        if isinstance(obj, lbry.Library):
+        if isinstance(obj, _lbry.Library):
             assert cell_name is None
             # A cell will be created in add() function below for each cell of the library
         else:
@@ -806,7 +814,7 @@ class _LayoutExporter:
 
         return cell
 
-    def _register_cell(self, cell: lbry._Cell):
+    def _register_cell(self, cell: _cell.Cell):
         assert self.layout is not None
         assert self.cell_lookup is not None
 
@@ -824,11 +832,11 @@ class _LayoutExporter:
         return self.cell_lookup[name][1]
 
     def _add(self,
-        o: Union[geo.MaskShape, geo.MaskShapes, lay._Layout, lbry._Cell, lbry.Library],
-        add_pin_label: bool, net: Optional[_net.Net],
+        o: Union[_geo.MaskShape, _geo.MaskShapes, _lay.LayoutT, _cell.Cell, _lbry.Library],
+        add_pin_label: bool, net: Optional[_net.NetT],
     ) -> None:
         assert self.shapeexporter is not None
-        if isinstance(o, geo.MaskShape):
+        if isinstance(o, _geo.MaskShape):
             assert (self.cell is not None) and (self.layerdict is not None)
             layer = self.layerdict[o.mask]
             shapes = self.cell.shapes(layer)
@@ -847,26 +855,25 @@ class _LayoutExporter:
                 if o.mask in self.pinmasks:
                     assert net is not None
                     for ps in o.shape.pointsshapes:
-                        if isinstance(ps, geo.Rect):
+                        if isinstance(ps, _geo.Rect):
                             point = ps.center
                         else:
-                            point = _util.first(ps.points)
+                            point = _util.get_first_of(ps.points)
                         shapes.insert(pya.DText(net.name, point.x, point.y))
-        elif isinstance(o, geo.MaskShapes):
+        elif isinstance(o, _geo.MaskShapes):
             for ms in o:
                 self._add(ms, add_pin_label=add_pin_label, net=net)
-        elif isinstance(o, lay._Layout):
+        elif isinstance(o, _lay.LayoutT):
             prims = o.fab.tech.primitives
             pinmasks = []
-            for prim in prims.__iter_type__(prm._PinAttribute):
+            for prim in prims.__iter_type__(_prm.PinAttrPrimitiveT):
                 if prim.pin is not None:
-                    for pin in prim.pin:
-                        pinmasks.append(pin.mask)
+                    pinmasks.append(prim.pin.mask)
             self.pinmasks = tuple(pinmasks)
-            for sl in o.sublayouts:
-                if isinstance(sl, lay.MaskShapesSubLayout):
+            for sl in o._sublayouts:
+                if isinstance(sl, _laylay._MaskShapesSubLayout):
                     self._add(sl.shapes, add_pin_label=add_pin_label, net=sl.net)
-                elif isinstance(sl, lay._InstanceSubLayout):
+                elif isinstance(sl, _laylay._InstanceSubLayout):
                     assert self.cell is not None
                     instcell = self._register_cell(sl.inst.cell)
                     rot, mirr = _rotation_to_rot_mirr[sl.rotation]
@@ -876,13 +883,13 @@ class _LayoutExporter:
                     raise RuntimeError(
                         "Internal error: unsupported",
                     )
-        elif isinstance(o, lbry._Cell):
+        elif isinstance(o, _cell.Cell):
             assert net is None
             # Register cell to be exported
             self._register_cell(o)
             # Make use of self.cell after this an error
             self.cell = None
-        elif isinstance(o, lbry.Library):
+        elif isinstance(o, _lbry.Library):
             # Initialize list of cells to tape-out, each of the cells will be
             # added in a loop below this function.
             assert net is None
@@ -898,7 +905,7 @@ class _LayoutExporter:
 
 @overload
 def export2db(
-    obj: geo._Shape, *,
+    obj: _geo._Shape, *,
     export_fullshape: Optional[bool]=None,
     add_pin_label: bool=False,
     gds_layers: None=None,
@@ -908,7 +915,7 @@ def export2db(
     ... # pragma: no cover
 @overload
 def export2db(
-    obj: Union[geo.MaskShape, geo.MaskShapes, lay._Layout, lbry.Library], *,
+    obj: Union[_geo.MaskShape, _geo.MaskShapes, _lay.LayoutT, _lbry.Library], *,
     export_fullshape: None=None,
     add_pin_label: bool=False,
     gds_layers: GDSLayerSpecDict,
@@ -917,7 +924,7 @@ def export2db(
 ) -> "pya.Layout": # type: ignore
     ... # pragma: no cover
 def export2db(
-    obj: Union[geo._Shape, geo.MaskShape, geo.MaskShapes, lay._Layout, lbry.Library], *,
+    obj: Union[_geo._Shape, _geo.MaskShape, _geo.MaskShapes, _lay.LayoutT, _lbry.Library], *,
     export_fullshape: Optional[bool]=None, add_pin_label: bool=False,
     gds_layers: Optional[GDSLayerSpecDict]=None,
     cell_name: Optional[str]=None,
@@ -956,7 +963,7 @@ def export2db(
     """
     # TODO: Provide facility to also export netlist information
 
-    if isinstance(obj, geo._Shape):
+    if isinstance(obj, _geo._Shape):
         assert (gds_layers is None) and (cell_name is None) and (not add_pin_label)
         if export_fullshape is None:
             export_fullshape = False
