@@ -1,14 +1,14 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 import abc
 from itertools import combinations
 from typing import (
-    Any, List, Tuple, Dict, Union, Iterable, Generator, Optional, cast, overload,
+    Any, List, Tuple, Dict, Union, Iterable, Optional, overload,
 )
 
 from ... import _util
 from ...technology import geometry as _geo, mask as _msk
-from ...design import layout as _lay, library as _lbry
-
+from ...design import layout as _lay, cell as _cell, library as _lbry
+from ...design.layout import layout_ as _laylay
 from .export import export2db
 
 import pya
@@ -56,7 +56,7 @@ def _import_polygon(polygon) -> _geo.Polygon:
             else polygon.to_simple_polygon()
         )
         spoly = poly2.to_dtype(_geo.epsilon)
-        p0 = _util.first(spoly.each_point())
+        p0 = _util.get_first_of(spoly.each_point())
         return _geo.Polygon(points=(
             *(_geo.Point(x=p.x, y=p.y) for p in spoly.each_point()),
             _geo.Point(x=p0.x, y=p0.y),
@@ -158,7 +158,8 @@ class _MPSDictElem:
 
         # Check sum of parts is fullshape
         region = sum(partregions, pya.Region())
-        region.merge()
+        # pyright wrongly derives in as output of sum
+        region.merge() # pyright: ignore
         if not (region ^ fullshaperegion).is_empty():
             raise ValueError("MultiPartShape parts do not match fullshape")
 
@@ -252,7 +253,7 @@ class _Reffed:
         return # pragma: no cover
 
     @abc.abstractmethod
-    def deref(self) -> Union[_geo._Shape, _geo.MaskShape, _geo.MaskShapes, _lay._SubLayout]:
+    def deref(self) -> Union[_geo._Shape, _geo.MaskShape, _geo.MaskShapes, _laylay._SubLayout]:
         raise RuntimeError("Non overloaded abstract method") # pragma: no cover
 
 
@@ -263,7 +264,7 @@ class _ShapeReffed(_Reffed):
 
 
 class _ShapeRef(_ShapeReffed):
-    def __init__(self, *, shape=_geo._Shape):
+    def __init__(self, *, shape: _geo._Shape):
         self.shape = shape
 
     def deref(self) -> _geo._Shape:
@@ -280,7 +281,7 @@ class _PartRef(_ShapeReffed):
         return self.elem.part_is_merged(idx=self.idx)
     @property
     def region(self):
-        return _util.nth(self.elem.partregions, self.idx)
+        return _util.get_nth_of(self.elem.partregions, n=self.idx)
 
     def deref(self) -> _geo.MultiPartShape._Part:
         return self.elem.deref_part(self.idx)
@@ -381,15 +382,15 @@ class _MaskShapesRef(_Reffed):
 
 class _SubLayoutReffed(_Reffed):
     @abc.abstractmethod
-    def deref(self) -> _lay._SubLayout:
+    def deref(self) -> _laylay._SubLayout:
         raise RuntimeError("Non overloaded abstract method") # pragma: no cover
 
 
 class _SubLayoutRef(_SubLayoutReffed):
-    def __init__(self, *, sublayout: _lay._SubLayout):
+    def __init__(self, *, sublayout: _laylay._SubLayout):
         self.sublayout = sublayout
 
-    def deref(self) -> _lay._SubLayout:
+    def deref(self) -> _laylay._SubLayout:
         return self.sublayout
 
 
@@ -398,8 +399,8 @@ class _MaskShapesSubLayoutRef(_SubLayoutReffed):
         self.net = net
         self.ref = ref
 
-    def deref(self) -> _lay.MaskShapesSubLayout:
-        return _lay.MaskShapesSubLayout(net=self.net, shapes=self.ref.deref())
+    def deref(self) -> _laylay._MaskShapesSubLayout:
+        return _laylay._MaskShapesSubLayout(net=self.net, shapes=self.ref.deref())
 
 
 class _MPSDict(Dict[_geo.MultiPartShape, _MPSDictElem]):
@@ -428,7 +429,7 @@ class _ShapeMerger:
         return self._mps_lookup
 
     @overload
-    def __call__(self, shape: _lay._Layout) -> _lay._Layout:
+    def __call__(self, shape: _lay.LayoutT) -> _lay.LayoutT:
         ... # pragma: no cover
     @overload
     def __call__(self, shape: _geo.MaskShape) -> _MaskShapeRef:
@@ -446,7 +447,7 @@ class _ShapeMerger:
     def __call__(self, shape: Iterable[_geo._Shape]) -> Tuple[_ShapeReffed, ...]:
         ... # pragma: no cover
     def __call__(self, shape):
-        if isinstance(shape, _lay._Layout):
+        if isinstance(shape, _lay.LayoutT):
             return self.merge_layout(shape)
         elif isinstance(shape, _geo.MaskShape):
             return self.merge_maskshape(shape)
@@ -566,18 +567,18 @@ class _ShapeMerger:
             result = _MultiShapeRef(shapes=tuple(shapes))
         return result
 
-    def merge_sublayout(self, sl: _lay._SubLayout) -> _SubLayoutReffed:
-        if isinstance(sl, _lay.MaskShapesSubLayout):
+    def merge_sublayout(self, sl: _laylay._SubLayout) -> _SubLayoutReffed:
+        if isinstance(sl, _laylay._MaskShapesSubLayout):
             return _MaskShapesSubLayoutRef(net=sl.net, ref=self(sl.shapes))
         else:
             # TODO: unit test with _InstanceSubLayout
             return _SubLayoutRef(sublayout=sl) # pragma: no cover
 
-    def merge_layout(self, layout: _lay._Layout) -> _lay._Layout:
+    def merge_layout(self, layout: _lay.LayoutT) -> _lay.LayoutT:
         # Create a tuple so we have merged all the sublayouts before we are going to deref
         # them
-        slrefs = tuple(map(self.merge_sublayout, layout.sublayouts))
-        sls = _lay.SubLayouts(slref.deref() for slref in slrefs)
+        slrefs = tuple(map(self.merge_sublayout, layout._sublayouts))
+        sls = _laylay._SubLayouts(slref.deref() for slref in slrefs)
         return layout.fab.new_layout(sublayouts=sls, boundary = layout.boundary)
 
     def merge_shapes(self, shapes: Iterable[_geo._Shape]) -> Tuple[_ShapeReffed, ...]:
@@ -597,10 +598,10 @@ class _ShapeMerger:
 
 
 @overload
-def merge(obj: _lay._Layout) -> _lay._Layout:
+def merge(obj: _lay.LayoutT) -> _lay.LayoutT:
     ... # pragma: no cover
 @overload
-def merge(obj: _lbry._Cell) -> None:
+def merge(obj: _cell.Cell) -> None:
     ... # pragma: no cover
 @overload
 def merge(obj: _lbry.Library) -> None:
@@ -616,11 +617,11 @@ def merge(obj: _geo.MaskShapes) -> _geo.MaskShapes:
     ... # pragma: no cover
 def merge(
     obj: Union[
-        _lay._Layout, _lbry._Cell, _lbry.Library, _geo.MultiShape,
+        _lay.LayoutT, _cell.Cell, _lbry.Library, _geo.MultiShape,
         _geo.MaskShape, _geo.MaskShapes,
     ],
 ) -> Optional[Union[
-    _lay._Layout, _geo._Shape, _geo.MaskShape, _geo.MaskShapes,
+    _lay.LayoutT, _geo._Shape, _geo.MaskShape, _geo.MaskShapes,
 ]]:
     """This function allows to use the KLayout engine to merge PDKMaster _Shape objects;
     either directly or as part of another PDKMaster object
@@ -641,15 +642,15 @@ def merge(
         For object of type `_Cell` and `Library` the merging will be done on the provided
         object and no value is returned.
     """
-    if isinstance(obj, _lbry._Cell):
-        for l in obj.layouts:
+    if isinstance(obj, _cell.Cell):
+        for l in obj._layouts:
             l.layout = merge(l.layout)
     elif isinstance(obj, _lbry.Library):
         for cell in obj.cells:
             merge(cell)
     else:
         merged = _ShapeMerger()(obj)
-        if isinstance(merged, _lay._Layout):
+        if isinstance(merged, _lay.LayoutT):
             return merged
         else:
             return merged.deref()
