@@ -1,10 +1,11 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 from matplotlib import pyplot as plt
 import shapely.geometry as sh_geo
 import descartes
 
 from ...technology import geometry as _geo
 from ...design import layout as _lay
+from ...design.layout import layout_ as _laylay
 
 from ... import _util
 
@@ -20,9 +21,9 @@ class Plotter:
         if _util.is_iterable(obj):
             for item in obj:
                 self.plot(item)
-        elif isinstance(obj, _lay._Layout):
-            self.plot(obj.sublayouts)
-        elif isinstance(obj, _lay.MaskShapesSubLayout):
+        elif isinstance(obj, _lay.LayoutT):
+            self.plot(obj._sublayouts)
+        elif isinstance(obj, _laylay._MaskShapesSubLayout):
             for ms in obj.shapes:
                 self.plot(ms)
         elif isinstance(obj, _geo.MaskShape):
