@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 from ... import _util
 from .skill_grammar import SkillFile, SkillContext, SkillInterpreter, _skill_if
 
@@ -392,7 +392,7 @@ def _extractDevice(elems, *, type_, **kwargs):
         if isinstance(elem, (dict, list)):
             if isinstance(elem, dict):
                 assert(len(elem)) == 1
-                key, v = _util.first(elem.items())
+                key, v = _util.get_first_of(elem.items())
             else:
                 key = elem[0]
                 v = elem[1:]
