@@ -1,24 +1,31 @@
-"""Generate a pdkmaster technology file"""
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
-from importlib import import_module
-from textwrap import indent, dedent
-from typing import Any
+"""Generate a pdkmaster technology file
 
-from ... import dispatch as dsp
+This module is currently not used and likely does not produce usable results.
+It is adviced to not use this module in it's current state; no support whatsoever
+is given on this module.  
+The module was originally used by the TSMC 0.18 PDKMaster PDK. This PDK is currently
+not maintained and is not up to date with recent API chances. This module will need
+to be updated if the TSMC 0.18 is updated.
+"""
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
+from textwrap import indent, dedent
+from typing import Tuple, Optional
+
+from ... import dispatch as _dsp
 from ...technology import (
-    primitive as prm, mask as msk, technology_ as tch,
+    property_ as _prp, primitive as _prm, mask as _msk, technology_ as _tch,
 )
-from ...design import circuit as ckt, library as lib
+from ...design import circuit as _ckt, library as _lbry
 
 
 __all__ = ["generate"]
 
 
-def _str_prim(prim):
+def _str_prim(prim: _prm.PrimitiveT):
     return f"prims['{prim.name}']"
 
 
-def _str_primtuple(t):
+def _str_primtuple(t: Tuple[_prm.PrimitiveT, ...]):
     if len(t) == 0:
         return "tuple()"
     elif len(t) == 1:
@@ -27,21 +34,24 @@ def _str_primtuple(t):
         return f"({', '.join(_str_prim(p) for p in t)})"
 
 
-def _str_enclosure(enc):
-    return f"Enclosure({enc.spec})"
+def _str_enclosure(enc: Optional[_prp.Enclosure]):
+    return "None" if enc is None else f"Enclosure({enc.spec})"
 
 
-def _str_enclosures(encs):
+def _str_enclosures(encs: Tuple[Optional[_prp.Enclosure], ...]):
     if len(encs) == 1:
         return f"({_str_enclosure(encs[0])},)"
     else:
         return f"({','.join(_str_enclosure(enc) for enc in encs)})"
 
 
-class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
+class _PrimitiveGenerator(_dsp.PrimitiveDispatcher):
     def _Primitive(self, prim):
         return self._prim_object(prim)
 
+    def MinWidth(self, prim):
+        return self._prim_object(prim, add_name=False)
+
     def Spacing(self, prim):
         return self._prim_object(prim, add_name=False)
 
@@ -60,21 +70,14 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
         else:
             return f"prims += {class_name}({s_name})\n"
 
-    def _params_unhandled(self, prim):
+    def _params_unhandled(self, prim): # pragma: no cover
         raise RuntimeError(f"Internal error: unhandled params for {prim.__class__.__name__}")
 
-    def _params_mask(self, prim: prm._MaskPrimitive, *,
-        add_fill_space=False,
-    ):
+    def _params_designmask(self, prim: _prm.DesignMaskPrimitiveT):
         s = ""
         if prim.grid is not None:
             s += f"grid={prim.grid},"
-        if add_fill_space:
-            assert isinstance(prim.mask, msk.DesignMask)
-            if s:
-                s += " "
-            s += f"fill_space='{prim.mask.fill_space}',"
-        if isinstance(prim, prm._BlockageAttribute) and prim.blockage is not None:
+        if isinstance(prim, _prm.BlockageAttrPrimitiveT) and prim.blockage is not None:
             if s:
                 s += " "
             s += f"blockage={_str_prim(prim.blockage)},"
@@ -82,9 +85,7 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
 
         return s
 
-    def _params_widthspace(self, prim: prm._WidthSpacePrimitive, *,
-        add_fill_space=False,
-    ):
+    def _params_widthspace(self, prim: _prm.WidthSpacePrimitiveT):
         s = f"min_width={prim.min_width}, min_space={prim.min_space},\n"
         if prim.space_table is not None:
             s += "space_table=(\n"
@@ -96,27 +97,37 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
             s += f"min_density={prim.min_density},\n"
         if prim.max_density is not None:
             s += f"max_density={prim.max_density},\n"
-        if isinstance(prim, prm._PinAttribute) and prim.pin is not None:
-            s += f"pin={_str_primtuple(prim.pin)},\n"
-        s += self._params_mask(prim, add_fill_space=add_fill_space)
-        
+        if isinstance(prim, _prm.PinAttrPrimitiveT) and prim.pin is not None:
+            s += f"pin={_str_prim(prim.pin)},\n"
+
         return s
 
-    def _params_Marker(self, prim: prm.Marker):
-        return self._params_mask(prim)
+    def _params_widthspacedesignmask(self, prim: _prm.WidthSpaceDesignMaskPrimitiveT):
+        return self._params_widthspace(prim) + self._params_designmask(prim)
 
-    def _params_Auxiliary(self, prim: prm.Auxiliary):
-        return self._params_mask(prim)
+    def _params_Base(self, prim: _prm.Base):
+        lookup = {
+            _prm.nBaseT: "nBaseT",
+            _prm.pBaseT: "pBaseT",
+            _prm.undopedBaseT: "undopedBaseT",
+        }
+        return f"type_={lookup[prim.type_]}"
 
-    def _params_ExtraProcess(self, prim: prm.ExtraProcess):
-        return self._params_widthspace(prim, add_fill_space=True)
+    def _params_Marker(self, prim: _prm.Marker):
+        return self._params_designmask(prim)
 
-    def _params_Implant(self, prim: prm.Implant):
-        s = f"type_='{prim.type_}',\n"
-        s += self._params_widthspace(prim)
+    def _params_Auxiliary(self, prim: _prm.Auxiliary):
+        return self._params_designmask(prim)
+
+    def _params_ExtraProcess(self, prim: _prm.ExtraProcess):
+        return self._params_widthspacedesignmask(prim)
+
+    def _params_Implant(self, prim: _prm.Implant):
+        s = f"type_='{prim.type_.value}',\n"
+        s += self._params_widthspacedesignmask(prim)
         return s
 
-    def _params_Well(self, prim: prm.Well):
+    def _params_Well(self, prim: _prm.Well):
         if prim.min_space_samenet is not None:
             s = f"min_space_samenet={prim.min_space_samenet},\n"
         else:
@@ -124,19 +135,29 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
         s += self._params_Implant(prim)
         return s
 
-    def _params_Insulator(self, prim: prm.Insulator):
-        return self._params_widthspace(prim, add_fill_space=True)
+    def _params_DeepWell(self, prim: _prm.DeepWell):
+        return (
+            f"well={_str_prim(prim.well)},\n"
+            f"min_well_overlap={prim.min_well_overlap},\n"
+            f"min_well_enclosure={prim.min_well_enclosure},\n"
+        ) + self._params_Implant(prim)
 
-    def _params_GateWire(self, prim: prm.GateWire):
-        return self._params_widthspace(prim)
+    def _params_Insulator(self, prim: _prm.Insulator):
+        return self._params_widthspacedesignmask(prim)
 
-    def _params_MetalWire(self, prim: prm.MetalWire):
-        return self._params_widthspace(prim)
+    def _params_GateWire(self, prim: _prm.GateWire):
+        return self._params_widthspacedesignmask(prim)
 
-    def _params_TopMetalWire(self, prim: prm.TopMetalWire):
+    def _params_MetalWire(self, prim: _prm.MetalWire):
+        return self._params_widthspacedesignmask(prim)
+
+    def _params_MIMTop(self, prim: _prm.MIMTop):
         return self._params_MetalWire(prim)
 
-    def _params_WaferWire(self, prim: prm.WaferWire):
+    def _params_TopMetalWire(self, prim: _prm.TopMetalWire):
+        return self._params_MetalWire(prim)
+
+    def _params_WaferWire(self, prim: _prm.WaferWire):
         s = f"allow_in_substrate={prim.allow_in_substrate},\n"
         s += f"implant={_str_primtuple(prim.implant)},\n"
         s += f"min_implant_enclosure={_str_enclosures(prim.min_implant_enclosure)},\n"
@@ -151,17 +172,20 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
             )
         s += f"allow_well_crossing={prim.allow_well_crossing},\n"
         if prim.oxide is not None:
-            assert prim.min_well_enclosure is not None
+            assert prim.min_oxide_enclosure is not None
             s += (
                 f"oxide={_str_primtuple(prim.oxide)},\n"
                 f"min_oxide_enclosure={_str_enclosures(prim.min_oxide_enclosure)},\n"
             )
-        s += self._params_widthspace(prim)
+        s += self._params_widthspacedesignmask(prim)
         return s
 
-    def _params_Resistor(self, prim: prm.Resistor):
+    def _params_Resistor(self, prim: _prm.Resistor):
         s = f"wire={_str_prim(prim.wire)}, indicator={_str_primtuple(prim.indicator)},\n"
         s += f"min_indicator_extension={prim.min_indicator_extension},\n"
+        s += f"min_width={prim.min_width},\n"
+        s += f"min_length={prim.min_length},\n"
+        s += f"min_space={prim.min_space},\n"
         if prim.contact is not None:
             assert prim.min_contact_space is not None
             s += (
@@ -174,15 +198,24 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
                 s += f", min_implant_enclosure={_str_enclosure(prim.min_implant_enclosure)},\n"
             else:
                 s += ",\n"
-        if prim.model is not None:
-            s += f"model='{prim.model}', subckt_params={prim.subckt_params},\n"
-        if prim.sheetres is not None:
-            s += f"sheetres={prim.sheetres},\n"
-        s += self._params_widthspace(prim)
         return s
 
-    def _params_Diode(self, prim: prm.Diode):
+    def _params_MIMCapacitor(self, prim: _prm.MIMCapacitor):
+        return (
+            f"bottom={_str_prim(prim.bottom)},\n"
+            f"top={_str_prim(prim.top)},\n"
+            f"via={_str_prim(prim.via)},\n"
+            f"min_width={prim.min_width},\n"
+            f"min_bottom_top_enclosure={_str_enclosure(prim.min_bottom_top_enclosure)},\n"
+            f"min_bottomvia_space={prim.min_bottomvia_top_space},\n"
+            f"min_top_via_enclosure={_str_enclosure(prim.min_top_via_enclosure)},\n"
+            f"min_bottom_space={prim.min_bottom_space},\n"
+            f"min_top2bottom_space={prim.min_top2bottom_space},\n"
+        )
+
+    def _params_Diode(self, prim: _prm.Diode):
         s = f"wire={_str_prim(prim.wire)}, indicator={_str_primtuple(prim.indicator)},\n"
+        s += f"min_width={prim.min_width},\n"
         s += f"min_indicator_enclosure={_str_enclosures(prim.min_indicator_enclosure)},\n"
         s += f"implant={_str_prim(prim.implant)}"
         if prim.min_implant_enclosure is not None:
@@ -193,33 +226,45 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
             if prim.min_well_enclosure is not None:
                 s += f", min_well_enclosure={_str_enclosure(prim.min_well_enclosure)}"
             s += ",\n"
-        if prim.model is not None:
-            s += f"model='{prim.model}',\n"
-        s += self._params_widthspace(prim)
         return s
-    
-    def _params_Via(self, prim: prm.Via):
+
+    def _params_Via(self, prim: _prm.Via):
         s = f"bottom={_str_primtuple(prim.bottom)},\n"
         s += f"top={_str_primtuple(prim.top)},\n"
         s += f"width={prim.width}, min_space={prim.min_space},\n"
         s += f"min_bottom_enclosure={_str_enclosures(prim.min_bottom_enclosure)},\n"
         s += f"min_top_enclosure={_str_enclosures(prim.min_top_enclosure)},\n"
-        s += self._params_mask(prim)
+        s += self._params_designmask(prim)
         return s
 
-    def _params_PadOpening(self, prim: prm.PadOpening):
+    def _params_PadOpening(self, prim: _prm.PadOpening):
         s = f"bottom={_str_prim(prim.bottom)},\n"
         s += f"min_bottom_enclosure={_str_enclosure(prim.min_bottom_enclosure)},\n"
-        s += self._params_widthspace(prim)
+        s += self._params_widthspacedesignmask(prim)
         return s
 
-    def _params_Spacing(self, prim: prm.Spacing):
+    def _params_MinWidth(self, prim: _prm.MinWidth):
+        s = f"prim={_str_prim(prim.prim)},\n"
+        s += f"min_width={prim.min_width},\n"
+        return s
+
+    def _params_Spacing(self, prim: _prm.Spacing):
         s = f"primitives1={_str_primtuple(prim.primitives1)},\n"
-        s += f"primitives2={_str_primtuple(prim.primitives2)},\n"
+        if prim.primitives2 is not None:
+            s += f"primitives2={_str_primtuple(prim.primitives2)},\n"
         s += f"min_space={prim.min_space},\n"
         return s
 
-    def _params_MOSFETGate(self, prim: prm.MOSFETGate):
+    def _params_Enclosure(self, prim: _prm.Enclosure):
+        return dedent(f"""
+            prim={_str_prim(prim.prim)}, by={_str_prim(prim.by)},
+            min_enclosure={_str_enclosure(prim.min_enclosure)},
+        """[1:])
+
+    def _params_NoOverlap(self, prim: _prm.NoOverlap):
+        return f"prim1={_str_prim(prim.prim1)}, prim2={_str_prim(prim.prim2)},\n"
+
+    def _params_MOSFETGate(self, prim: _prm.MOSFETGate):
         s = f"active={_str_prim(prim.active)}, poly={_str_prim(prim.poly)},\n"
         if prim.oxide is not None:
             s += f"oxide={_str_prim(prim.oxide)},\n"
@@ -249,7 +294,7 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
             s += f"contact={_str_prim(prim.contact)}, min_contactgate_space={prim.min_contactgate_space},\n"
         return s
 
-    def _params_MOSFET(self, prim: prm.MOSFET):
+    def _params_MOSFET(self, prim: _prm.MOSFET):
         s = f"gate={_str_prim(prim.gate)},\n"
         if prim.implant is not None:
             s += f"implant={_str_primtuple(prim.implant)},\n"
@@ -271,69 +316,31 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
             s += f"min_gate_space={prim.min_gate_space},\n"
         if prim.contact is not None:
             s += f"contact={_str_prim(prim.contact)}, min_contactgate_space={prim.min_contactgate_space},\n"
-        if prim.model is not None:
-            s += f"model='{prim.model}',\n"
         return s
 
-    def _params_Bipolar(self, prim: prm.Bipolar):
+    def _params_Bipolar(self, prim: _prm.Bipolar):
         s = f""
 
+
 class PDKMasterGenerator:
     def __call__(self, obj):
-        if isinstance(obj, tch.Technology):
+        if isinstance(obj, _tch.Technology):
             return self._gen_tech(obj)
-        elif isinstance(obj, ckt._Circuit):
+        elif isinstance(obj, _ckt._Circuit):
             return self._gen_ckt(obj)
-        elif isinstance(obj, lib.Library):
+        elif isinstance(obj, _lbry.Library):
             return self._gen_lib(obj)
         else:
             raise TypeError("obj has to be of type 'Technology' or '_Circuit'")
 
-    def _gen_header(self, tech, cktfab, layoutfab):
-        assert (
-            isinstance(tech, tch.Technology)
-            and isinstance(cktfab, ckt.CircuitFactory)
-        ), "Internal error"
-
+    def _gen_header(self, tech: _tch.Technology):
         s = "# Autogenerated file. Changes will be overwritten.\n\n"
 
-        mod = tech.__class__.__module__
-        mod_split = mod.split(".")
-        module: Any
-        if mod.startswith("pdkmaster.techs"):
-            module = import_module(".".join(mod_split[:3]))
-        else:
-            module = None
-
-        if module and hasattr(module, "tech") and (tech == module.tech):
-            s += f"from {module.__name__} import tech\n"
-        else:
-            mod_name = type(tech).__module__
-            class_name = type(tech).__name__
-            s += (
-                f"from {mod_name} import {class_name}\n"
-                f"tech = {class_name}()\n"
-            )
-
-        if cktfab is not None:
-            if module and hasattr(module, "cktfab") and (cktfab == module.cktfab):
-                s += f"from {module.__name__} import cktfab\n"
-            else:
-                s += dedent(f"""
-                    from pdkmaster.design.circuit import CircuitFactory
-                    cktfab = CircuitFactory(tech)
-                """[1:])
-
-        if layoutfab is not None:
-            if (module and hasattr(module, "layoutfab")
-                and (layoutfab == module.layoutfab)
-            ):
-                s += f"from {module.__name__} import layoutfab\n"
-            else:
-                s += dedent(f"""
-                    from pdkmaster.design.layout import LayoutFactory
-                    layoutfab = LayoutFactory(tech)
-                """[1:])
+        class_name = tech.__class__.__name__
+        s += (
+            f"from {tech.__class__.__module__} import {class_name}\n"
+            f"tech = {class_name}()\n"
+        )
 
         s += "\n"
 
@@ -348,7 +355,7 @@ class PDKMasterGenerator:
             from pdkmaster.technology.technology_ import Technology
             from pdkmaster.design.layout import LayoutFactory
             from pdkmaster.design.circuit import CircuitFactory
-            
+
             __all__ = [
                 "technology", "tech",
                 "layoutfab", "layout_factory",
@@ -358,11 +365,10 @@ class PDKMasterGenerator:
             class {tech.name}(Technology):
                 name = "{tech.name}"
                 grid = {tech.grid}
-                substrate_type = "{tech.substrate_type}"
-            
+
                 def _init(self):
                     prims = self._primitives
-            
+
         """[1:])
         gen = _PrimitiveGenerator()
         s += indent(
@@ -381,10 +387,10 @@ class PDKMasterGenerator:
     def _gen_ckt(self, circuit, *, lib=None, header=True):
         s = ""
         if header:
-            s += self._gen_header(circuit.fab.tech, circuit.fab, None)
+            s += self._gen_header(circuit.fab.tech)
 
             s += dedent(f"""
-                __all__ ["circuit", "ckt"]
+                __all__ = ["circuit", "ckt"]
 
                 circuit = ckt = cktfab.new_circuit("{circuit.name}")
 
@@ -396,13 +402,13 @@ class PDKMasterGenerator:
         s += "\n"
 
         for inst in circuit.instances:
-            if isinstance(inst, ckt._PrimitiveInstance):
+            if isinstance(inst, _ckt._PrimitiveInstance):
                 primname = inst.prim.name
                 s += f"ckt.instantiate(tech.primitives['{primname}'], name='{inst.name}',\n"
                 for param in inst.prim.params:
                     s += f"    {param.name}={inst.params[param.name]!r},\n"
                 s += ")\n"
-            elif isinstance(inst, ckt._CellInstance):
+            elif isinstance(inst, _ckt._CellInstance):
                 if lib is None:
                     raise ValueError(
                         "Can't export single Circuit with Cell instances outside library export"
@@ -419,19 +425,20 @@ class PDKMasterGenerator:
                     f", circuitname={inst.circuitname}"
                 )
                 s += ")\n"
-            else:
+            else: # pragma: no cover
                 raise AssertionError("Internal error: unsupported instance class")
 
         s += "\n"
 
         for net in circuit.nets:
-            s += f"ckt.nets['{net.name}'].childports += (\n"
-            for port in net.childports:
-                inst = port.inst
-                s += (
-                    f"    ckt.instances['{inst.name}'].ports['{port.name}'],\n"
-                )
-            s += ")\n"
+            if len(net.childports) > 0:
+                s += f"ckt.nets['{net.name}'].childports += (\n"
+                for port in net.childports:
+                    inst = port.inst
+                    s += (
+                        f"    ckt.instances['{inst.name}'].ports['{port.name}'],\n"
+                    )
+                s += ")\n"
 
         return s
 
@@ -446,7 +453,7 @@ class PDKMasterGenerator:
 
             """[1:])
 
-            s += self._gen_header(library.tech, library.cktfab, library.layoutfab)
+            s += self._gen_header(library.tech)
 
             s += dedent("""
                 __all__ = ["library", "lib"]
@@ -460,7 +467,7 @@ class PDKMasterGenerator:
                 # cell: {cell.name}
                 cell = lib.new_cell(name='{cell.name}')
             """)
-            for circuit in cell.circuits:
+            for circuit in cell._circuits:
                 s += f"ckt = cell.new_circuit('{circuit.name}')\n"
                 s += self._gen_ckt(circuit, lib=library, header=False)
 
