@@ -1,31 +1,31 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 import abc
 from typing import Tuple, Iterable, Optional, ClassVar, cast
 
 from ..typing import MultiT, cast_MultiT
-from . import rule as rle, property_ as prp
+from . import rule as _rle, property_ as _prp
 
 
-__all__ = ["DesignMask", "Join", "Intersect"]
+__all__ = ["MaskT", "RuleMaskT", "DesignMask", "Join", "Intersect"]
 
 
-class _MaskProperty(prp.Property):
-    """`_MaskProperty` is a `Property` object on a single _Mask object.
+class _MaskProperty(_prp._Property):
+    """`_MaskProperty` is a `Property` object on a single `MaskT` object.
     
     Typical examples are width, area, density etc.
     """
-    def __init__(self, *, mask: "_Mask", name: str):
-        super().__init__(mask.name + "." + name)
+    def __init__(self, *, mask: "MaskT", name: str):
+        super().__init__(name=(mask.name + "." + name))
         self.mask = mask
         self.prop_name = name
 
 
-class _DualMaskProperty(prp.Property):
-    """`_MaskProperty` if a `Property` object on two _Mask objects.
+class _DualMaskProperty(_prp._Property):
+    """`_MaskProperty` if a `Property` object on two `MaskT` objects.
     
     Typical example is the spacing or overlap between two masks.
     """
-    def __init__(self, *, mask1: "_Mask", mask2: "_Mask", name: str, commutative: bool):
+    def __init__(self, *, mask1: "MaskT", mask2: "MaskT", name: str, commutative: bool):
         if commutative:
             supername = f"{name}({mask1.name},{mask2.name})"
         else:
@@ -37,10 +37,10 @@ class _DualMaskProperty(prp.Property):
         self.prop_name = name
 
 
-class _DualMaskEnclosureProperty(prp.EnclosureProperty):
+class _DualMaskEnclosureProperty(_prp._EnclosureProperty):
     """`_DualMaskProperty` is a `Property` object with on two `_Mask` objects with
     an `Enclosure` object as value."""
-    def __init__(self, *, mask1: "_Mask", mask2: "_Mask", name: str):
+    def __init__(self, *, mask1: "MaskT", mask2: "MaskT", name: str):
         super().__init__(name=f"{mask1.name}.{name}({mask2.name})")
 
         self.mask1 = mask1
@@ -48,7 +48,7 @@ class _DualMaskEnclosureProperty(prp.EnclosureProperty):
         self.prop_name = name
 
 
-class _MultiMaskCondition(rle._Condition):
+class _MultiMaskCondition(_rle._Condition):
     """_MultiMaskCondition is a `_Condition` object involving multiple masks.
 
     This class is a base class that needs to be subclassed with a `str` value given
@@ -58,7 +58,7 @@ class _MultiMaskCondition(rle._Condition):
     """
     operation: ClassVar[str]
 
-    def __init__(self, *, mask: "_Mask", others: MultiT["_Mask"]):
+    def __init__(self, *, mask: "MaskT", others: MultiT["MaskT"]):
         try:
             self.operation
         except AttributeError:
@@ -67,15 +67,15 @@ class _MultiMaskCondition(rle._Condition):
             )
         others = cast_MultiT(others)
         super().__init__(elements=(mask, others))
-        self._elements: Tuple["_Mask", Tuple["_Mask"]]
+        self._elements: Tuple["MaskT", Tuple["MaskT"]]
 
         self._hash: Optional[int] = None
 
     @property
-    def mask(self) -> "_Mask":
+    def mask(self) -> "MaskT":
         return self._elements[0]
     @property
-    def others(self) -> Tuple["_Mask"]:
+    def others(self) -> Tuple["MaskT"]:
         return self._elements[1]
 
     def __eq__(self, other: object) -> bool:
@@ -85,12 +85,12 @@ class _MultiMaskCondition(rle._Condition):
             other = cast(_MultiMaskCondition, other)
             return set((self.mask, *self.others)) == set((other.mask, *other.others))
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         if self._hash is None:
             self._hash = hash(tuple(sorted(m.name for m in (self.mask, *self.others))))
         return self._hash
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "{}.{}({})".format(
             repr(self.mask), self.operation,
             ",".join(repr(mask) for mask in self.others),
@@ -125,50 +125,50 @@ class _Mask(abc.ABC):
     @abc.abstractmethod
     def __init__(self, *, name: str):
         self.name = name
-        self.width = _MaskProperty(mask=self, name="width")
-        self.length = _MaskProperty(mask=self, name="length")
-        self.space = _MaskProperty(mask=self, name="space")
-        self.area = _MaskProperty(mask=self, name="area")
-        self.density = _MaskProperty(mask=self, name="density")
+        self.width: _prp.PropertyT = _MaskProperty(mask=self, name="width")
+        self.length: _prp.PropertyT = _MaskProperty(mask=self, name="length")
+        self.space: _prp.PropertyT = _MaskProperty(mask=self, name="space")
+        self.area: _prp.PropertyT = _MaskProperty(mask=self, name="area")
+        self.density: _prp.PropertyT = _MaskProperty(mask=self, name="density")
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.name
 
-    def extend_over(self, other: "_Mask") -> prp.Property:
+    def extend_over(self, other: "MaskT") -> _prp.PropertyT:
         """Returns a `Property` object representing the extension of one
-        the shapes on one `_Mask` object over the shapes on another `_Mask`
+        the shapes on one `MaskT` object over the shapes on another `MaskT`
         object.
         """
         return _DualMaskProperty(
             mask1=self, mask2=other, name="extend_over", commutative=False,
         )
 
-    def enclosed_by(self, other: "_Mask") -> prp.EnclosureProperty:
+    def enclosed_by(self, other: "MaskT") -> _prp.EnclosurePropertyT:
         """Returns a `EnclosureProperty` object representing the enclosure of one
-        the shapes on one `_Mask` object by the shapes on another `_Mask`
+        the shapes on one `MaskT` object by the shapes on another `MaskT`
         object.
         """
         return _DualMaskEnclosureProperty(mask1=self, mask2=other, name="enclosed_by")
 
-    def is_inside(self, other: MultiT["_Mask"], *others: "_Mask") -> rle._Condition:
-        """Returns a `_Condition` object representing wether all the shapes on a '_Mask`
-        object are inside one of the other `_Mask` objects.
+    def is_inside(self, other: MultiT["MaskT"], *others: "MaskT") -> _rle.ConditionT:
+        """Returns a `_Condition` object representing wether all the shapes on a 'MaskT`
+        object are inside one of the other `MaskT` objects.
         """
         masks = (*cast_MultiT(other), *others)
 
         return _InsideCondition(mask=self, others=masks)
 
-    def is_outside(self, other: MultiT["_Mask"], *others: "_Mask") -> rle._Condition:
-        """Returns a `_Condition` object representing wether all the shapes on a '_Mask`
-        object are outside any of the other `_Mask` objects.
+    def is_outside(self, other: MultiT["MaskT"], *others: "MaskT") -> _rle.ConditionT:
+        """Returns a `_Condition` object representing wether all the shapes on a 'MaskT`
+        object are outside any of the other `MaskT` objects.
         """
         masks = (*cast_MultiT(other), *others)
 
         return _OutsideCondition(mask=self, others=masks)
 
-    def parts_with(self, condition: MultiT[prp._Comparison]) -> "_Mask":
-        """Returns a derived `_Mask` representing the parts of the shapes on
-        the `_Mask` object that fulfill the given condition(s).  
+    def parts_with(self, condition: MultiT[_prp.ComparisonT]) -> "MaskT":
+        """Returns a derived `MaskT` representing the parts of the shapes on
+        the `MaskT` object that fulfill the given condition(s).  
         The condition may only use properties from the same mask on which one
         calls the `parts_with` method.
 
@@ -176,26 +176,26 @@ class _Mask(abc.ABC):
         """
         return _PartsWith(mask=self, condition=condition)
 
-    def remove(self, what: "_Mask") -> "_Mask":
-        """Returns a derived `_Mask` representing the parts of the shapes on
-        the `_Mask` that don't overlap with shapes of the other `_Mask` object.
+    def remove(self, what: "MaskT") -> "MaskT":
+        """Returns a derived `MaskT` representing the parts of the shapes on
+        the `MaskT` that don't overlap with shapes of the other `MaskT` object.
         """
         return _MaskRemove(from_=self, what=what)
 
-    def alias(self, name: str) -> "_MaskAlias":
-        """Returns a derived `_Mask` given an alias for another `_Mask` object.
+    def alias(self, name: str) -> "RuleMaskT":
+        """Returns a derived `MaskT` given an alias for another `MaskT` object.
         The return object is also a `_Rule` object in order for scripts that
         are generated from rules can define a variable representing the
-        `_Mask` that has been aliased. Typically the variable name will be
+        `MaskT` that has been aliased. Typically the variable name will be
         the alias name and further on in generated rules then this variable
-        will be used where this alias is used in other derived `_Mask` object
+        will be used where this alias is used in other derived `MaskT` object
         or properties.
         """
         return _MaskAlias(name=name, mask=self)
 
     @property
-    def same_net(self) -> "_SameNet":
-        """Returns a derived `_Mask` representing the shapes on a `_Mask` that
+    def same_net(self) -> "MaskT":
+        """Returns a derived `MaskT` representing the shapes on a `MaskT` that
         are on the same net. It's thus connectivity related as defined by the
         `Connect` object.  
         The derived mask actually is a collection of separate masks for each
@@ -209,8 +209,8 @@ class _Mask(abc.ABC):
 
     @abc.abstractproperty
     def designmasks(self) -> Iterable["DesignMask"]: # pragma: no cover
-        """The designasks property of a `_Mask` object gives a list of
-        all designamsks used in a `_Mask`.
+        """The designasks property of a `MaskT` object gives a list of
+        all designamsks used in a `MaskT`.
 
         API Notes:
             * The returned Iterable may contain same `DesignMask` object multiple
@@ -228,27 +228,28 @@ class _Mask(abc.ABC):
     def __hash__(self) -> int:
         # Assume mask names are different so will also give different hash
         return hash(self.name)
+MaskT = _Mask
 
 
-class DesignMask(_Mask, rle._Rule):
+class _RuleMask(_Mask, _rle._Rule):
+    "A `MaskT` object that is also a `RuleT` object"
+    pass
+RuleMaskT = _RuleMask
+
+
+class DesignMask(_RuleMask):
     """A `DesignMask` object is a `_Mask` object with the shapes on the mask
     provided by shapes by the user. It is not a derived mask.
 
     Arguments:
         name: the name of the mask
-        fill_space: wether space between two shapes may be filled up.
-            Value has to be one of ("no", "same_net", "yes")
     """
-    def __init__(self, *, name: str, fill_space: str):
+    def __init__(self, *, name: str):
         super().__init__(name=name)
 
-        if not fill_space in ("no", "same_net", "yes"):
-            raise ValueError("fill_space has to be one of ('no', 'same_net', 'yes')")
-        self.fill_space = fill_space
+        self.grid: _prp.PropertyT = _MaskProperty(mask=self, name="grid")
 
-        self.grid = _MaskProperty(mask=self, name="grid")
-
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"design({self.name})"
 
     @property
@@ -259,7 +260,6 @@ class DesignMask(_Mask, rle._Rule):
         return (
             isinstance(other, DesignMask)
             and (self.name == other.name)
-            and (self.fill_space == other.fill_space)
         )
 
     def __hash__(self) -> int:
@@ -269,7 +269,7 @@ class DesignMask(_Mask, rle._Rule):
 class _PartsWith(_Mask):
     """`_Mask.parts_with()` support class"""
     def __init__(self, *,
-        mask: _Mask, condition: MultiT[prp._Comparison],
+        mask: MaskT, condition: MultiT[_prp.ComparisonT],
     ):
         self.mask = mask
 
@@ -312,7 +312,7 @@ class Join(_Mask):
     """A derived `_Mask` object that represenet the shapes resulting of joining
     all the shapes of the provided masks.
     """
-    def __init__(self, masks: MultiT[_Mask]):
+    def __init__(self, masks: MultiT[MaskT]):
         self.masks = masks = cast_MultiT(masks)
 
         super().__init__(name="join({})".format(",".join(mask.name for mask in masks)))
@@ -345,7 +345,7 @@ class Intersect(_Mask):
     """A derived `_Mask` object that represenet the shapes resulting of
     the intersection of all the shapes of the provided masks.
     """
-    def __init__(self, masks: MultiT[_Mask]):
+    def __init__(self, masks: MultiT[MaskT]):
         self.masks = masks = cast_MultiT(masks)
 
         super().__init__(name="intersect({})".format(",".join(mask.name for mask in masks)))
@@ -375,7 +375,7 @@ class Intersect(_Mask):
 
 class _MaskRemove(_Mask):
     """`_Mask.remove()` support class"""
-    def __init__(self, *, from_: _Mask, what: _Mask):
+    def __init__(self, *, from_: MaskT, what: MaskT):
         super().__init__(name=f"{from_.name}.remove({what.name})")
         self.from_ = from_
         self.what = what
@@ -399,14 +399,14 @@ class _MaskRemove(_Mask):
         return hash((self.from_, self.what))
 
 
-class _MaskAlias(_Mask, rle._Rule):
+class _MaskAlias(_RuleMask):
     """`_Mask.alias()` support class"""
-    def __init__(self, *, name: str, mask: _Mask):
+    def __init__(self, *, name: str, mask: MaskT):
         self.mask = mask
 
         super().__init__(name=name)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"{self.mask.name}.alias({self.name})"
 
     @property
@@ -431,31 +431,19 @@ class Spacing(_DualMaskProperty):
     """A `Spacing` object is a `Property` that represents the spacing
     between two shapes on two different masks.  
     The masks may not be the same. For the space between shapes on the same
-    mask use the `_Mask.space` property.
+    mask use the `MaskT.space` property.
     """
-    def __init__(self, mask1: _Mask, mask2: _Mask):
+    def __init__(self, mask1: MaskT, mask2: MaskT):
         if mask1 == mask2:
             raise ValueError(
                 f"mask1 and mask2 may not be the same for 'Spacing'\n"
-                "use `_Mask.space` property for that"
+                "use `MaskT.space` property for that"
             )
         super().__init__(mask1=mask1, mask2=mask2, name="space", commutative=True)
 
 
-class OverlapWidth(_DualMaskProperty):
-    """A `OverlapWidth` object is a `Property` that represents the spacing
-    between two shapes on two different masks.
-    """
-    def __init__(self, mask1: _Mask, mask2: _Mask):
-        if mask1 == mask2:
-            raise ValueError(
-                f"mask1 and mask2 may not be the same for 'OverlapWidth'\n"
-            )
-        super().__init__(mask1=mask1, mask2=mask2, name="overlapwidth", commutative=True)
-
-
-class Connect(rle._Rule):
-    """A `Connect` objext is a `_Rule` indicating that overlapping shapes on two
+class Connect(_rle._Rule):
+    """A `Connect` object is a `_Rule` indicating that overlapping shapes on two
     different layers are connecting with each other. This rule is base to determine
     connectivity and which shapes are on the same net.
 
@@ -471,7 +459,7 @@ class Connect(rle._Rule):
     the masks from `mask2` is connecting to it.
     """
     def __init__(self,
-        mask1: MultiT[_Mask], mask2: MultiT[_Mask],
+        mask1: MultiT[MaskT], mask2: MultiT[MaskT],
     ):
         self.mask1 = mask1 = cast_MultiT(mask1)
         self.mask2 = mask2 = cast_MultiT(mask2)
@@ -495,12 +483,12 @@ class Connect(rle._Rule):
                 or ((masks1 == others2) and (masks2 == others1))
             )
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         if self._hash is None:
             self._hash = hash(tuple(sorted(m.name for m in (*self.mask1, *self.mask2))))
         return self._hash
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         s1 = self.mask1[0].name if len(self.mask1) == 1 else "({})".format(
             ",".join(m.name for m in self.mask1)
         )
@@ -512,7 +500,7 @@ class Connect(rle._Rule):
 
 class _SameNet(_Mask):
     """`_Mask.same_net()` support class"""
-    def __init__(self, mask: _Mask):
+    def __init__(self, mask: MaskT):
         self.mask = mask
 
         super().__init__(name=f"same_net({mask.name})")
