@@ -1,13 +1,11 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 from math import floor, ceil
 import abc
 from typing import Optional, Union, Iterable, cast, overload
 
-from .. import _util
-from ..typing import IntFloat
 from . import (
-    property_ as prp, rule as rle, mask as msk, wafer_ as wfr, geometry as geo,
-    primitive as prm,
+    property_ as _prp, rule as _rle, mask as _msk, wafer_ as _wfr, geometry as _geo,
+    primitive as _prm,
 )
 
 
@@ -22,12 +20,14 @@ class Technology(abc.ABC):
     the `Technology` class is an abstract base class so subclasses need to
     implement some of the abstract methods defined in this base class.
 
-    The setup of the primitives for a technology is done in the `_init()` method.
-    Typically subclasses should not overload the `__init__()` method.
+    Subclasses need to overload the `__init__()` method and call the parent's
+    `__init__()` with the list of the primitives for this technology. After this
+    list has been passed to the super class's `__init__()` it is final and will
+    be frozen.
 
-    Next to the `_init()` abstract method subclasses also need to define some
+    Next to the `__init__()` abstract method subclasses also need to define some
     abstract properties to define some properties of the technology. These are
-    `name()`, `grid()` and `substrate_type()`.
+    `name()`, `grid()`.
     """
     class ConnectionError(Exception):
         pass
@@ -43,16 +43,16 @@ class Technology(abc.ABC):
 
         @overload
         def min_space(self,
-            primitive1: "prm._MaskPrimitive", primitive2: None=None,
+            primitive1: "_prm.MaskPrimitiveT", primitive2: None=None,
         ) -> float:
             ... # pragma: no cover
         @overload
         def min_space(self,
-            primitive1: "prm._Primitive", primitive2: "prm._Primitive",
+            primitive1: "_prm.PrimitiveT", primitive2: "_prm.PrimitiveT",
         ) -> float:
             ... # pragma: no cover
         def min_space(self,
-            primitive1: "prm._Primitive", primitive2: Optional["prm._Primitive"]=None,
+            primitive1: "_prm.PrimitiveT", primitive2: Optional["_prm.PrimitiveT"]=None,
         ) -> float:
             """Compute the minimum space between one or two primitives. It will go
             over all the rules to determine the minimum space for the provided
@@ -68,7 +68,7 @@ class Technology(abc.ABC):
                     if specified the minimum space between shape on layer primitive1
                     and shapes on layer primitive2 is returned.
             """
-            def check_prim(prim: prm.Spacing) -> bool:
+            def check_prim(prim: _prm.Spacing) -> bool:
                 if primitive2 is None:
                     return (
                         (primitive1 in prim.primitives1)
@@ -90,12 +90,12 @@ class Technology(abc.ABC):
 
             spaces = list(
                 p.min_space for p in filter(
-                    check_prim, self.tech.primitives.__iter_type__(prm.Spacing),
+                    check_prim, self.tech.primitives.__iter_type__(_prm.Spacing),
                 )
             )
             if (primitive2 is None) or (primitive1 == primitive2):
                 try:
-                    space = cast(prm._WidthSpacePrimitive, primitive1).min_space
+                    space = cast(_prm.WidthSpacePrimitiveT, primitive1).min_space
                 except AttributeError: # pragma: no cover
                     pass
                 else:
@@ -108,9 +108,9 @@ class Technology(abc.ABC):
                     f"min_space between {primitive1} and {primitive2} not found",
                 )
 
-        def min_width(self, primitive: "prm._WidthSpacePrimitive", *,
+        def min_width(self, primitive: "_prm.WidthSpacePrimitiveT", *,
             up: bool=False, down: bool=False, min_enclosure: bool=False,
-        ):
+        ) -> float:
             """Compute the minimum width of a primitive.
             This method allows to take into account via enclosure rules to compute
             minimum width but still be contacted through a via.
@@ -138,7 +138,7 @@ class Technology(abc.ABC):
                     enc = via.min_top_enclosure[idx]
                     w = via.width
                 else:
-                    enc = prp.Enclosure(0.0)
+                    enc = _prp.Enclosure(0.0)
                     w = 0.0
 
                 enc = enc.min() if min_enclosure else enc.max()
@@ -146,12 +146,12 @@ class Technology(abc.ABC):
 
             return max((
                 primitive.min_width,
-                *(wupdown(via) for via in self.tech.primitives.__iter_type__(prm.Via)),
+                *(wupdown(via) for via in self.tech.primitives.__iter_type__(_prm.Via)),
             ))
 
-        def min_pitch(self, primitive: "prm._WidthSpacePrimitive", *,
+        def min_pitch(self, primitive: "_prm.WidthSpacePrimitiveT", *,
             up: bool=False, down: bool=False, min_enclosure: bool=False,
-        ):
+        ) -> float:
             """Compute the minimum pitch of a primitive.
             It's the minimum width plus the minimum space.
 
@@ -180,38 +180,27 @@ class Technology(abc.ABC):
         Optionally primitives may define a bigger grid for their shapes.
         """
         ... # pragma: no cover
-    @property
+
     @abc.abstractmethod
-    def substrate_type(self) -> str:
-        """property with the substrate type of the technology
+    def __init__(self, *, primitives: "_prm.Primitives"):
+        self._primitives = primitives
 
-        It has to be "n", "p" or "undoped".
-        """
-        ... # pragma: no cover
-
-    def __init__(self):
-        self._init_done = False
-
-        if not self.substrate_type in ("n", "p", "undoped"):
-            raise ValueError(
-                "substrate_type Technology class attribute has to be 'n', 'p' or 'undoped'",
-            )
-
-        self._primitives = prims = prm.Primitives()
-
-        self._init()
-
-        self._init_done = True
-        self._substrate = None
+        well_masks = tuple(
+            prim.mask for prim in self._primitives.__iter_type__(_prm.Well)
+        )
+        if not well_masks:
+            self._substrate = _wfr.wafer
+        else:
+            self._substrate = _wfr.outside(well_masks, alias=f"substrate:{self.name}")
 
         self._build_interconnect()
         self._build_rules()
 
-        prims._freeze_()
+        primitives._freeze_()
 
         self.computed = self._ComputedSpecs(self)
 
-    def is_ongrid(self, v: float, mult: int=1) -> float:
+    def is_ongrid(self, v: float, mult: int=1) -> bool:
         """Returns wether a value is on grid or not.
 
         Arguments:
@@ -220,21 +209,21 @@ class Technology(abc.ABC):
         """
         g = mult*self.grid
         ng = round(v/g)
-        return abs(v - ng*g) < geo.epsilon
+        return abs(v - ng*g) < _geo.epsilon
 
     @overload
     def on_grid(self,
-        dim: IntFloat, *, mult: int=1, rounding: str="nearest",
+        dim: float, *, mult: int=1, rounding: str="nearest",
     ) -> float:
         ... # pragma: no cover
     @overload
     def on_grid(self,
-        dim: geo.Point, *, mult: int=1, rounding: str="nearest",
-    ) -> geo.Point:
+        dim: _geo.Point, *, mult: int=1, rounding: str="nearest",
+    ) -> _geo.Point:
         ... # pragma: no cover
     def on_grid(self,
-        dim: Union[IntFloat, geo.Point], *, mult: int=1, rounding: str="nearest",
-    ) -> Union[float, geo.Point]:
+        dim: Union[float, _geo.Point], *, mult: int=1, rounding: str="nearest",
+    ) -> Union[float, _geo.Point]:
         """Compute a value on grid from a given value.
 
         Arguments:
@@ -244,16 +233,15 @@ class Technology(abc.ABC):
                 Has to be one of "floor", "nearest", "ceiling"; "nearest" is the
                 default.
         """
-        if isinstance(dim, geo.Point):
-            return geo.Point(
+        if isinstance(dim, _geo.Point):
+            return _geo.Point(
                 x=self.on_grid(dim.x, mult=mult, rounding=rounding),
                 y=self.on_grid(dim.y, mult=mult, rounding=rounding),
             )
-        dim = _util.i2f(dim)
         if rounding == "floor":
-            dim += geo.epsilon
+            dim += _geo.epsilon
         elif rounding == "ceiling":
-            dim -= geo.epsilon
+            dim -= _geo.epsilon
         else:
             if rounding != "nearest":
                 raise ValueError(
@@ -268,6 +256,10 @@ class Technology(abc.ABC):
 
         return f(dim/(mult*self.grid))*mult*self.grid
 
+    @property
+    def base(self) -> "_prm.Base":
+        return cast(_prm.Base, self.primitives.base)
+
     @property
     def dbu(self) -> float:
         """Returns database unit compatible with technology grid. An exception is
@@ -284,20 +276,7 @@ class Technology(abc.ABC):
         else:
             return 1e-3
 
-    @abc.abstractmethod
-    def _init(self):
-        """Abstract method that needs to setup the primitives.
-
-        When entering this function the object's `_primitives` attribute will be empty
-        and upon return from this method this attribute will have to be filled with
-        all the primitives of the given technology.
-
-        After the initialization has called this function it will perform connectivity
-        and other consistency checks on the provided list of primitives.
-        """
-        ... # pragma: no cover
-
-    def _build_interconnect(self):
+    def _build_interconnect(self) -> None:
         prims = self._primitives
 
         neworder = []
@@ -312,6 +291,17 @@ class Technology(abc.ABC):
         def get_name(prim):
             return prim.name
 
+        # primitives needs to contain exectly one Base primitive which
+        # is always called base
+        try:
+            base = prims.base
+        except:
+            raise ValueError("A technology needs exactly one 'Base` primitive")
+        else:
+            if not isinstance(base, _prm.Base):
+                raise ValueError("A technology needs exactly one 'Base` primitive")
+        add_prims((base,))
+
         # set that are build up when going over the primitives
         # bottomwires: primitives that still need to be bottomconnected by a via
         bottomwires = set()
@@ -319,25 +309,25 @@ class Technology(abc.ABC):
         implants = set() # Implants to add
         markers = set() # Markers to add
         # the wells, fixed
-        deepwells = set(prims.__iter_type__(prm.DeepWell))
-        wells = set(prims.__iter_type__(prm.Well))
+        deepwells = set(prims.__iter_type__(_prm.DeepWell))
+        wells = set(prims.__iter_type__(_prm.Well))
 
         # Wells are the first primitives in line
         add_prims(sorted(deepwells, key=get_name))
         add_prims(sorted(wells, key=get_name))
 
         # process waferwires
-        waferwires = set(prims.__iter_type__(prm.WaferWire))
+        waferwires = set(prims.__iter_type__(_prm.WaferWire))
         bottomwires.update(waferwires) # They also need to be connected
         conn_wells = set()
         for wire in waferwires:
             implants.update((*wire.implant, *wire.well))
             conn_wells.update(wire.well)
         if conn_wells != wells:
-            raise prm.UnconnectedPrimitiveError((wells - conn_wells).pop())
+            raise _prm.UnconnectedPrimitiveError(primitive=(wells - conn_wells).pop())
 
         # process gatewires
-        bottomwires.update(prims.__iter_type__(prm.GateWire))
+        bottomwires.update(prims.__iter_type__(_prm.GateWire))
 
         # Already add implants that are used in the waferwires
         add_prims(sorted(implants, key=get_name))
@@ -349,15 +339,15 @@ class Technology(abc.ABC):
                 add_prims(sorted(ww.oxide, key=get_name))
 
         # process vias
-        vias = set(prims.__iter_type__(prm.Via))
+        vias = set(prims.__iter_type__(_prm.Via))
 
         def allwires(wire):
-            if isinstance(wire, prm.Resistor):
+            if isinstance(wire, _prm.Resistor):
                 yield from allwires(wire.wire)
                 yield from wire.indicator
-            if isinstance(wire, prm._PinAttribute) and wire.pin is not None:
-                yield from wire.pin
-            if isinstance(wire, prm._BlockageAttribute) and wire.blockage is not None:
+            if isinstance(wire, _prm.PinAttrPrimitiveT) and wire.pin is not None:
+                yield wire.pin
+            if isinstance(wire, _prm.BlockageAttrPrimitiveT) and wire.blockage is not None:
                 yield wire.blockage
             yield wire
 
@@ -373,7 +363,7 @@ class Technology(abc.ABC):
 
                 noconn = tuple(filter(
                     # MIMTop does not need to be connected from bottom
-                    lambda l: not isinstance(l, prm.MIMTop),
+                    lambda l: not isinstance(l, _prm.MIMTop),
                     bottomwires - viabottoms,
                 ))
                 if noconn:
@@ -401,19 +391,19 @@ class Technology(abc.ABC):
             )
 
         # Add via and it's blockage layers
-        vias = tuple(prims.__iter_type__(prm.Via))
+        vias = tuple(prims.__iter_type__(_prm.Via))
         add_prims(prim.blockage for prim in vias)
         # Now add all vias
         add_prims(vias)
 
         # process mosfets
-        mosfets = set(prims.__iter_type__(prm.MOSFET))
+        mosfets = set(prims.__iter_type__(_prm.MOSFET))
         gates = {mosfet.gate for mosfet in mosfets}
-        allgates = set(prims.__iter_type__(prm.MOSFETGate))
+        allgates = set(prims.__iter_type__(_prm.MOSFETGate))
         if gates != allgates:
             diff = allgates - gates
             if diff:
-                raise prm.UnusedPrimitiveError(diff.pop())
+                raise _prm.UnusedPrimitiveError(primitive=diff.pop())
             raise RuntimeError("Unhandled error condition") # pragma: no cover
         actives = {gate.active for gate in gates}
         polys = {gate.poly for gate in gates}
@@ -434,60 +424,63 @@ class Technology(abc.ABC):
         markers = set()
 
         # proces pad openings
-        padopenings = set(prims.__iter_type__(prm.PadOpening))
+        padopenings = set(prims.__iter_type__(_prm.PadOpening))
         viabottoms = set()
         for padopening in padopenings:
             add_prims(allwires(padopening.bottom))
         add_prims(padopenings)
 
         # process top metal wires
-        add_prims(prims.__iter_type__(prm.TopMetalWire))
+        add_prims(prims.__iter_type__(_prm.TopMetalWire))
 
         # process resistors
-        resistors = set(prims.__iter_type__(prm.Resistor))
+        resistors = set(prims.__iter_type__(_prm.Resistor))
         for resistor in resistors:
             markers.update(resistor.indicator)
 
         # process capacitors
-        mimtops = set(prims.__iter_type__(prm.MIMTop))
-        mimcaps = tuple(prims.__iter_type__(prm.MIMCapacitor))
+        mimtops = set(prims.__iter_type__(_prm.MIMTop))
+        mimcaps = tuple(prims.__iter_type__(_prm.MIMCapacitor))
         usedtops = set(c.top for c in mimcaps)
         unusedtops = mimtops - usedtops
         if unusedtops:
-            raise prm.UnusedMIMTop(unusedtops.pop())
+            s_tops = ",".join(f"'{top}.name'" for top in unusedtops)
+            raise _prm.UnusedPrimitiveError(
+                primitive=unusedtops.pop(), msg=f"MIMTops {s_tops} not used in MIMCapacitor",
+            )
 
         # process diodes
-        diodes = set(prims.__iter_type__(prm.Diode))
+        diodes = set(prims.__iter_type__(_prm.Diode))
         for diode in diodes:
             markers.update(diode.indicator)
 
         # process bipolars
-        bipolars = set(prims.__iter_type__(prm.Bipolar))
+        bipolars = set(prims.__iter_type__(_prm.Bipolar))
         for bipolar in bipolars:
             markers.update(bipolar.indicator)
 
         # extra rules
-        rules = set(prims.__iter_type__(prm._RulePrimitive))
+        rules = set(prims.__iter_type__(_prm.RulePrimitiveT))
 
         add_prims((*markers, *resistors, *mimcaps, *diodes, *bipolars, *rules))
 
         # process auxiliary
-        def aux_key(aux: prm.Auxiliary) -> str:
+        def aux_key(aux: _prm.Auxiliary) -> str:
             return aux.name
-        add_prims(sorted(prims.__iter_type__(prm.Auxiliary), key=aux_key))
+        add_prims(sorted(prims.__iter_type__(_prm.Auxiliary), key=aux_key))
 
         # reorder primitives
         unused = set(range(len(prims))) - set(neworder)
         if unused:
-            raise prm.UnusedPrimitiveError(prims[unused.pop()])
-        prims._reorder_(neworder)
+            raise _prm.UnusedPrimitiveError(primitive=prims[unused.pop()])
+        prims._reorder_(neworder=neworder)
 
-    def _build_rules(self):
+    def _build_rules(self) -> None:
         prims = self._primitives
-        self._rules = rules = rle.Rules()
+        self._rules = rules = _rle.Rules()
 
         # grid
-        rules += wfr.wafer.grid == self.grid
+        rules += _wfr.wafer.grid == self.grid
 
         # Generate the rule but don't add them yet.
         for prim in prims:
@@ -496,10 +489,10 @@ class Technology(abc.ABC):
         # First add substrate alias if needed. This will only be clear
         # after the rules have been generated.
         sub = self.substrate
-        if isinstance(sub, msk._MaskAlias):
+        if isinstance(sub, _msk._MaskAlias):
             self._rules += sub
-        if sub != wfr.wafer:
-            self._rules += msk.Connect(sub, wfr.wafer)
+        if sub != _wfr.wafer:
+            self._rules += _msk.Connect(sub, _wfr.wafer)
 
         # Now we can add the rules
         for prim in prims:
@@ -508,7 +501,7 @@ class Technology(abc.ABC):
         rules._freeze_()
 
     @property
-    def substrate(self) -> msk._Mask:
+    def substrate(self) -> _msk.MaskT:
         """Property representing the substrate of the technology; it's defined as the area
         that is outside any of the wells of the technology.
 
@@ -516,31 +509,23 @@ class Technology(abc.ABC):
         technology has been initialized and is not available during run of the `_init()`
         method.
         """
-        if not self._init_done:
+        if not hasattr(self, "_substrate"):
             raise AttributeError("substrate may not be accessed during object initialization")
-        if self._substrate is None:
-            well_masks = tuple(
-                prim.mask for prim in self._primitives.__iter_type__(prm.Well)
-            )
-            if not well_masks:
-                self._substrate = wfr.wafer
-            else:
-                self._substrate = wfr.outside(well_masks, alias=f"substrate:{self.name}")
         return self._substrate
 
     @property
-    def rules(self) -> Iterable[rle._Rule]:
+    def rules(self) -> Iterable[_rle.RuleT]:
         """Return all the rules that are derived from the primitives of the technology."""
         return self._rules
 
     @property
-    def primitives(self) -> "prm.Primitives":
+    def primitives(self) -> "_prm.Primitives":
         """Return the primitives of the technology."""
         return self._primitives
 
     @property
-    def designmasks(self) -> Iterable[msk.DesignMask]:
-        """Return all the `DesignMask`s defined by the primitives of the technology
+    def designmasks(self) -> Iterable[_msk.DesignMask]:
+        """Return all the `DesignMask` objects defined by the primitives of the technology.
 
         The property makes sure there are no duplicates in the returned iterable.
         """
@@ -550,4 +535,3 @@ class Technology(abc.ABC):
                 if mask not in masks:
                     yield mask
                     masks.add(mask)
-
