@@ -1,23 +1,10 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 import unittest
 
 from pdkmaster import _util
 
 class UtilTest(unittest.TestCase):
-    def test_i2f(self):
-        self.assertAlmostEqual(_util.i2f(2.0), 2.0, 12)
-        self.assertAlmostEqual(_util.i2f(1), 1.0, 12)
-        self.assertEqual(_util.i2f(None), None)
-        with self.assertRaisesRegex(
-            ValueError, "Use of bool as float not allowed",
-        ):
-            self.assertEqual(_util.i2f(True), True)
-        with self.assertRaisesRegex(
-            ValueError, "could not convert string to float: 'Yoh'",
-        ):
-            self.assertEqual(_util.i2f("Yoh"), "Yoh")
-
     def test_i2f_recursive(self):
         f = _util.i2f_recursive(2)
         self.assertIsInstance(f, float)
@@ -30,17 +17,6 @@ class UtilTest(unittest.TestCase):
         self.assertAlmostEqual(t[1], 3.0, 12)
         self.assertAlmostEqual(t[2], 4.0, 12)
 
-    def test_v2t(self):
-        self.assertEqual(_util.v2t(None), (None,))
-        self.assertEqual(_util.v2t(False), (False,))
-        self.assertEqual(_util.v2t(2.0), (2.0,))
-        self.assertEqual(_util.v2t(range(3)), (0, 1, 2))
-        self.assertEqual(_util.v2t("Yoh"), ("Yoh",))
-        self.assertEqual(_util.v2t(2, n=2), (2, 2))
-
-        with self.assertRaises(AssertionError):
-            _util.v2t(range(3), n=2)
-
     def test_is_iterable(self):
         self.assertEqual(_util.is_iterable(None), False)
         self.assertEqual(_util.is_iterable(1), False)
@@ -50,20 +26,20 @@ class UtilTest(unittest.TestCase):
         self.assertEqual(_util.is_iterable({0}), True)
         self.assertEqual(_util.is_iterable("a"), False)
 
-    def test_nth(self):
-        self.assertEqual(_util.nth(range(5), 2), 2)
+    def test_get_nth_of(self):
+        self.assertEqual(_util.get_nth_of(range(5), n=2), 2)
         with self.assertRaises(StopIteration):
-            _util.nth(range(2), 3)
+            _util.get_nth_of(range(2), n=3)
 
     def test_first(self):
-        self.assertEqual(_util.first(range(5)), 0)
+        self.assertEqual(_util.get_first_of(range(5)), 0)
         with self.assertRaises(StopIteration):
-            _util.first(tuple())
+            _util.get_first_of(tuple())
 
     def test_last(self):
-        self.assertEqual(_util.last(range(2)), 1)
+        self.assertEqual(_util.get_last_of(range(2)), 1)
         with self.assertRaises(StopIteration):
-            _util.last({})
+            _util.get_last_of({})
 
     def test_strip_literal(self):
         self.assertEqual(_util.strip_literal('"Yoh"'), 'Yoh')
@@ -72,13 +48,8 @@ class UtilTest(unittest.TestCase):
         self.assertEqual(_util.strip_literal('"Yoh'), '"Yoh')
         self.assertEqual(_util.strip_literal('Yoh"'), 'Yoh"')
 
-    def test_typedlist(self):
-        class MyList(_util.TypedList[float]):
-            @property
-            def _elem_type_(self):
-                return (float, int)
-        
-        l = MyList()
+    def test_extendedlist(self):
+        l = _util.ExtendedList[float]()
         self.assertEqual(list(l + 3.0), [3.0])
         self.assertEqual(list(l + [2.0, 3.0]), [2.0, 3.0])
 
@@ -107,12 +78,15 @@ class UtilTest(unittest.TestCase):
         l.sort()
         self.assertEqual(list(l), [0.0, 1.0])
 
-        l._reorder_([1, 0])
-        self.assertEqual(list(l), [1.0, 0.0])
+        l.append(2.0)
+        self.assertEqual(list(l), [0.0, 1.0, 2.0])
+
+        l._reorder_(neworder=[2, 1, 0])
+        self.assertEqual(list(l), [2.0, 1.0, 0.0])
 
         with self.assertRaises(ValueError):
             # indices mismatch
-            l._reorder_([2, 1, 0])
+            l._reorder_(neworder=[1, 0])
         with self.assertRaises(TypeError):
             # hash for non-frozen TypedList
             hash(l)
@@ -121,6 +95,8 @@ class UtilTest(unittest.TestCase):
         self.assertEqual(hash(l), hash(tuple(l)))
         with self.assertRaises(TypeError):
             del l[0]
+        with self.assertRaises(TypeError):
+            l += 10.0
         with self.assertRaises(TypeError):
             l *= 2
         with self.assertRaises(TypeError):
@@ -144,20 +120,14 @@ class UtilTest(unittest.TestCase):
         with self.assertRaises(TypeError):
             l.sort()
         with self.assertRaises(TypeError):
-            l._reorder_([1, 0])
+            l._reorder_(neworder=[2, 1, 0])
     
-    def test_typedlistmapping(self):
+    def test_extendedlistmapping(self):
         class C:
             def __init__(self, *, name: str) -> None:
                 self.name=name
 
-        class MyList(_util.TypedListMapping[C, str]):
-            @property
-            def _elem_type_(self):
-                return C
-            @property
-            def _index_type_(self):
-                return str
+        class MyList(_util.ExtendedListMapping[C, str]):
             @property
             def _index_attribute_(self) -> str:
                 return "name"
@@ -203,10 +173,11 @@ class UtilTest(unittest.TestCase):
 
         self.assertEqual(set(l.values()), set(cs))
 
-        with self.assertRaises(TypeError):
+        with self.assertRaises(KeyError):
+            # wrong index type just leads to key not present
             del l[3.0]
-        with self.assertRaises(TypeError):
-            # wrong element type
+        with self.assertRaises(AttributeError):
+            # wrong element type so it will try to access name attribute
             l[0] = "error"
 
         c = C(name="6")
@@ -216,7 +187,7 @@ class UtilTest(unittest.TestCase):
 
         order = tuple(reversed(range(len(l))))
         cs = tuple(cs[n] for n in order)
-        l._reorder_(order)
+        l._reorder_(neworder=order)
         self.assertEqual(tuple(l), cs)
 
         l.clear()
@@ -247,7 +218,7 @@ class UtilTest(unittest.TestCase):
             l += c
         with self.assertRaises(TypeError):
             order = tuple(reversed(range(len(l))))
-            l._reorder_(order)
+            l._reorder_(neworder=order)
 
     def test_liststrmappingoverride(self):
         class C1:
@@ -257,15 +228,9 @@ class UtilTest(unittest.TestCase):
         class C2(C1):
             pass
 
-        class MyList1(_util.TypedListStrMapping[C1]):
-            @property
-            def _elem_type_(self):
-                return C1
-
+        MyList1 = _util.ExtendedListStrMapping[C1]
         class MyList2(_util.ListStrMappingOverride[C2], MyList1):
-            @property
-            def _elem_type_(self):
-                return C2
+            pass
 
         MyList1()
         MyList2()
