@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 import unittest
 from typing import cast
@@ -6,24 +6,191 @@ from typing import cast
 from pdkmaster.technology import (
     property_ as _prp, mask as _msk, net as _net, geometry as _geo, primitive as _prm,
 )
-from pdkmaster.design import layout as _lay, library as _lbry
+from pdkmaster.design import layout as _lay, cell as _cell
+from pdkmaster.design.layout import (
+    layout_ as _laylay, _primitivelayouter as _layprim,
+)
 
 from ..dummy import dummy_tech, dummy_cktfab, dummy_layoutfab, dummy_lib
 prims = dummy_tech.primitives
 
 
-class TestNet(_net.Net):
+class TestNet(_net._Net):
     # Non-abstract net class
     def __init__(self, name: str):
         super().__init__(name)
 
 
+class LayoutParamTest(unittest.TestCase):
+    def test_enclosurelayoutparam(self):
+        param = _layprim._EnclosureLayoutParam(primitive=prims.active, name="test")
+
+        with self.assertRaises(TypeError):
+            param.cast(None)
+        with self.assertRaises(TypeError):
+            param.cast("error")
+
+        param = _layprim._EnclosureLayoutParam(
+            primitive=prims.active, name="test", default=_prp.Enclosure(2.0),
+        )
+
+        enc = cast(_prp.Enclosure, param.cast(None))
+        self.assertFalse(enc.is_assymetric)
+        self.assertAlmostEqual(enc.first, 2.0)
+
+        enc = cast(_prp.Enclosure, param.cast(3.0))
+        self.assertFalse(enc.is_assymetric)
+        self.assertAlmostEqual(enc.first, 3.0)
+
+    def test_enclosureslayoutparam(self):
+        param = _layprim._EnclosuresLayoutParam(primitive=prims.active, name="test", n=2)
+
+        with self.assertRaises(TypeError):
+            param.cast(None)
+        with self.assertRaises(TypeError):
+            param.cast("error")
+        with self.assertRaises(TypeError):
+            param.cast(("error", "error"))
+
+        default = (_prp.Enclosure(0.2), _prp.Enclosure(0.3))
+        param = _layprim._EnclosuresLayoutParam(
+            primitive=prims.active, name="test", n=2, default=default,
+        )
+
+        self.assertEqual(param.cast(None), default)
+
+        v = 0.25
+        enc = _prp.Enclosure(v)
+        self.assertEqual(param.cast(v), (enc, enc))
+
+        v = (None, _prp.Enclosure(0.15))
+        self.assertEqual(param.cast(v), v)
+
+
+class LayoutParamCasterTest(unittest.TestCase):
+    def test_caster(self):
+        caster = _layprim._LayoutParamCaster()
+
+        nwell = prims.nwell
+        deepwell = prims.deepwell
+        active = prims.active
+        poly = prims.poly
+        nplus = prims.nplus
+        pplus = prims.pplus
+        hvox = prims.hvox
+        ch = prims.contact
+        metal = prims.metal
+        metalpin = prims.metalpin
+        metal2 = prims.metal2
+        metal2pin = prims.metal2pin
+        ndiode = prims.ndiode
+
+        class MyNet(_net._Net):
+            def __init__(self, name: str):
+                super().__init__(name)
+        net = MyNet("net")
+
+        # Unexpected parameter
+        with self.assertRaises(TypeError):
+            caster(prim=active, implant=nplus, error="error")
+
+        params = caster(prim=active, implant=nplus, height=4.0)
+
+        self.assertAlmostEqual(params["width"], active.min_width)
+        self.assertAlmostEqual(params["height"], 4.0)
+
+        params = caster(prim=metalpin, width=2.0)
+
+        self.assertAlmostEqual(params["width"], 2.0)
+        self.assertIsNone(params["height"])
+
+        with self.assertRaises(ValueError):
+            caster(prim=metal, pin=metal2pin)
+
+        # WaferWire with single implant
+        active2 = _prm.WaferWire(
+            name="active2", min_width=0.2, min_space=0.5,
+            allow_in_substrate=False,
+            implant=nplus, min_implant_enclosure=_prp.Enclosure(0.05),
+            implant_abut=nplus, allow_contactless_implant=False,
+            well=nwell, min_well_enclosure=_prp.Enclosure(0.1),
+            min_well_enclosure_same_type=_prp.Enclosure(0.08),
+            allow_well_crossing=False,
+            oxide=hvox, min_oxide_enclosure=_prp.Enclosure(0.08),
+        )
+        params = caster(prim=active2, well_net=net, oxide=hvox)
+
+        self.assertEqual(params["implant"], nplus)
+        self.assertAlmostEqual(params["implant_enclosure"].first, 0.05)
+        self.assertEqual(params["well"], nwell)
+        self.assertAlmostEqual(params["oxide_enclosure"].first, 0.08)
+
+        ### Via errors
+
+        # no bottom_implant
+        with self.assertRaises(TypeError):
+            caster(prim=ch, bottom=active)
+
+        # wrong bottom_well
+        with self.assertRaises(ValueError):
+            caster(prim=ch, bottom=active, bottom_implant=nplus, bottom_well=deepwell)
+
+        # bottom WaferWire parameters given for GateWire bottom
+        with self.assertRaises(TypeError):
+            caster(prim=ch, bottom=poly, bottom_implant=nplus)
+        with self.assertRaises(TypeError):
+            caster(prim=ch, bottom=poly, bottom_implant_enclosure=0.3)
+        with self.assertRaises(TypeError):
+            caster(prim=ch, bottom=poly, bottom_well=nwell)
+        with self.assertRaises(TypeError):
+            caster(prim=ch, bottom=poly, bottom_well_enclosure=0.3)
+        with self.assertRaises(TypeError):
+            caster(prim=ch, bottom=poly, well_net=net)
+
+        ### Via coverage
+
+        # default bottom_well_enclosure
+        params = caster(
+            prim=ch, bottom=active, bottom_implant=pplus, bottom_well=nwell,
+        )
+
+        self.assertEqual(params["bottom_well_enclosure"], active.min_well_enclosure[0])
+
+        # ch with active not allowed in substrate
+        ch2 = _prm.Via(
+            name="contact2", width=0.2, min_space=0.2,
+            bottom=(active2, poly), top=metal,
+            min_bottom_enclosure=_prp.Enclosure(0.2), min_top_enclosure=_prp.Enclosure(0.15),
+        )
+        # no bottom_well for waferwire not allowed in substrate
+        with self.assertRaises(TypeError):
+            caster(prim=ch2, bottom=active2, bottom_implant=nplus)
+
+        # via with one bottom and one top
+        via2 = _prm.Via(
+            name="via2", width=0.35, min_space=0.35,
+            bottom=metal, top=metal2,
+            min_bottom_enclosure=_prp.Enclosure(0.2), min_top_enclosure=_prp.Enclosure(0.15),
+        )
+        # default bottom & top
+        params = caster(prim=via2)
+
+        self.assertEqual(params["bottom"], metal)
+        self.assertEqual(params["top"], metal2)
+
+        # Diode to cover _DevicePrimitive
+
+        params = caster(prim=ndiode, width=2.0, height=None)
+        self.assertAlmostEqual(params["width"], 2.0)
+        self.assertAlmostEqual(params["height"], active.min_width)
+
+
 class LayoutTest(unittest.TestCase):
     def test__rect(self):
 
         self.assertEqual(
-            _lay._rect(left=0.0, bottom=0.0, right=1.0, top=1.0, enclosure=1.0),
-            _lay._rect(
+            _layprim._rect(left=0.0, bottom=0.0, right=1.0, top=1.0, enclosure=1.0),
+            _layprim._rect(
                 left=0.0, bottom=0.0, right=1.0, top=1.0,
                 enclosure=_prp.Enclosure(1.0),
             ),
@@ -39,13 +206,13 @@ class LayoutTest(unittest.TestCase):
         point = _geo.Point(x=(left + 0.5*width), y=(bottom + 0.5*width))
 
         self.assertEqual(
-            _lay._via_array(
+            _layprim._via_array(
                 left=left, bottom=bottom, width=width, pitch=pitch, rows=1, columns=1,
             ),
             via + point,
         )
         self.assertEqual(
-            _lay._via_array(
+            _layprim._via_array(
                 left=left, bottom=bottom, width=width, pitch=pitch, rows=2, columns=2,
             ),
             _geo.ArrayShape(
@@ -54,8 +221,8 @@ class LayoutTest(unittest.TestCase):
         )
 
     def test_maskshapessublayout(self):
-        m1 = _msk.DesignMask(name="mask1", fill_space="no")
-        m2 = _msk.DesignMask(name="mask2", fill_space="no")
+        m1 = _msk.DesignMask(name="mask1")
+        m2 = _msk.DesignMask(name="mask2")
         n1 = TestNet("net1")
         p = _geo.Point(x=3.0, y=-2.0)
         rot = _geo.Rotation.R90
@@ -63,13 +230,13 @@ class LayoutTest(unittest.TestCase):
         r2 = _geo.Rect(left=1.0, bottom=-1.0, right=3.0, top=1.0)
         ms1 = _geo.MaskShape(mask=m1, shape=r1)
         ms2 = _geo.MaskShape(mask=m2, shape=r2)
-        mssl1 = _lay.MaskShapesSubLayout(
+        mssl1 = _laylay._MaskShapesSubLayout(
             net=n1, shapes=_geo.MaskShapes(ms1),
         )
-        mssl2 = _lay.MaskShapesSubLayout(
+        mssl2 = _laylay._MaskShapesSubLayout(
             net=None, shapes=_geo.MaskShapes(ms1),
         )
-        mssl3 = _lay.MaskShapesSubLayout(
+        mssl3 = _laylay._MaskShapesSubLayout(
             net=n1, shapes=_geo.MaskShapes((ms1, ms2)),
         )
         mssl4 = mssl1.dup()
@@ -98,17 +265,26 @@ class LayoutTest(unittest.TestCase):
         self.assertEqual(mssl6, mssl7)
 
     def test__instancesublayout(self):
-        cell = _lbry._Cell(lib=dummy_lib, name="cell")
+        cell = _cell.Cell(
+            name="cell",
+            tech=dummy_tech, cktfab=dummy_cktfab, layoutfab=dummy_layoutfab,
+        )
         ckt = cell.new_circuit()
 
         inst = ckt.instantiate(dummy_lib.cells.cell1, name="libcell1")
 
-        cell2 = _lbry._Cell(lib=dummy_lib, name="cell2")
+        cell2 = _cell.Cell(
+            name="cell2",
+            tech=dummy_tech, cktfab=dummy_cktfab, layoutfab=dummy_layoutfab,
+        )
         # Create empty layout for cell2
         cell2.new_circuit()
         cell2.new_circuitlayouter()
 
-        cell3 = _lbry._Cell(lib=dummy_lib, name="cell3")
+        cell3 = _cell.Cell(
+            name="cell3",
+            tech=dummy_tech, cktfab=dummy_cktfab, layoutfab=dummy_layoutfab,
+        )
         ckt3 = cell3.new_circuit()
 
         inst2 = ckt3.instantiate(cell, name="cell")
@@ -119,19 +295,19 @@ class LayoutTest(unittest.TestCase):
 
         with self.assertRaises(ValueError):
             # No layout
-            _lay._InstanceSubLayout(
+            _laylay._InstanceSubLayout(
                 inst=inst2, origin=_geo.origin, layoutname=None, rotation=_geo.Rotation.R0,
             )
         with self.assertRaises(ValueError):
             # wrong layoutname
-            _lay._InstanceSubLayout(
+            _laylay._InstanceSubLayout(
                 inst=inst2, origin=_geo.origin, layoutname="error", rotation=_geo.Rotation.R0,
             )
 
-        sl1 = _lay._InstanceSubLayout(
+        sl1 = _laylay._InstanceSubLayout(
             inst=inst, origin=_geo.origin, layoutname=None, rotation=_geo.Rotation.R0,
         )
-        sl2 = _lay._InstanceSubLayout(
+        sl2 = _laylay._InstanceSubLayout(
             inst=inst3, origin=_geo.origin, layoutname=None, rotation=_geo.Rotation.R0,
         )
         sl3 = sl2.rotated(rotation=rot).moved(dxy=dxy)
@@ -164,16 +340,15 @@ class LayoutTest(unittest.TestCase):
         ms2 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=r2))
         ms3 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=r3))
 
-        sl1 = _lay.MaskShapesSubLayout(net=None, shapes=ms1)
-        sl2 = _lay.MaskShapesSubLayout(net=None, shapes=ms2)
-        sl3 = _lay.MaskShapesSubLayout(net=None, shapes=ms3)
+        sl1 = _laylay._MaskShapesSubLayout(net=None, shapes=ms1)
+        sl2 = _laylay._MaskShapesSubLayout(net=None, shapes=ms2)
+        sl3 = _laylay._MaskShapesSubLayout(net=None, shapes=ms3)
 
         with self.assertRaises(ValueError):
             # sublayouts for same net
-            _lay.SubLayouts((sl1, sl2))
+            _laylay._SubLayouts((sl1, sl2))
 
-        sls = _lay.SubLayouts((sl1,))
-        self.assertIs(sls._elem_type_, _lay._SubLayout)
+        sls = _laylay._SubLayouts((sl1,))
 
         # Code coverage
         self.assertEqual(len(sls), 1)
@@ -195,21 +370,21 @@ class LayoutTest(unittest.TestCase):
         ms1 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=mps.parts[0]))
         ms2 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=mps.parts[1]))
 
-        sl1 = _lay.MaskShapesSubLayout(net=None, shapes=ms1)
-        sl2 = _lay.MaskShapesSubLayout(net=None, shapes=ms2)
+        sl1 = _laylay._MaskShapesSubLayout(net=None, shapes=ms1)
+        sl2 = _laylay._MaskShapesSubLayout(net=None, shapes=ms2)
 
         lay = dummy_layoutfab.new_layout()
         lay += sl1
         lay += sl2
 
         lay2 = dummy_layoutfab.new_layout()
-        lay2.add_shape(prim=prims.metal, net=None, shape=mps.parts[0])
-        lay2.add_shape(prim=prims.metal, net=None, shape=mps.parts[1])
+        lay2.add_shape(net=None, shape=ms1[0])
+        lay2.add_shape(layer=prims.metal.mask, net=None, shape=mps.parts[1])
 
         for l, m in ((lay, "lay"), (lay2, "lay2")):
-            self.assertEqual(len(l.sublayouts), 1)
-            sl = cast(_lay.MaskShapesSubLayout, l.sublayouts[0])
-            self.assertIsInstance(sl, _lay.MaskShapesSubLayout)
+            self.assertEqual(len(l._sublayouts), 1)
+            sl = cast(_laylay._MaskShapesSubLayout, l._sublayouts[0])
+            self.assertIsInstance(sl, _laylay._MaskShapesSubLayout)
             self.assertIsNone(sl.net)
             self.assertEqual(len(sl.shapes), 1)
             ms = sl.shapes[0]
@@ -234,9 +409,9 @@ class LayoutTest(unittest.TestCase):
         mus = _geo.MultiShape(shapes=(mps.parts[0], mps.parts[1]))
         mss3 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=mus))
 
-        sl1 = _lay.MaskShapesSubLayout(net=None, shapes=mss1)
-        sl2 = _lay.MaskShapesSubLayout(net=None, shapes=mss2)
-        sl3 = _lay.MaskShapesSubLayout(net=None, shapes=mss3)
+        sl1 = _laylay._MaskShapesSubLayout(net=None, shapes=mss1)
+        sl2 = _laylay._MaskShapesSubLayout(net=None, shapes=mss2)
+        sl3 = _laylay._MaskShapesSubLayout(net=None, shapes=mss3)
 
         layout = dummy_layoutfab.new_layout(sublayouts=sl3)
 
@@ -253,8 +428,8 @@ class LayoutTest(unittest.TestCase):
         l += sl2
         layout4 = l.moved(dxy=dxy)
 
-        def get_parts(l: _lay._Layout):
-            for sl in l.sublayouts.__iter_type__(_lay.MaskShapesSubLayout):
+        def get_parts(l: _lay.LayoutT):
+            for sl in l._sublayouts.__iter_type__(_laylay._MaskShapesSubLayout):
                 for ms in sl.shapes:
                     s = ms.shape
                     if isinstance(s, _geo.MultiPartShape._Part):
@@ -291,13 +466,10 @@ class LayoutTest(unittest.TestCase):
         # Code coverage
         lay2 = lay.dup()
 
-        tuple(lay.net_polygons(net=i_net))
         tuple(lay.filter_polygons(net=i_net))
         tuple(lay.filter_polygons(net=i_net, mask=metalpin.mask))
         tuple(lay.filter_polygons(net=i_net, mask=metalpin.mask, split=True))
 
-        with self.assertRaises(TypeError):
-            lay.bounds(depth=1)
         lay.bounds(net=i_net)
 
         self.assertEqual(lay, lay)
@@ -320,7 +492,7 @@ class LayoutTest(unittest.TestCase):
         lay._hier_str_
 
         # code coverage
-        dummy_layoutfab.new_layout(sublayouts=lay.sublayouts[0])
+        dummy_layoutfab.new_layout(sublayouts=lay._sublayouts[0])
 
         ###
 
@@ -329,10 +501,24 @@ class LayoutTest(unittest.TestCase):
         ckt = cell2.circuit
         lay = cell2.layout
 
-        # TODO: supprt _InstanceSubLayout when net=None; see
-        # https://gitlab.com/Chips4Makers/PDKMaster/-/issues/34
-        with self.assertRaises(AssertionError):
-            tuple(lay.filter_polygons(net=None, mask=metalpin.mask))
+        self.assertEqual(len(lay._sublayouts), 1)
+        sl = lay._sublayouts[0]
+        self.assertIsInstance(sl, _laylay._InstanceSubLayout)
+
+        subckt = sl.inst.cell.circuit
+        sublay = sl.inst.cell.layout
+
+        shapes1 = tuple(sublay.filter_polygons(net=subckt.nets.i, mask=prims.metal.mask))
+        self.assertEqual(len(shapes1), 1)
+
+        shapes2 = tuple(lay.filter_polygons(net=ckt.nets.i, mask=prims.metal.mask))
+        self.assertEqual(len(shapes2), 1)
+
+        shapes3 = tuple(lay.filter_polygons(net=None, mask=prims.metal.mask))
+        self.assertEqual(len(shapes3), 1)
+
+        self.assertEqual(shapes2[0], shapes1[0] + sl.origin)
+        self.assertEqual(shapes3[0], shapes1[0] + sl.origin)
 
         ###
 
@@ -347,7 +533,7 @@ class LayoutTest(unittest.TestCase):
 
 class PrimitiveLayouterTest(unittest.TestCase):
     def test__primitivelayouter(self):
-        layouter = _lay._PrimitiveLayouter(fab=dummy_layoutfab)
+        layouter = _layprim._PrimitiveLayouter(fab=dummy_layoutfab)
 
         enc = _prp.Enclosure(0.2)
         enc2 = _prp.Enclosure(1.0)
@@ -363,8 +549,8 @@ class PrimitiveLayouterTest(unittest.TestCase):
             layouter(prims.metalpin)
 
         lay = layouter(prims.metalpin, width=1.0, height=1.0)
-        self.assertEqual(len(lay.sublayouts), 1)
-        sl = lay.sublayouts[0]
+        self.assertEqual(len(lay._sublayouts), 1)
+        sl = lay._sublayouts[0]
         self.assertIs(sl.net, None)
         self.assertEqual(len(sl.shapes), 1)
         ms = sl.shapes[0]
@@ -375,8 +561,8 @@ class PrimitiveLayouterTest(unittest.TestCase):
 
         # _WidthSpaceConductor without portnets
         lay = layouter(prims.metal, width=1.0, height=1.0)
-        self.assertEqual(len(lay.sublayouts), 1)
-        sl = lay.sublayouts[0]
+        self.assertEqual(len(lay._sublayouts), 1)
+        sl = lay._sublayouts[0]
         self.assertIs(sl.net, prims.metal.ports.conn)
         self.assertEqual(len(sl.shapes), 1)
         ms = sl.shapes[0]
@@ -391,15 +577,15 @@ class PrimitiveLayouterTest(unittest.TestCase):
             implant=prims.nplus, implant_enclosure=enc,
         )
         self.assertEqual(
-            lay.sublayouts,
-            _lay.SubLayouts((
-                _lay.MaskShapesSubLayout(
+            lay._sublayouts,
+            _laylay._SubLayouts((
+                _laylay._MaskShapesSubLayout(
                     net=prims.active.ports.conn,
                     shapes=_geo.MaskShapes(_geo.MaskShape(
                         mask=prims.active.mask, shape=rect,
                     ))
                 ),
-                _lay.MaskShapesSubLayout(
+                _laylay._MaskShapesSubLayout(
                     net=None,
                     shapes=_geo.MaskShapes(_geo.MaskShape(
                         mask=prims.nplus.mask, shape=rect_enc,
@@ -417,16 +603,16 @@ class PrimitiveLayouterTest(unittest.TestCase):
             well_net=prims.active.ports.conn,
         )
         self.assertEqual(
-            lay.sublayouts,
-            _lay.SubLayouts((
-                _lay.MaskShapesSubLayout(
+            lay._sublayouts,
+            _laylay._SubLayouts((
+                _laylay._MaskShapesSubLayout(
                     net=prims.active.ports.conn,
                     shapes=_geo.MaskShapes((
                         _geo.MaskShape(mask=prims.active.mask, shape=rect),
                         _geo.MaskShape(mask=prims.nwell.mask, shape=rect_enc2),
                     ))
                 ),
-                _lay.MaskShapesSubLayout(
+                _laylay._MaskShapesSubLayout(
                     net=None,
                     shapes=_geo.MaskShapes((
                         _geo.MaskShape(mask=prims.nplus.mask, shape=rect_enc),
@@ -437,7 +623,7 @@ class PrimitiveLayouterTest(unittest.TestCase):
         )
 
         # WaferWire with well but no well_net
-        with self.assertRaises(TypeError):
+        with self.assertRaises(ValueError):
             lay = layouter(
                 prims.active, width=1.0, height=1.0,
                 implant=prims.nplus, implant_enclosure=_prp.Enclosure(0.2),
@@ -460,9 +646,9 @@ class PrimitiveLayouterTest(unittest.TestCase):
         mrect = _geo.Rect.from_rect(rect=viarect, bias=enc)
         mwidth = mrect.bounds.width
         self.assertEqual(
-            lay.sublayouts,
-            _lay.SubLayouts((
-                _lay.MaskShapesSubLayout(
+            lay._sublayouts,
+            _laylay._SubLayouts((
+                _laylay._MaskShapesSubLayout(
                     net=prims.via.ports.conn,
                     shapes=_geo.MaskShapes((
                         _geo.MaskShape(mask=prims.via.mask, shape=viarect),
@@ -540,6 +726,23 @@ class PrimitiveLayouterTest(unittest.TestCase):
             ),
         )
 
+        # Via with bottom_implant_enclosure as str
+        # enclosure is symmetric so "wide" and "tall" should give same layout
+        self.assertEqual(
+            layouter(
+                prims.contact, space=0.3, rows=1, columns=1,
+                bottom=prims.active, bottom_enclosure="tall",
+                bottom_implant=prims.nplus, bottom_implant_enclosure="wide",
+                top_enclosure="tall",
+            ),
+            layouter(
+                prims.contact, space=0.3, rows=1, columns=1,
+                bottom=prims.active, bottom_enclosure="tall",
+                bottom_implant=prims.nplus, bottom_implant_enclosure="tall",
+                top_enclosure="tall",
+            ),
+        )
+
         # Via with wrong portnets
         with self.assertRaises(ValueError):
             lay = layouter(prims.via, portnets={"error": prims.via.ports.conn})
@@ -550,64 +753,54 @@ class PrimitiveLayouterTest(unittest.TestCase):
         layouter(
             prims.contact, space=0.35, rows=1, columns=1,
             bottom=prims.active, bottom_enclosure=enc,
-            bottom_width=None, bottom_height=None,
             bottom_implant=prims.pplus, bottom_implant_enclosure=enc,
             bottom_oxide=prims.hvox, bottom_oxide_enclosure=enc,
             bottom_well=prims.nwell, well_net=prims.active.ports.conn,
             bottom_well_enclosure=enc,
-            top=prims.metal, top_enclosure=enc,
-            top_width=None, top_height=None,
+            top_enclosure=enc,
         )
         layouter(
             prims.contact, space=0.35, rows=1, columns=1,
             bottom=prims.active, bottom_enclosure=enc,
-            bottom_width=None, bottom_height=None,
             bottom_implant=prims.nplus, bottom_implant_enclosure=enc,
             bottom_oxide=prims.hvox, bottom_oxide_enclosure=None,
             bottom_well=prims.nwell, well_net=prims.contact.ports.conn,
             bottom_well_enclosure=enc,
-            top=prims.metal, top_enclosure=enc,
-            top_width=None, top_height=None,
+            top_enclosure=enc,
         )
-        # bottom_well not given
+        # well_net not given
         with self.assertRaises(TypeError):
             layouter(
                 prims.contact, space=0.35, rows=1, columns=1,
                 bottom=prims.active, bottom_enclosure=enc,
-                bottom_width=None, bottom_height=None,
                 bottom_implant=prims.pplus, bottom_implant_enclosure=enc,
                 bottom_oxide=prims.hvox, bottom_oxide_enclosure=enc,
-                bottom_well=prims.nwell, well_net=None,
+                bottom_well=prims.nwell,
                 bottom_well_enclosure=enc,
-                top=prims.metal, top_enclosure=enc,
-                top_width=None, top_height=None,
+                top_enclosure=enc,
             )
         # bottom_well type is bottom_implant type but net is different
         with self.assertRaises(ValueError):
             layouter(
                 prims.contact, space=0.35, rows=1, columns=1,
                 bottom=prims.active, bottom_enclosure=enc,
-                bottom_width=None, bottom_height=None,
                 bottom_implant=prims.nplus, bottom_implant_enclosure=enc,
-                bottom_oxide=prims.hvox, bottom_oxide_enclosure=None,
+                bottom_oxide=prims.hvox,
                 bottom_well=prims.nwell, well_net=prims.poly.ports.conn,
                 bottom_well_enclosure=enc,
-                top=prims.metal, top_enclosure=enc,
-                top_width=None, top_height=None,
+                top_enclosure=enc,
             )
         # bottom_well type == implant type; same well
         lay = layouter(
             prims.contact, space=0.35, rows=1, columns=1,
             bottom=prims.active, bottom_enclosure=enc,
-            bottom_width=None, bottom_height=None,
             bottom_implant=prims.nplus, bottom_implant_enclosure=enc,
-            bottom_oxide=prims.hvox, bottom_oxide_enclosure=None,
+            bottom_oxide=prims.hvox,
             bottom_well=prims.nwell,
             bottom_well_enclosure=enc,
-            top=prims.metal, top_enclosure=enc,
-            top_width=None, top_height=None,
+            top_enclosure=enc,
         )
-        self.assertEqual(len(lay.sublayouts), 2)
+        self.assertEqual(len(lay._sublayouts), 2)
 
         # DeppWell
         with self.assertRaises(NotImplementedError):
@@ -618,13 +811,13 @@ class PrimitiveLayouterTest(unittest.TestCase):
         with self.assertRaises(NotImplementedError):
             layouter(prims.metal2res)
         self.assertEqual(
-            layouter(prims.resistor, width=1.0, height=1.0),
+            layouter(prims.resistor, width=1.0, length=1.0),
             layouter(
                 prims.resistor, portnets={
                     "port1": prims.resistor.ports.port1,
                     "port2": prims.resistor.ports.port2,
                 },
-                width=1.0, height=1.0,
+                width=1.0, length=1.0,
             ),
         )
         # wrong ports
@@ -737,7 +930,7 @@ class CircuitLayouterTest(unittest.TestCase):
         layouter.add_wire(net=net, wire=wire, shape=shape)
 
         layout = dummy_layoutfab.new_layout()
-        layout.add_shape(net=net, prim=wire, shape=shape)
+        layout.add_shape(net=net, layer=wire, shape=shape)
 
         self.assertEqual(layouter.layout, layout)
 
@@ -776,16 +969,16 @@ class CircuitLayouterTest(unittest.TestCase):
         ms1 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=mps.parts[0]))
         ms2 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=mps.parts[1]))
 
-        sl1 = _lay.MaskShapesSubLayout(net=net, shapes=ms1)
-        sl2 = _lay.MaskShapesSubLayout(net=net2, shapes=ms2)
+        sl1 = _laylay._MaskShapesSubLayout(net=net, shapes=ms1)
+        sl2 = _laylay._MaskShapesSubLayout(net=net2, shapes=ms2)
 
         lay = dummy_layoutfab.new_layout()
         lay += sl1
         lay += sl2
         layouter.place(lay, origin=_geo.origin)
 
-        def get_parts(l: _lay._Layout):
-            for sl in l.sublayouts.__iter_type__(_lay.MaskShapesSubLayout):
+        def get_parts(l: _lay.LayoutT):
+            for sl in l._sublayouts.__iter_type__(_laylay._MaskShapesSubLayout):
                 for ms in sl.shapes:
                     s = ms.shape
                     if isinstance(s, _geo.MultiPartShape._Part):
@@ -820,16 +1013,16 @@ class CircuitLayouterTest(unittest.TestCase):
         ms1 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=mps.parts[0]))
         ms2 = _geo.MaskShapes(_geo.MaskShape(mask=mask, shape=mps.parts[1]))
 
-        sl1 = _lay.MaskShapesSubLayout(net=net, shapes=ms1)
-        sl2 = _lay.MaskShapesSubLayout(net=net, shapes=ms2)
+        sl1 = _laylay._MaskShapesSubLayout(net=net, shapes=ms1)
+        sl2 = _laylay._MaskShapesSubLayout(net=net, shapes=ms2)
 
         lay = dummy_layoutfab.new_layout()
         lay += sl1
         lay += sl2
         layouter.place(lay, origin=_geo.origin)
 
-        def get_parts(l: _lay._Layout):
-            for sl in l.sublayouts.__iter_type__(_lay.MaskShapesSubLayout):
+        def get_parts(l: _lay.LayoutT):
+            for sl in l._sublayouts.__iter_type__(_laylay._MaskShapesSubLayout):
                 for ms in sl.shapes:
                     s = ms.shape
                     if isinstance(s, _geo.MultiPartShape._Part):
