@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 import unittest
 
@@ -8,7 +8,7 @@ from pdkmaster.dispatch.edge import EdgeDispatcher
 
 # Simple dispatcher that just returns the type of the edge
 class MyDispatcher(EdgeDispatcher):
-    def _Edge(self, edge: _edg._Edge):
+    def _Edge(self, edge: _edg.EdgeT):
         return type(edge)
 
 
@@ -16,7 +16,7 @@ class EdgeDispatchTest(unittest.TestCase):
     def test_notimplemented(self):
         with self.assertRaises(NotImplementedError):
             # Call ShapeDispatched._Shape() method
-            mask = _msk.DesignMask(name="test", fill_space="yes")
+            mask = _msk.DesignMask(name="test")
             EdgeDispatcher()(_edg.MaskEdge(mask))
 
     def test_dispatch(self):
@@ -25,8 +25,8 @@ class EdgeDispatchTest(unittest.TestCase):
         with self.assertRaises(RuntimeError):
             disp("error")
 
-        mask1 = _msk.DesignMask(name="mask1", fill_space="yes")
-        mask2 = _msk.DesignMask(name="mask2", fill_space="yes")
+        mask1 = _msk.DesignMask(name="mask1")
+        mask2 = _msk.DesignMask(name="mask2")
 
         edge1 = _edg.MaskEdge(mask1)
         edge2 = _edg.MaskEdge(mask2)
