@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 import unittest
 
@@ -8,7 +8,7 @@ from pdkmaster.dispatch.mask import MaskDispatcher
 
 # Simple dispatcher that just returns the type of the mask
 class MyDispatcher(MaskDispatcher):
-    def _Mask(self, mask: _msk._Mask):
+    def _Mask(self, mask: _msk.MaskT):
         return type(mask)
 
 
@@ -16,9 +16,7 @@ class MaskDispatchTest(unittest.TestCase):
     def test_notimplemented(self):
         with self.assertRaises(NotImplementedError):
             # Call ShapeDispatched._Shape() method
-            MaskDispatcher()(_msk.DesignMask(
-                name="test", fill_space="yes",
-            ))
+            MaskDispatcher()(_msk.DesignMask(name="test"))
 
     def test_dispatch(self):
         disp = MyDispatcher()
@@ -26,8 +24,8 @@ class MaskDispatchTest(unittest.TestCase):
         with self.assertRaises(RuntimeError):
             disp("error")
 
-        mask1 = _msk.DesignMask(name="mask1", fill_space="yes")
-        mask2 = _msk.DesignMask(name="mask2", fill_space="no")
+        mask1 = _msk.DesignMask(name="mask1")
+        mask2 = _msk.DesignMask(name="mask2")
         alias = mask1.alias("alias")
         partswith = mask1.parts_with(mask1.width > 1.0)
         join = _msk.Join(masks=(mask1, mask2))
