@@ -1,14 +1,16 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 from typing import cast
 
 from pdkmaster.typing import GDSLayerSpecDict
 from pdkmaster.technology import (
     property_ as _prp, geometry as _geo, net as _net, primitive as _prm, technology_ as _tch
 )
-from pdkmaster.design import circuit as _ckt, layout as _lay,  library as _lbry
+from pdkmaster.design import (
+    circuit as _ckt, layout as _lay,  library as _lbry, factory as _fab,
+)
+from pdkmaster.io.spice import SpicePrimsParamSpec
 
-
-class MyNet(_net.Net):
+class MyNet(_net._Net):
     def __init__(self, *, name:str):
         super().__init__(name=name)
 
@@ -20,12 +22,11 @@ class EmptyTech(_tch.Technology):
     @property
     def grid(self) -> float:
         return 0.005
-    @property
-    def substrate_type(self) -> str:
-        return "n"
 
-    def _init(self):
-        pass
+    def __init__(self):
+        super().__init__(
+            primitives=_prm.Primitives(_prm.Base(type_=_prm.undopedBaseT)),
+        )
 empty_tech = EmptyTech()
 
 
@@ -36,23 +37,18 @@ class DummyTech(_tch.Technology):
     @property
     def grid(self) -> float:
         return 0.005
-    @property
-    def substrate_type(self) -> str:
-        return "n"
     
-    def _init(self):
-        prims = self.primitives
+    def __init__(self):
+        prims = _prm.Primitives(_prm.Base(type_=_prm.nBaseT))
 
-        nwell = _prm.Well(type_="n", name="nwell", min_width=1.5, min_space=1.5)
+        nwell = _prm.Well(type_=_prm.nImplT, name="nwell", min_width=1.5, min_space=1.5)
         deepwell = _prm.DeepWell(
             name="deepwell", min_width=3.0, min_space=3.0,
             well=nwell, min_well_overlap=0.8, min_well_enclosure=0.8,
         )
-        nplus = _prm.Implant(type_="n", name="nplus", min_width=1.0, min_space=1.0)
-        pplus = _prm.Implant(type_="p", name="pplus", min_width=1.0, min_space=1.0)
-        hvox = _prm.Insulator(
-            name="hvox", min_width=0.5, min_space=0.5, fill_space="yes",
-        )
+        nplus = _prm.Implant(type_=_prm.nImplT, name="nplus", min_width=1.0, min_space=1.0)
+        pplus = _prm.Implant(type_=_prm.pImplT, name="pplus", min_width=1.0, min_space=1.0)
+        hvox = _prm.Insulator(name="hvox", min_width=0.5, min_space=0.5)
         active = _prm.WaferWire(
             name="active", min_width=0.3, min_space=0.2,
             allow_in_substrate=True,
@@ -67,7 +63,11 @@ class DummyTech(_tch.Technology):
         prims += (nwell, deepwell, nplus, pplus, hvox, active, poly)
 
         metalpin = _prm.Marker(name="metalpin")
-        metal = _prm.MetalWire(name="metal", min_width=0.1, min_space=0.1, pin=metalpin)
+        metal = _prm.MetalWire(
+            name="metal",
+            min_width=0.1, min_space=0.1, space_table=({0.2, 0.5},), min_density=0.20,
+            pin=metalpin,
+        )
         contact = _prm.Via(
             name="contact", width=0.35, min_space=0.35, bottom=(active, poly), top=metal,
             min_bottom_enclosure=_prp.Enclosure(0.2), min_top_enclosure=_prp.Enclosure(0.15),
@@ -87,9 +87,9 @@ class DummyTech(_tch.Technology):
         )
         metal2mark = _prm.Marker(name="metal2mark")
         metal2res = _prm.Resistor(
-            name="metal2res", wire=metal2,
+            name="metal2res", wire=metal2, min_length=0.5,
             indicator=metal2mark, min_indicator_extension=0.4,
-            contact=None, sheetres=10,
+            contact=None,
         )
         via = _prm.Via(
             name="via", width=0.35, min_space=0.35,
@@ -99,28 +99,31 @@ class DummyTech(_tch.Technology):
         prims += (via, metal2pin, metal2block, metal2, metal2mark, metal2res)
 
         silblock = _prm.ExtraProcess(
-            name="silblock", min_width=0.4, min_space=0.4, fill_space="no",
+            name="silblock", min_width=0.4, min_space=0.4, grid=0.010,
         )
         resistor = _prm.Resistor(
             name="resistor", wire=active, implant=nplus, contact=contact,
             min_contact_space=0.2,
             indicator=silblock, min_indicator_extension=0.4,
-            model="resistor", subckt_params={"width": "w", "height": "l"},
         )
-        prims += (silblock, resistor)
+        polyres = _prm.Resistor(
+            name="polyres", wire=poly, implant=pplus, contact=contact,
+            min_implant_enclosure=_prp.Enclosure(0.3), min_contact_space=0.2,
+            indicator=silblock, min_indicator_extension=0.4,
+        )
+        prims += (silblock, resistor, polyres)
 
         diodemark = _prm.Marker(name="diodemark")
         ndiode = _prm.Diode(
             name="ndiode", wire=active,
             indicator=diodemark, min_indicator_enclosure=_prp.Enclosure(0.2),
             implant=nplus, min_implant_enclosure=_prp.Enclosure(0.2),
-            model="ndiode",
         )
         pdiode = _prm.Diode(
             name="pdiode", wire=active,
             indicator=diodemark, min_indicator_enclosure=_prp.Enclosure(0.2),
             implant=pplus, min_implant_enclosure=_prp.Enclosure(0.2),
-            well=nwell, model="pdiode", 
+            well=nwell, min_well_enclosure=_prp.Enclosure(2.0),
         )
         prims += (diodemark, ndiode, pdiode)
 
@@ -138,7 +141,9 @@ class DummyTech(_tch.Technology):
             min_l=0.5, min_w=0.5,
         )
         esdgate = _prm.MOSFETGate(
-            name="esdgate", active=active, oxide=hvox, inside=esd,
+            name="esdgate", active=active,
+            oxide=hvox, min_gateoxide_enclosure=_prp.Enclosure(0.4),
+            inside=esd, min_gateinside_enclosure=_prp.Enclosure(0.4),
             min_sd_width=0.35,
             poly=poly, min_polyactive_extension=0.45,
             min_l=0.5, min_w=0.5,
@@ -171,17 +176,27 @@ class DummyTech(_tch.Technology):
         prims += (mosgate, hvgate, esdgate, nmos, pmos, esd, hvnmos, hvpmos, esdnmos)
 
         bip = _prm.Marker(name="bip")
-        npn = _prm.Bipolar(name="npn", type_="npn", indicator=bip)
-        prims += (bip, npn)
+        npn = _prm.Bipolar(name="npn", type_=_prm.npnBipolarT, indicator=bip)
+        pnp = _prm.Bipolar(
+            name="pnp", type_=_prm.pnpBipolarT, indicator=bip,
+        )
+        prims += (bip, npn, pnp)
 
         mimcap = _prm.MIMCapacitor(
             name="MIMCap", bottom=metal, top=mimtop, via=via,
             min_bottom_top_enclosure=_prp.Enclosure(0.2),
             min_bottomvia_top_space=0.25,
             min_top_via_enclosure=_prp.Enclosure(0.25),
-            min_bottom_space=None, min_top2bottom_space=None, model="MIMCap",
+            min_bottom_space=None, min_top2bottom_space=None,
         )
-        prims += mimcap
+        mimcap2 = _prm.MIMCapacitor(
+            name="MIMCap2", bottom=metal, top=mimtop, via=via,
+            min_bottom_top_enclosure=_prp.Enclosure(0.2),
+            min_bottomvia_top_space=0.25,
+            min_top_via_enclosure=_prp.Enclosure(0.25),
+            min_bottom_space=None, min_top2bottom_space=None,
+        )
+        prims += (mimcap, mimcap2)
 
         aux = _prm.Auxiliary(name="anything_goes")
         prims += aux
@@ -193,7 +208,12 @@ class DummyTech(_tch.Technology):
             _prm.Spacing(
                 primitives1=(nwell, pplus), min_space=2.0,
             ),
+            _prm.MinWidth(prim=active.in_(hvox), min_width=0.5),
+            _prm.Enclosure(prim=active, by=esd, min_enclosure=_prp.Enclosure(0.1)),
+            _prm.NoOverlap(prim1=nplus, prim2=pplus),
         )
+
+        super().__init__(primitives=prims)
 dummy_tech = DummyTech()
 dummy_cktfab = _ckt.CircuitFactory(tech=dummy_tech)
 dummy_layoutfab = _lay.LayoutFactory(tech=dummy_tech)
@@ -203,10 +223,45 @@ dummy_gdslayers: GDSLayerSpecDict = {
     for i, mask in enumerate(dummy_tech.designmasks)
 }
 
+def _gen_spiceparams() -> SpicePrimsParamSpec:
+    prims = dummy_tech.primitives
 
-dummy_lib = _lbry.Library(
-    name="dummy_lib", tech=dummy_tech, cktfab=dummy_cktfab, layoutfab=dummy_layoutfab,
-)
+    params = SpicePrimsParamSpec()
+    params.add_device_params(
+        prim=cast(_prm.Resistor, prims.metal2res),
+        model="mres2", sheetres=10.0,
+    )
+    params.add_device_params(
+        prim=cast(_prm.Resistor, prims.resistor), is_subcircuit=True,
+    )
+    params.add_device_params(
+        prim=cast(_prm.Resistor, prims.polyres), is_subcircuit=True,
+        model="polyres", subcircuit_paramalias={"width": "w", "length": "l"},
+    )
+
+    params.add_device_params(prim=cast(_prm.Diode, prims.ndiode))
+    params.add_device_params(prim=cast(_prm.Diode, prims.pdiode), is_subcircuit=True)
+
+    params.add_device_params(prim=cast(_prm.MOSFET, prims.nmos))
+    params.add_device_params(prim=cast(_prm.MOSFET, prims.pmos), is_subcircuit=True)
+    params.add_device_params(prim=cast(_prm.MOSFET, prims.hvnmos))
+    params.add_device_params(prim=cast(_prm.MOSFET, prims.hvpmos))
+    params.add_device_params(prim=cast(_prm.MOSFET, prims.esdnmos), model="nmos_5v_esd")
+
+    params.add_device_params(prim=cast(_prm.Bipolar, prims.npn))
+    params.add_device_params(prim=cast(_prm.Bipolar, prims.pnp), is_subcircuit=True)
+
+    params.add_device_params(prim=cast(_prm.MIMCapacitor, prims.MIMCap))
+    params.add_device_params(
+        prim=cast(_prm.MIMCapacitor, prims.MIMCap2),
+        model="cap2", subcircuit_paramalias={"width": "w", "height": "h"},
+    )
+
+    return params
+dummy_prims_spiceparams = _gen_spiceparams()
+
+dummy_lib = _lbry.Library(name="dummy_lib", tech=dummy_tech)
+dummy_fab = _fab.BaseCellFactory(lib=dummy_lib, cktfab=dummy_cktfab, layoutfab=dummy_layoutfab)
 def _lib_init():
     # The dummy_lib will be initialized with different shapes to maximize
     # code coverage when being used for example in export code etc.
@@ -221,7 +276,7 @@ def _lib_init():
     ))
 
     # cell1: rect shape
-    cell1 = dummy_lib.new_cell(name="cell1")
+    cell1 = dummy_fab.new_cell(name="cell1")
     ckt = cell1.new_circuit()
     layouter = cell1.new_circuitlayouter()
     layout = layouter.layout
@@ -231,16 +286,18 @@ def _lib_init():
     layouter.layout.boundary = rect1
 
     # cell2: cell instance
-    cell2 = dummy_lib.new_cell(name="cell2")
+    cell2 = dummy_fab.new_cell(name="cell2")
     ckt = cell2.new_circuit()
     layouter = cell2.new_circuitlayouter()
 
     inst = ckt.instantiate(cell1, name="inst")
-    layouter.place(object_=inst, origin=_geo.origin)
+    i = ckt.new_net(name="i", external=True, childports=inst.ports.i)
+
+    layouter.place(object_=inst, origin=_geo.Point(x=1.0, y=0.0))
     layouter.layout.boundary = rect1
 
     # cell3: polygon
-    cell3 = dummy_lib.new_cell(name="cell3")
+    cell3 = dummy_fab.new_cell(name="cell3")
     ckt = cell3.new_circuit()
     layouter = cell3.new_circuitlayouter()
 
@@ -249,15 +306,32 @@ def _lib_init():
     layouter.layout.boundary = lshape.bounds
 
     # cell4: multipartshape
-    cell4 = dummy_lib.new_cell(name="cell4")
+    cell4 = dummy_fab.new_cell(name="cell4")
     ckt = cell4.new_circuit()
     layouter = cell4.new_circuitlayouter()
+    layout = layouter.layout
 
     mps = _geo.MultiPartShape(fullshape=rect12, parts=(rect1, rect2))
     
     i1 = ckt.new_net(name="i1", external=False)
     i2 = ckt.new_net(name="i2", external=False)
 
-    layout.add_shape(prim=metal, net=i1, shape=mps.parts[0])
-    layout.add_shape(prim=metal, net=i2, shape=mps.parts[1])
+    layout.add_shape(layer=metal, net=i1, shape=mps.parts[0])
+    layout.add_shape(layer=metal, net=i2, shape=mps.parts[1])
+
+    # cell5: primitive instance
+    cell5 = dummy_fab.new_cell(name="cell5")
+    ckt = cell5.new_circuit()
+    layouter = cell5.new_circuitlayouter()
+    layout = layouter.layout
+
+    res = ckt.instantiate(
+        cast(_prm.Resistor, prims.resistor), name="res",
+        width=1.0, height=5.0,
+    )
+
+    ckt.new_net(name="p1", external=True, childports=res.ports.port1)
+    ckt.new_net(name="p2", external=True, childports=res.ports.port2)
+
+    layouter.place(res)
 _lib_init()
