@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 from textwrap import dedent
 import unittest
@@ -9,7 +9,10 @@ from pdkmaster.technology import (
 )
 from pdkmaster.io import klayout as _kl
 
-from ...dummy import dummy_tech, dummy_gdslayers, dummy_lib, empty_tech
+from ...dummy import (
+    dummy_tech, dummy_gdslayers, dummy_prims_spiceparams, dummy_lib,
+    empty_tech,
+)
 
 import pya
 
@@ -21,9 +24,9 @@ class FileExportTest(unittest.TestCase):
 
         mc = _MaskConverter(tech=dummy_tech)
 
-        mask = _msk.DesignMask(name="layer0", fill_space="no")
+        mask = _msk.DesignMask(name="layer0")
         alias = mask.alias("0layer:s1.s2")
-        mask2 = _msk.DesignMask(name="layer1", fill_space="no")
+        mask2 = _msk.DesignMask(name="layer1")
 
         self.assertEqual(mc(mask), "layer0")
         self.assertEqual(mc(alias), "_0layer__s1_s2")
@@ -46,9 +49,9 @@ class FileExportTest(unittest.TestCase):
 
         ec = export._edge_conv
 
-        mask = _msk.DesignMask(name="mask", fill_space="no")
+        mask = _msk.DesignMask(name="mask")
         edge = _edg.MaskEdge(mask)
-        mask2 = _msk.DesignMask(name="mask2", fill_space="no")
+        mask2 = _msk.DesignMask(name="mask2")
         edge2 = _edg.MaskEdge(mask2)
 
         interact1 = edge.interact_with(mask2)
@@ -68,7 +71,7 @@ class FileExportTest(unittest.TestCase):
         from pdkmaster.io.klayout import export
         export._mask_conv = export._MaskConverter(tech=dummy_tech)
 
-        mask = _msk.DesignMask(name="mask", fill_space="no")
+        mask = _msk.DesignMask(name="mask")
         alias = mask.alias("alias")
         self.assertEqual(export._str_alias(alias), "alias = mask\n")
 
@@ -81,10 +84,10 @@ class FileExportTest(unittest.TestCase):
 
         rc = export._rule_conv
 
-        mask = _msk.DesignMask(name="mask", fill_space="no")
+        mask = _msk.DesignMask(name="mask")
         alias = mask.alias("alias")
         edge = _edg.MaskEdge(mask)
-        mask2 = _msk.DesignMask(name="mask2", fill_space="no")
+        mask2 = _msk.DesignMask(name="mask2")
 
         self.assertEqual(
             rc(mask.area >= 0.1),
@@ -142,16 +145,6 @@ class FileExportTest(unittest.TestCase):
                 )
             """[1:]),
         )
-        self.assertEqual(
-            rc(_msk.OverlapWidth(mask, mask2) >= 0.5),
-            dedent("""
-                # overlapwidth(mask,mask2) >= 0.5
-                (mask&mask2).width(0.5).output(
-                    "mask:mask2 overlap width",
-                    "Minimum overlap widht of mask and mask2: 0.5µm"
-                )
-            """[1:]),
-        )
         self.assertEqual(
             rc(mask.extend_over(mask2) >= 0.1),
             dedent("""
@@ -316,7 +309,10 @@ class FileExportTest(unittest.TestCase):
         # We just generate the code to get the code coverage.
         # Correctness of out is assumed to be checked with lower level unit tests
         # on the Exporter classes and higher level test using gds files etc.
-        _kl.FileExporter(tech=dummy_tech, gds_layers=dummy_gdslayers)()
+        _kl.FileExporter(
+            tech=dummy_tech, gds_layers=dummy_gdslayers,
+            prims_spiceparams=dummy_prims_spiceparams,
+        )()
 
 
 class Export2DBTest(unittest.TestCase):
@@ -339,7 +335,7 @@ class Export2DBTest(unittest.TestCase):
     def test_maskshapes(self):
         # This mainly runs code to get coverage but does not extensive test of correctness
         # of results. This is assumed to be done with lower level unit tests.
-        mask = _util.first(dummy_tech.designmasks)
+        mask = _util.get_first_of(dummy_tech.designmasks)
 
         rect1 = _geo.Rect(left=0.0, bottom=0.0, right=1.0, top=1.0)
         rect2 = _geo.Rect(left=2.0, bottom=0.0, right=3.0, top=1.0)
