@@ -1,10 +1,11 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 import unittest
 
 from pdkmaster import _util
 from pdkmaster.technology import geometry as _geo
 from pdkmaster.design import layout as _lay
+from pdkmaster.design.layout import layout_ as _laylay
 from pdkmaster.io import klayout as _kl
 from pdkmaster.io.klayout import merge_ as _klmrg
 
@@ -360,7 +361,7 @@ class MergeTest(unittest.TestCase):
         self.assertEqual(shape, polygon2)
 
     def test_maskshape(self):
-        mask = _util.first(dummy_tech.designmasks)
+        mask = _util.get_first_of(dummy_tech.designmasks)
 
         rect1 = _geo.Rect(left=0.0, bottom=0.0, right=1.0, top=1.0)
         poly2 = _geo.Polygon.from_floats(points=(
@@ -379,7 +380,7 @@ class MergeTest(unittest.TestCase):
         self.assertEqual(_kl.merge(ms2), ms2)
 
     def test_maskshapes(self):
-        mask = _util.first(dummy_tech.designmasks)
+        mask = _util.get_first_of(dummy_tech.designmasks)
 
         rect1 = _geo.Rect(left=0.0, bottom=0.0, right=1.0, top=1.0)
         rect2 = _geo.Rect(left=2.0, bottom=0.0, right=3.0, top=1.0)
@@ -396,7 +397,7 @@ class MergeTest(unittest.TestCase):
         net1 = MyNet(name="net1")
         rect1 = _geo.Rect(left=0.0, bottom=0.0, right=1.0, top=1.0)
         layout1 = fab.new_layout()
-        layout1.add_shape(prim=metal, net=net1, shape=rect1)
+        layout1.add_shape(layer=metal, net=net1, shape=rect1)
 
         layout1_merged = _kl.merge(layout1)
         self.assertEqual(layout1_merged, layout1)
@@ -430,21 +431,21 @@ class MergeTest(unittest.TestCase):
         net2 = ckt.new_net(name="net2", external=False)
         net3 = ckt.new_net(name="net3", external=False)
 
-        layout.add_shape(prim=active, net=net1, shape=r1)
-        layout.add_shape(prim=active, net=net1, shape=mps234.parts[0])
-        layout.add_shape(prim=active, net=None, shape=mps234.parts[1])
-        layout.add_shape(prim=active, net=net2, shape=mps234.parts[2])
-        layout.add_shape(prim=active, net=net2, shape=mps567.parts[0])
-        layout.add_shape(prim=active, net=None, shape=mps567.parts[1])
-        layout.add_shape(prim=active, net=net3, shape=mps567.parts[2])
-        layout.add_shape(prim=active, net=net3, shape=r8)
+        layout.add_shape(layer=active, net=net1, shape=r1)
+        layout.add_shape(layer=active, net=net1, shape=mps234.parts[0])
+        layout.add_shape(layer=active, net=None, shape=mps234.parts[1])
+        layout.add_shape(layer=active, net=net2, shape=mps234.parts[2])
+        layout.add_shape(layer=active, net=net2, shape=mps567.parts[0])
+        layout.add_shape(layer=active, net=None, shape=mps567.parts[1])
+        layout.add_shape(layer=active, net=net3, shape=mps567.parts[2])
+        layout.add_shape(layer=active, net=net3, shape=r8)
 
         # print("test_layout_multishapepart:merge()")
         lay_m = _kl.merge(layouter.layout)
         # print("done")
 
-        for sl in lay_m.sublayouts:
-            self.assertIsInstance(sl, _lay.MaskShapesSubLayout)
+        for sl in lay_m._sublayouts:
+            self.assertIsInstance(sl, _laylay._MaskShapesSubLayout)
             self.assertEqual(len(sl.shapes), 1)
             ms = sl.shapes[0]
             self.assertEqual(ms.mask, active.mask)
