@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 import unittest
 
@@ -6,15 +6,15 @@ from pdkmaster.technology import mask as _msk, edge as _edg
 
 class GeometryTest(unittest.TestCase):
     def test_error(self):
-        mask1 = _msk.DesignMask(name="mask1", fill_space="yes")
-        mask2 = _msk.DesignMask(name="mask2", fill_space="no")
+        mask1 = _msk.DesignMask(name="mask1")
+        mask2 = _msk.DesignMask(name="mask2")
 
         with self.assertRaises(TypeError):
             _edg.Intersect((mask1, mask2))
 
     def test_edgeproperty(self):
-        mask1 = _msk.DesignMask(name="mask1", fill_space="yes")
-        mask2 = _msk.DesignMask(name="mask2", fill_space="no")
+        mask1 = _msk.DesignMask(name="mask1")
+        mask2 = _msk.DesignMask(name="mask2")
 
         edge1 = _edg.MaskEdge(mask1)
         edge2 = _edg.MaskEdge(mask2)
@@ -30,8 +30,8 @@ class GeometryTest(unittest.TestCase):
         )
 
     def test_maskedge(self):
-        mask1 = _msk.DesignMask(name="mask1", fill_space="yes")
-        mask2 = _msk.DesignMask(name="mask2", fill_space="no")
+        mask1 = _msk.DesignMask(name="mask1")
+        mask2 = _msk.DesignMask(name="mask2")
 
         # Just do coverage of code
         edge1 = _edg.MaskEdge(mask1)
