@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
+# SPDX-License-Identifier: AGPL-3.0-or-later OR GPL-2.0-or-later OR CERN-OHL-S-2.0+ OR Apache-2.0
 # type: ignore
 import unittest
 
@@ -15,8 +15,8 @@ class WaferTest(unittest.TestCase):
         self.assertEqual(_wfr.wafer.designmasks, tuple())
 
     def test_outside(self):
-        mask1 = _msk.DesignMask(name="mask1", fill_space="yes")
-        mask2 = _msk.DesignMask(name="mask2", fill_space="no")
+        mask1 = _msk.DesignMask(name="mask1")
+        mask2 = _msk.DesignMask(name="mask2")
         masks = (mask1, mask2)
         self.assertEqual(_wfr.outside(mask1), _wfr.wafer.remove(mask1))
         self.assertEqual(
